@@ -7,27 +7,25 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  * 
- * $Id: actions.c 4969 2008-02-07 20:27:15Z charles $
+ * $Id: actions.c 6489 2008-08-11 19:05:00Z charles $
  */
 
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <libtransmission/transmission.h>
+#include "actions.h"
 #include "conf.h"
-#include "torrent-inspector.h"
-#include "tr_core.h"
-#include "tr_prefs.h"
+#include "tr-core.h"
+#include "tr-prefs.h"
 #include "lock.h"
 #include "logo.h"
 
 #define UNUSED G_GNUC_UNUSED
 
-extern void doAction (const char * action_name, gpointer user_data );
+static TrCore * myCore = NULL;
 
-static TrCore * myCore = 0;
-
-static GtkActionGroup * myGroup = 0;
+static GtkActionGroup * myGroup = NULL;
 
 static void
 action_cb ( GtkAction * a, gpointer user_data )
@@ -43,30 +41,15 @@ action_cb ( GtkAction * a, gpointer user_data )
 #define GTK_STOCK_SELECT_ALL NULL
 #endif
 
-static GtkRadioActionEntry priority_toggle_entries[] =
-{
-  { "priority-high", NULL, N_("_High"), NULL, NULL, TR_PRI_HIGH },
-  { "priority-normal", NULL, N_("_Normal"), NULL, NULL, TR_PRI_NORMAL },
-  { "priority-low", NULL, N_("_Low"), NULL, NULL, TR_PRI_LOW }
-};
-
-extern void set_selected_file_priority ( tr_priority_t ); 
-
-static void
-priority_changed_cb (GtkAction *action UNUSED, GtkRadioAction *current)
-{
-  const int priority = gtk_radio_action_get_current_value (current);
-  set_selected_file_priority ( priority );
-}
-
 static GtkRadioActionEntry sort_radio_entries[] =
 {
   { "sort-by-activity",   NULL, N_("Sort by _Activity"),   NULL, NULL, 0 },
-  { "sort-by-date-added", NULL, N_("Sort by _Date Added"), NULL, NULL, 1 },
-  { "sort-by-name",       NULL, N_("Sort by _Name"),       NULL, NULL, 2 },
-  { "sort-by-progress",   NULL, N_("Sort by _Progress"),   NULL, NULL, 3 },
+  { "sort-by-name",       NULL, N_("Sort by _Name"),       NULL, NULL, 1 },
+  { "sort-by-progress",   NULL, N_("Sort by _Progress"),   NULL, NULL, 2 },
+  { "sort-by-ratio",      NULL, N_("Sort by _Ratio"),      NULL, NULL, 3 },
   { "sort-by-state",      NULL, N_("Sort by _State"),      NULL, NULL, 4 },
-  { "sort-by-tracker",    NULL, N_("Sort by _Tracker"),    NULL, NULL, 5 }
+  { "sort-by-tracker",    NULL, N_("Sort by _Tracker"),    NULL, NULL, 5 },
+  { "sort-by-age",        NULL, N_("Sort by A_ge"),        NULL, NULL, 6 }
 };
 
 static void
@@ -83,9 +66,9 @@ sort_changed_cb( GtkAction            * action UNUSED,
 static GtkToggleActionEntry show_toggle_entries[] = 
 {
   { "toggle-main-window", NULL, 
-    N_("Show _Main Window"), NULL, NULL, G_CALLBACK(action_cb), TRUE }, 
+    N_("_Main Window"), NULL, NULL, G_CALLBACK(action_cb), TRUE }, 
   { "toggle-message-log", NULL,
-    N_("Show Message _Log"), NULL, NULL, G_CALLBACK(action_cb), FALSE }
+    N_("Message _Log"), NULL, NULL, G_CALLBACK(action_cb), FALSE }
 };
 
 static void
@@ -102,12 +85,14 @@ static GtkToggleActionEntry pref_toggle_entries[] =
     N_("_Minimal View"), "<alt>M", NULL, G_CALLBACK(toggle_pref_cb), FALSE },
   { "sort-reversed", NULL,
     N_("_Reverse Sort Order"), NULL, NULL, G_CALLBACK(toggle_pref_cb), FALSE },
-  { "show-filter-bar", NULL,
-    N_("Show _Filter Bar"), NULL, NULL, G_CALLBACK(toggle_pref_cb), FALSE },
-  { "show-status-bar", NULL,
-    N_("Show _Status Bar"), NULL, NULL, G_CALLBACK(toggle_pref_cb), FALSE },
+  { "show-filterbar", NULL,
+    N_("_Filterbar"), NULL, NULL, G_CALLBACK(toggle_pref_cb), FALSE },
+  { "show-statusbar", NULL,
+    N_("_Statusbar"), NULL, NULL, G_CALLBACK(toggle_pref_cb), FALSE },
   { "show-toolbar", NULL,
-    N_("Show _Toolbar"), NULL, NULL, G_CALLBACK(toggle_pref_cb), FALSE }
+    N_("_Toolbar"), NULL, NULL, G_CALLBACK(toggle_pref_cb), FALSE },
+  { PREF_KEY_SHOW_TRAY_ICON, NULL,
+    N_("Tray _Icon" ), NULL, NULL, G_CALLBACK(toggle_pref_cb), FALSE }
 };
 
 static GtkActionEntry entries[] =
@@ -117,53 +102,36 @@ static GtkActionEntry entries[] =
   { "sort-menu", NULL, N_("_Sort Torrents By"), NULL, NULL, NULL },
   { "edit-menu", NULL, N_("_Edit"), NULL, NULL, NULL },
   { "help-menu", NULL, N_("_Help"), NULL, NULL, NULL },
-  { "priority-menu", NULL, N_("_Priority"), NULL, NULL, NULL },
-  { "add-torrent", GTK_STOCK_OPEN,
-    NULL, NULL, N_("Open Torrent"), G_CALLBACK(action_cb) },
+  { "add-torrent-toolbar", GTK_STOCK_ADD, NULL, NULL, N_("Add a torrent"), G_CALLBACK(action_cb) },
+  { "add-torrent-menu", GTK_STOCK_ADD, N_("_Add..."), "<control>D", N_("Add a torrent"), G_CALLBACK(action_cb) },
   { "start-torrent", GTK_STOCK_MEDIA_PLAY,
-    N_("_Start"), "<control>S", NULL, G_CALLBACK(action_cb) },
+    N_("_Start"), "<control>S", N_("Start torrent"), G_CALLBACK(action_cb) },
   { "show-stats", NULL, N_("_Statistics"), NULL, NULL, G_CALLBACK(action_cb) },
   { "verify-torrent", NULL,
     N_("_Verify Local Data"), NULL, NULL, G_CALLBACK(action_cb) },
   { "pause-torrent", GTK_STOCK_MEDIA_PAUSE,
-    N_("_Pause"), "<control>P", NULL, G_CALLBACK(action_cb) },
-  { "remove-torrent", GTK_STOCK_REMOVE,
-    N_("_Remove"), "<control>Delete", NULL, G_CALLBACK(action_cb) },
-  { "create-torrent", GTK_STOCK_NEW,
-    N_("Create _New Torrent"), NULL, NULL, G_CALLBACK(action_cb) },
-  { "close", GTK_STOCK_CLOSE,
-    N_("_Close"), "<control>W", NULL, G_CALLBACK(action_cb) },
-  { "quit", GTK_STOCK_QUIT,
-    N_("_Quit"), "<control>Q", NULL, G_CALLBACK(action_cb) },
+    N_("_Pause"), "<control>P", N_("Pause torrent"), G_CALLBACK(action_cb) },
+  { "remove-torrent", GTK_STOCK_REMOVE, NULL, "Delete", N_("Remove torrent"), G_CALLBACK(action_cb) },
+  { "delete-torrent", GTK_STOCK_DELETE, N_("_Delete Files and Remove"), "<shift>Delete", NULL, G_CALLBACK(action_cb) },
+  { "new-torrent", GTK_STOCK_NEW, N_("_New..."), NULL,
+    N_("Create a torrent"),
+    G_CALLBACK(action_cb) },
+  { "quit", GTK_STOCK_QUIT, N_("_Quit"), NULL, NULL, G_CALLBACK(action_cb) },
   { "select-all", GTK_STOCK_SELECT_ALL,
-    N_("Select _All"), "<control>A", NULL, G_CALLBACK(action_cb) },
-  { "unselect-all", NULL,
+    N_( "Select _All" ), "<control>A", NULL, G_CALLBACK(action_cb) },
+  { "deselect-all", NULL,
     N_("Dese_lect All"), "<shift><control>A", NULL, G_CALLBACK(action_cb) },
-  { "edit-preferences", GTK_STOCK_PREFERENCES,
-    NULL, NULL, NULL, G_CALLBACK(action_cb) },
+  { "edit-preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL, G_CALLBACK(action_cb) },
   { "show-torrent-details", GTK_STOCK_INFO,
-    N_("_Details"), "<alt>Return", N_("Torrent Details"), G_CALLBACK(action_cb) },
-  { "show-about-dialog", GTK_STOCK_ABOUT,
-    N_("_About Transmission"), NULL, NULL, G_CALLBACK(action_cb) },
+    N_("_Details"), "<alt>Return", N_("Torrent details"), G_CALLBACK(action_cb) },
+  { "open-torrent-folder", GTK_STOCK_OPEN,
+    N_("_Open Folder"), NULL, NULL, G_CALLBACK(action_cb) },
+  { "show-about-dialog", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK(action_cb) },
+  { "help", GTK_STOCK_HELP, N_("_Contents"), "F1", NULL, G_CALLBACK(action_cb) },
   { "update-tracker", GTK_STOCK_NETWORK,
     N_("Ask Tracker for _More Peers"), NULL, NULL, G_CALLBACK(action_cb) }
 };
 
-static void
-ensure_tooltip (GtkActionEntry * e)
-{
-    if( !e->tooltip && e->label )
-    {
-        const char * src;
-        char *tgt;
-        e->tooltip = g_malloc( strlen( e->label ) + 1 );
-        for( src=e->label, tgt=(char*)e->tooltip; *src; ++src )
-            if( *src != '_' )
-                *tgt++ = *src;
-        *tgt++ = '\0';
-    }
-}
-
 typedef struct
 {
    const guint8* raw;
@@ -171,7 +139,7 @@ typedef struct
 }
 BuiltinIconInfo;
 
-const BuiltinIconInfo my_fallback_icons [] =
+static const BuiltinIconInfo my_fallback_icons [] =
 {
     { tr_icon_logo, "transmission" },
     { tr_icon_lock, "transmission-lock" }
@@ -196,7 +164,7 @@ register_my_icons ( void )
             GdkPixbuf * p;
             GtkIconSet * icon_set;
 
-            p = gdk_pixbuf_new_from_inline( -1, my_fallback_icons[i].raw, FALSE, 0 );
+            p = gdk_pixbuf_new_from_inline( -1, my_fallback_icons[i].raw, FALSE, NULL );
             width = gdk_pixbuf_get_width( p );
             icon_set = gtk_icon_set_new_from_pixbuf( p );
             gtk_icon_theme_add_builtin_icon( name, width, p );
@@ -223,26 +191,17 @@ actions_init( GtkUIManager * ui_manager, gpointer callback_user_data )
 {
   int i, n;
   int active;
-  char * match;
+  const char * match;
   const int n_entries = G_N_ELEMENTS( entries );
   GtkActionGroup * action_group;
 
   myUIManager = ui_manager;
 
-  register_my_icons ();
-
-  for( i=0; i<n_entries; ++i )
-    ensure_tooltip (&entries[i]);
+  register_my_icons( );
 
   action_group = myGroup = gtk_action_group_new( "Actions" );
   gtk_action_group_set_translation_domain( action_group, NULL );
 
-  gtk_action_group_add_radio_actions( action_group,
-                                      priority_toggle_entries,
-                                      G_N_ELEMENTS(priority_toggle_entries),
-                                      TR_PRI_NORMAL,
-                                      G_CALLBACK(priority_changed_cb), NULL);
-
 
   match = pref_string_get( PREF_KEY_SORT_MODE );
   for( i=0, n=G_N_ELEMENTS(sort_radio_entries), active=-1; active==-1 && i<n; ++i )
@@ -276,7 +235,6 @@ actions_init( GtkUIManager * ui_manager, gpointer callback_user_data )
 
   gtk_ui_manager_insert_action_group( ui_manager, action_group, 0 );
   g_object_unref (G_OBJECT(action_group));
-  g_free( match );
 }
 
 /****
