@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: conf.c 5753 2008-05-06 11:04:16Z charles $
+ * $Id: conf.c 6166 2008-06-13 00:52:55Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -51,9 +51,9 @@ cf_init(const char *dir, char **errstr)
     if( errstr != NULL )
         *errstr = NULL;
 
-    gl_confdir = g_build_filename( dir, "gtk", NULL );
+    gl_confdir = g_strdup( dir );
 
-    if( mkdir_p(gl_confdir, 0755 ) )
+    if( mkdir_p( gl_confdir, 0755 ) )
         return TRUE;
 
     if( errstr != NULL )
@@ -122,64 +122,76 @@ cf_lock( char ** errstr )
     return didLock;
 }
 
-char*
-cf_sockname( void )
-{
-    assert( gl_confdir != NULL );
-    return g_build_filename( gl_confdir, "socket", NULL );
-}
-
 /***
 ****
 ****  Preferences
 ****
 ***/
 
-#define GROUP "general"
-
 static char*
 getPrefsFilename( void )
 {
     assert( gl_confdir != NULL );
-    return g_build_filename( gl_confdir, "prefs.ini", NULL );
+    return g_build_filename( gl_confdir, "settings.json", NULL );
 }
 
-static GKeyFile*
-getPrefsKeyFile( void )
+static tr_benc*
+getPrefs( void )
 {
-    static GKeyFile * myKeyFile = NULL;
+    static tr_benc dict;
+    static gboolean loaded = FALSE;
 
-    if( myKeyFile == NULL )
+    if( !loaded )
     {
         char * filename = getPrefsFilename( );
-        myKeyFile = g_key_file_new( );
-        g_key_file_load_from_file( myKeyFile, filename, 0, NULL );
+        if( tr_bencLoadJSONFile( filename, &dict ) )
+            tr_bencInitDict( &dict, 100 );
         g_free( filename );
+        loaded = TRUE;
     }
 
-    return myKeyFile;
+    return &dict;
 }
 
+/***
+****
+***/
+
 int
-pref_int_get( const char * key ) {
-    return g_key_file_get_integer( getPrefsKeyFile( ), GROUP, key, NULL );
+pref_int_get( const char * key )
+{
+    int64_t i;
+    tr_bencDictFindInt( getPrefs( ), key, &i );
+    return i;
 }
 void
-pref_int_set( const char * key, int value ) {
-    g_key_file_set_integer( getPrefsKeyFile( ), GROUP, key, value );
+pref_int_set( const char * key, int value )
+{
+    tr_benc * d = getPrefs( );
+    tr_bencDictRemove( d, key );
+    tr_bencDictAddInt( d, key, value );
 }
 void
-pref_int_set_default( const char * key, int value ) {
-    if( !g_key_file_has_key( getPrefsKeyFile( ), GROUP, key, NULL ) )
+pref_int_set_default( const char * key, int value )
+{
+    if( !tr_bencDictFind( getPrefs( ), key ) )
         pref_int_set( key, value );
 }
 
+/***
+****
+***/
+
 gboolean
-pref_flag_get ( const char * key ) {
-    return g_key_file_get_boolean( getPrefsKeyFile( ), GROUP, key, NULL );
+pref_flag_get ( const char * key )
+{
+    int64_t i;
+    tr_bencDictFindInt( getPrefs( ), key, &i );
+    return i != 0;
 }
 gboolean
-pref_flag_eval( pref_flag_t val, const char * key ) {
+pref_flag_eval( pref_flag_t val, const char * key )
+{
     switch( val ) {
         case PREF_FLAG_TRUE: return TRUE;
         case PREF_FLAG_FALSE: return FALSE;
@@ -187,54 +199,56 @@ pref_flag_eval( pref_flag_t val, const char * key ) {
     }
 }
 void
-pref_flag_set( const char * key, gboolean value ) {
-    g_key_file_set_boolean( getPrefsKeyFile( ), GROUP, key, value );
+pref_flag_set( const char * key, gboolean value )
+{
+    pref_int_set( key, value!=0 );
 }
 void
-pref_flag_set_default( const char * key, gboolean value ) {
-    if( !g_key_file_has_key( getPrefsKeyFile( ), GROUP, key, NULL ) )
-        pref_flag_set( key, value );
+pref_flag_set_default( const char * key, gboolean value )
+{
+    pref_int_set_default( key, value!=0 );
 }
 
-char*
-pref_string_get( const char * key ) {
-    return g_key_file_get_string( getPrefsKeyFile( ), GROUP, key, NULL );
+/***
+****
+***/
+
+const char*
+pref_string_get( const char * key )
+{
+    const char * str = NULL;
+    tr_bencDictFindStr( getPrefs( ), key, &str );
+    return str;
 }
+
 void
-pref_string_set( const char * key, const char * value ) {
-    g_key_file_set_string( getPrefsKeyFile( ), GROUP, key, value );
+pref_string_set( const char * key, const char * value )
+{
+    tr_benc * d = getPrefs( );
+    tr_bencDictRemove( d, key );
+    tr_bencDictAddStr( d, key, value );
 }
+
 void
-pref_string_set_default( const char * key, const char * value ) {
-    if( !g_key_file_has_key( getPrefsKeyFile( ), GROUP, key, NULL ) )
+pref_string_set_default( const char * key, const char * value )
+{
+    if( !tr_bencDictFind( getPrefs( ), key ) )
         pref_string_set( key, value );
 }
 
+/***
+****
+***/
+
 void
-pref_save(char **errstr)
+pref_save( void )
 {
-    gsize datalen;
-    GError * err = NULL;
-    char * data;
-    char * filename;
-    char * path;
+    char * filename = getPrefsFilename( );
+    char * path = g_path_get_dirname( filename );
 
-    filename = getPrefsFilename( );
-    path = g_path_get_dirname( filename );
     mkdir_p( path, 0755 );
+    tr_bencSaveJSONFile( filename, getPrefs( ) );
 
-    data = g_key_file_to_data( getPrefsKeyFile(), &datalen, &err );
-    if( !err ) {
-        GIOChannel * out = g_io_channel_new_file( filename, "w+", &err );
-        g_io_channel_write_chars( out, data, datalen, NULL, &err );
-        g_io_channel_unref( out );
-    }
-
-    if( errstr != NULL )
-        *errstr = err ? g_strdup( err->message ) : NULL;
-
-    g_clear_error( &err );
-    g_free( data );
     g_free( path );
     g_free( filename );
 }
@@ -257,19 +271,25 @@ tr_file_set_contents( const char * filename, const void * out, size_t len, GErro
 #endif
 
 static char*
-getCompat08PrefsFilename( void )
+getCompat080PrefsFilename( void )
 {
     assert( gl_confdir != NULL );
     return g_build_filename( g_get_home_dir(), ".transmission", "gtk", "prefs", NULL );
 }
 
 static char*
-getCompat09PrefsFilename( void )
+getCompat090PrefsFilename( void )
 {
     assert( gl_confdir != NULL );
     return g_build_filename( g_get_home_dir(), ".transmission", "gtk", "prefs.ini", NULL );
 }
 
+static char*
+getCompat121PrefsFilename( void )
+{
+    return g_build_filename( g_get_user_config_dir(), "transmission", "gtk", "prefs.ini", NULL );
+}
+
 static void
 translate_08_to_09( const char* oldfile, const char* newfile )
 {
@@ -321,22 +341,98 @@ translate_08_to_09( const char* oldfile, const char* newfile )
     g_free( contents );
 }
 
+static void
+translate_keyfile_to_json( const char * old_file, const char * new_file )
+{
+    tr_benc dict;
+    GKeyFile * keyfile;
+    gchar ** keys;
+    gsize i;
+    gsize length;
+
+    static struct pref_entry {
+	const char* oldkey;
+	const char* newkey;
+    } renamed[] = {
+	{ "default-download-directory", "download-dir" },
+        { "encrypted-connections-only", "encryption" },
+	{ "listening-port", "peer-port" },
+        { "nat-traversal-enabled", "port-forwarding-enabled" },
+        { "open-dialog-folder", "open-dialog-dir" },
+        { "watch-folder", "watch-dir" },
+        { "watch-folder-enabled", "watch-dir-enabled" }
+    };
+
+    keyfile = g_key_file_new( );
+    g_key_file_load_from_file( keyfile, old_file, 0, NULL );
+    length = 0;
+    keys = g_key_file_get_keys( keyfile, "general", &length, NULL );
+
+    tr_bencInitDict( &dict, length );
+    for( i=0; i<length; ++i )
+    {
+        guint j;
+        const char * key = keys[i];
+        gchar * val = g_key_file_get_value( keyfile, "general", key, NULL );
+
+        for( j=0; j<G_N_ELEMENTS(renamed); ++j )
+            if( !strcmp( renamed[j].oldkey, key ) )
+                key = renamed[j].newkey;
+
+        if( !strcmp(val,"true") || !strcmp(val,"false") )
+            tr_bencDictAddInt( &dict, key, !strcmp( val, "true" ) );
+        else {
+            char * end;
+            long l;
+            errno = 0;
+            l = strtol( val, &end, 10 );
+            if( !errno && end && !*end )
+                tr_bencDictAddInt( &dict, key, l );
+            else
+                tr_bencDictAddStr( &dict, key, val );
+        }
+
+        g_free( val );
+    }
+
+    g_key_file_free( keyfile );
+    tr_bencSaveJSONFile( new_file, &dict );
+    tr_bencFree( &dict );
+}
+
 void
 cf_check_older_configs( void )
 {
-    char * cfn = getPrefsFilename( );
-    char * cfn08 = getCompat08PrefsFilename( );
-    char * cfn09 = getCompat09PrefsFilename( );
+    char * filename = getPrefsFilename( );
 
-    if( !g_file_test( cfn,   G_FILE_TEST_IS_REGULAR )
-      && g_file_test( cfn09, G_FILE_TEST_IS_REGULAR ) )
-      g_rename( cfn09, cfn );
+    if( !g_file_test( filename, G_FILE_TEST_IS_REGULAR ) )
+    {
+        char * key1 = getCompat121PrefsFilename( );
+        char * key2 = getCompat090PrefsFilename( );
+        char * benc = getCompat080PrefsFilename( );
 
-    if( !g_file_test( cfn,   G_FILE_TEST_IS_REGULAR )
-      && g_file_test( cfn08, G_FILE_TEST_IS_REGULAR ) )
-        translate_08_to_09( cfn08, cfn );
+        if( g_file_test( key1, G_FILE_TEST_IS_REGULAR ) )
+        {
+            translate_keyfile_to_json( key1, filename );
+        }
+        else if( g_file_test( key2, G_FILE_TEST_IS_REGULAR ) )
+        {
+            translate_keyfile_to_json( key2, filename );
+        }
+        else if( g_file_test( benc, G_FILE_TEST_IS_REGULAR ) )
+        {
+            char * tmpfile;
+            int fd = g_file_open_tmp( "transmission-prefs-XXXXXX", &tmpfile, NULL );
+            if( fd != -1 ) close( fd );
+            translate_08_to_09( benc, tmpfile );
+            translate_keyfile_to_json( tmpfile, filename );
+            unlink( tmpfile );
+        }
 
-    g_free( cfn09 );
-    g_free( cfn08 );
-    g_free( cfn );
+        g_free( benc );
+        g_free( key2 );
+        g_free( key1 );
+    }
+
+    g_free( filename );
 }
