@@ -7,9 +7,10 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  * 
- * $Id: details.c 5673 2008-04-24 01:42:53Z charles $
+ * $Id: details.c 6501 2008-08-13 03:05:52Z charles $
  */
 
+#include <errno.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -17,11 +18,13 @@
 #include <gtk/gtk.h>
 
 #include <libtransmission/transmission.h>
+#include <libtransmission/utils.h> /* tr_httpIsValidURL */
 
 #include "actions.h"
 #include "details.h"
 #include "file-list.h"
 #include "tr-torrent.h"
+#include "tracker-list.h"
 #include "hig.h"
 #include "util.h"
 
@@ -229,14 +232,50 @@ refresh_pieces( GtkWidget * da, GdkEventExpose * event UNUSED, gpointer gtor )
 *****  PEERS TAB
 ****/
 
+enum
+{
+  WEBSEED_COL_URL,
+  WEBSEED_COL_DOWNLOAD_RATE,
+  N_WEBSEED_COLS
+};
+
+static const char * webseed_column_names[N_WEBSEED_COLS] =
+{
+    N_( "Web Seeds" ),
+    /* 'download speed' column header. terse to keep the column narrow. */
+    N_( "Down" )
+};
+
+static GtkTreeModel*
+webseed_model_new( const tr_torrent * tor )
+{
+    int i;
+    const tr_info * inf = tr_torrentInfo( tor );
+    float * speeds = tr_torrentWebSpeeds( tor );
+    GtkListStore * store = gtk_list_store_new( N_WEBSEED_COLS,
+                                               G_TYPE_STRING,
+                                               G_TYPE_FLOAT );
+    for( i=0; i<inf->webseedCount; ++i )
+    {
+        GtkTreeIter iter;
+        gtk_list_store_append( store, &iter );
+        gtk_list_store_set( store, &iter, WEBSEED_COL_URL, inf->webseeds[i],
+                                          WEBSEED_COL_DOWNLOAD_RATE, speeds[i],
+                                          -1 );
+    }
+
+    tr_free( speeds );
+    return GTK_TREE_MODEL( store );
+}
+
 enum
 {
   PEER_COL_ADDRESS,
+  PEER_COL_DOWNLOAD_RATE,
+  PEER_COL_UPLOAD_RATE,
   PEER_COL_CLIENT,
   PEER_COL_PROGRESS,
   PEER_COL_IS_ENCRYPTED,
-  PEER_COL_DOWNLOAD_RATE,
-  PEER_COL_UPLOAD_RATE,
   PEER_COL_STATUS,
   N_PEER_COLS
 };
@@ -244,14 +283,14 @@ enum
 static const char* peer_column_names[N_PEER_COLS] =
 {
   N_("Address"),
-  N_("Client"),
-  /* 'percent done' column header. terse to keep the column narrow. */
-  N_("%"),
-  " ",
   /* 'download speed' column header. terse to keep the column narrow. */
   N_("Down"),
   /* 'upload speed' column header.  terse to keep the column narrow. */
   N_("Up"),
+  N_("Client"),
+  /* 'percent done' column header. terse to keep the column narrow. */
+  N_("%"),
+  " ",
   N_("Status")
 };
 
@@ -269,7 +308,7 @@ static int compare_addr_to_peer (const void * a, const void * b)
 }
 
 static void
-peer_row_set (GtkTreeStore        * store,
+peer_row_set (GtkListStore        * store,
               GtkTreeIter         * iter,
               const tr_peer_stat  * peer)
 {
@@ -278,7 +317,7 @@ peer_row_set (GtkTreeStore        * store,
   if (!client || !strcmp(client,"Unknown Client"))
     client = " ";
 
-  gtk_tree_store_set (store, iter,
+  gtk_list_store_set( store, iter,
                       PEER_COL_ADDRESS, peer->addr,
                       PEER_COL_CLIENT, client,
                       PEER_COL_IS_ENCRYPTED, peer->isEncrypted,
@@ -290,14 +329,14 @@ peer_row_set (GtkTreeStore        * store,
 }
 
 static void
-append_peers_to_model (GtkTreeStore        * store,
+append_peers_to_model (GtkListStore        * store,
                        const tr_peer_stat  * peers,
                        int                   n_peers)
 {
   int i;
   for (i=0; i<n_peers; ++i) {
     GtkTreeIter iter;
-    gtk_tree_store_append (store, &iter, NULL);
+    gtk_list_store_append( store, &iter );
     peer_row_set (store, &iter, &peers[i]);
   }
 }
@@ -305,13 +344,13 @@ append_peers_to_model (GtkTreeStore        * store,
 static GtkTreeModel*
 peer_model_new (tr_torrent * tor)
 {
-  GtkTreeStore * m = gtk_tree_store_new (N_PEER_COLS,
+  GtkListStore * m = gtk_list_store_new (N_PEER_COLS,
                                          G_TYPE_STRING,   /* addr */
+                                         G_TYPE_FLOAT,    /* downloadFromRate */
+                                         G_TYPE_FLOAT,    /* uploadToRate */
                                          G_TYPE_STRING,   /* client */
                                          G_TYPE_INT,      /* progress [0..100] */
                                          G_TYPE_BOOLEAN,  /* isEncrypted */
-                                         G_TYPE_FLOAT,    /* downloadFromRate */
-                                         G_TYPE_FLOAT,    /* uploadToRate */
                                          G_TYPE_STRING ); /* flagString */
 
   int n_peers = 0;
@@ -392,7 +431,8 @@ typedef struct
 {
   TrTorrent * gtor;
   GtkTreeModel * model; /* same object as store, but recast */
-  GtkTreeStore * store; /* same object as model, but recast */
+  GtkListStore * store; /* same object as model, but recast */
+  GtkListStore * webseeds;
   GtkWidget * completeness;
   GtkWidget * seeders_lb;
   GtkWidget * leechers_lb;
@@ -421,9 +461,23 @@ refresh_peers (GtkWidget * top)
   PeerData * p = (PeerData*) g_object_get_data (G_OBJECT(top), "peer-data");
   tr_torrent * tor = tr_torrent_handle ( p->gtor );
   GtkTreeModel * model = p->model;
-  GtkTreeStore * store = p->store;
+  GtkListStore * store = p->store;
   tr_peer_stat * peers;
   const tr_stat * stat = tr_torrent_stat( p->gtor );
+  const tr_info * inf = tr_torrent_info( p->gtor );
+
+  if( inf->webseedCount )
+  {
+    float * speeds = tr_torrentWebSpeeds( tor );
+    for( i=0; i<inf->webseedCount; ++i )
+    {
+        GtkTreeIter iter;
+        gtk_tree_model_iter_nth_child( GTK_TREE_MODEL( p->webseeds ), &iter, NULL, i );
+        gtk_list_store_set( p->webseeds, &iter, WEBSEED_COL_DOWNLOAD_RATE, speeds[i],
+                                                -1 );
+    }
+    tr_free( speeds );
+  }
 
   /**
   ***  merge the peer diffs into the tree model.
@@ -437,7 +491,6 @@ refresh_peers (GtkWidget * top)
   peers = tr_torrentPeers (tor, &n_peers);
   qsort (peers, n_peers, sizeof(tr_peer_stat), compare_peers);
 
-  i = 0;
   if (gtk_tree_model_get_iter_first (model, &iter)) do
   {
     char * addr = NULL;
@@ -459,7 +512,7 @@ refresh_peers (GtkWidget * top)
       g_memmove (peer, peer+1, sizeof(tr_peer_stat)*n_rhs);
       --n_peers;
     }
-    else if (!gtk_tree_store_remove (store, &iter))
+    else if (!gtk_list_store_remove (store, &iter))
       break; /* we removed the model's last item */
   }
   while (gtk_tree_model_iter_next (model, &iter));
@@ -471,9 +524,9 @@ refresh_peers (GtkWidget * top)
     refresh_pieces (p->completeness, NULL, p->gtor);
 #endif
 
-  fmtpeercount (p->seeders_lb, stat->seeders);
-  fmtpeercount (p->leechers_lb, stat->leechers);
-  fmtpeercount (p->completed_lb, stat->completedFromTracker );
+  fmtpeercount (p->seeders_lb, stat->seeders );
+  fmtpeercount (p->leechers_lb, stat->leechers );
+  fmtpeercount (p->completed_lb, stat->timesCompleted );
 
   free( peers );
 }
@@ -503,12 +556,12 @@ onPeerViewQueryTooltip( GtkWidget  * widget,
             const char * txt = NULL;
             switch( *pch ) {
                 case 'O': txt = _( "Optimistic unchoke" ); break;
-                case 'D': txt = _( "Downloading from peer" ); break;
-                case 'd': txt = _( "We'd download from peer if they'd let us" ); break;
+                case 'D': txt = _( "Downloading from this peer" ); break;
+                case 'd': txt = _( "We would download from this peer if they would let us" ); break;
                 case 'U': txt = _( "Uploading to peer" ); break;
-                case 'u': txt = _( "We'd upload to peer if they'd ask us" ); break;
+                case 'u': txt = _( "We would upload to this peer if they asked" ); break;
                 case 'K': txt = _( "Peer has unchoked us, but we're not interested" ); break;
-                case '?': txt = _( "We unchoked the peer, but they're not interested" ); break;
+                case '?': txt = _( "We unchoked this peer, but they're not interested" ); break;
                 case 'E': txt = _( "Encrypted connection" ); break;
                 case 'X': txt = _( "Peer was discovered through Peer Exchange (PEX)" ); break;
                 case 'I': txt = _( "Peer is an incoming connection" ); break;
@@ -534,9 +587,11 @@ peer_page_new ( TrTorrent * gtor )
 {
   guint i;
   GtkTreeModel *m;
-  GtkWidget *h, *v, *w, *ret, *sw, *l, *vbox, *hbox;
+  GtkWidget *v, *w, *ret, *sw, *l, *vbox, *hbox;
+  GtkWidget *webtree = NULL;
   tr_torrent * tor = tr_torrent_handle (gtor);
   PeerData * p = g_new (PeerData, 1);
+  const tr_info * inf = tr_torrent_info( gtor );
 
   /* TODO: make this configurable? */
   int view_columns[] = { PEER_COL_IS_ENCRYPTED,
@@ -547,6 +602,42 @@ peer_page_new ( TrTorrent * gtor )
                          PEER_COL_ADDRESS,
                          PEER_COL_CLIENT };
 
+ 
+  if( inf->webseedCount )
+  {
+    GtkTreeViewColumn * c;
+    GtkCellRenderer * r;
+    const char * t;
+    GtkWidget * fr;
+
+    m = webseed_model_new( tr_torrent_handle( gtor ) );
+    webtree = gtk_tree_view_new_with_model( m );
+    gtk_tree_view_set_rules_hint( GTK_TREE_VIEW( webtree ), TRUE );
+    p->webseeds = GTK_LIST_STORE( m );
+    g_object_unref( G_OBJECT( m ) );
+
+    t = _( webseed_column_names[WEBSEED_COL_URL] );
+    r = gtk_cell_renderer_text_new ();
+    g_object_set( G_OBJECT( r ), "ellipsize", PANGO_ELLIPSIZE_END, NULL );
+    c = gtk_tree_view_column_new_with_attributes( t, r, "text", WEBSEED_COL_URL, NULL);
+    g_object_set( G_OBJECT( c ), "expand", TRUE, NULL );
+    gtk_tree_view_column_set_sort_column_id( c, WEBSEED_COL_URL );
+    gtk_tree_view_append_column( GTK_TREE_VIEW( webtree ), c );
+
+    t = _( webseed_column_names[WEBSEED_COL_DOWNLOAD_RATE] );
+    r = gtk_cell_renderer_text_new ();
+    c = gtk_tree_view_column_new_with_attributes (t, r, "text", WEBSEED_COL_DOWNLOAD_RATE, NULL);
+    gtk_tree_view_column_set_cell_data_func (c, r, render_dl_rate,
+                                                   NULL, NULL);
+    gtk_tree_view_column_set_sort_column_id( c, WEBSEED_COL_DOWNLOAD_RATE );
+    gtk_tree_view_append_column( GTK_TREE_VIEW( webtree ), c );
+
+    fr = gtk_frame_new( NULL );
+    gtk_frame_set_shadow_type( GTK_FRAME( fr ), GTK_SHADOW_IN );
+    gtk_container_add( GTK_CONTAINER( fr ), webtree );
+    webtree = fr;
+  } 
+
   m  = peer_model_new (tor);
   v = GTK_WIDGET( g_object_new( GTK_TYPE_TREE_VIEW,
                                 "model", m,
@@ -650,38 +741,12 @@ peer_page_new ( TrTorrent * gtor )
   vbox = gtk_vbox_new (FALSE, GUI_PAD);
   gtk_container_set_border_width (GTK_CONTAINER(vbox), GUI_PAD_BIG);
 
-#ifdef SHOW_PIECES
-    l = gtk_label_new (NULL);
-    gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-    gtk_label_set_markup (GTK_LABEL(l), _( "<b>Availability</b>" ) );
-    gtk_box_pack_start (GTK_BOX(vbox), l, FALSE, FALSE, 0);
+    if( webtree )
+        gtk_box_pack_start( GTK_BOX( vbox ), webtree, FALSE, FALSE, 0 );
 
-    w = da = p->completeness = gtk_drawing_area_new ();
-    gtk_widget_set_size_request (w, 0u, 100u);
-    g_object_set_data (G_OBJECT(w), "draw-mode", GINT_TO_POINTER(DRAW_AVAIL));
-    g_signal_connect (w, "expose-event", G_CALLBACK(refresh_pieces), gtor);
-
-    h = gtk_hbox_new (FALSE, GUI_PAD);
-    w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-    gtk_widget_set_size_request (w, GUI_PAD_BIG, 0);
-    gtk_box_pack_start (GTK_BOX(h), w, FALSE, FALSE, 0);
-    gtk_box_pack_start_defaults (GTK_BOX(h), da);
-    gtk_box_pack_start (GTK_BOX(vbox), h, FALSE, FALSE, 0);
-
-    /* a small vertical spacer */
-    w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-    gtk_widget_set_size_request (w, 0u, GUI_PAD);
-    gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
-
-    l = gtk_label_new (NULL);
-    gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-    gtk_label_set_markup (GTK_LABEL(l), _( "<b>Connected Peers</b>" ) );
-    gtk_box_pack_start (GTK_BOX(vbox), l, FALSE, FALSE, 0);
-#endif
-
-    h = gtk_hbox_new (FALSE, GUI_PAD);
-    gtk_box_pack_start_defaults (GTK_BOX(h), sw);
-    gtk_box_pack_start_defaults (GTK_BOX(vbox), h);
+    //h = gtk_hbox_new (FALSE, GUI_PAD);
+    //gtk_box_pack_start_defaults (GTK_BOX(h), sw);
+    gtk_box_pack_start_defaults( GTK_BOX( vbox ), sw );
 
     hbox = gtk_hbox_new (FALSE, GUI_PAD);
         l = gtk_label_new (NULL);
@@ -699,7 +764,7 @@ peer_page_new ( TrTorrent * gtor )
     gtk_box_pack_start_defaults (GTK_BOX(hbox),
                                  gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f));
         l = gtk_label_new (NULL);
-        gtk_label_set_markup (GTK_LABEL(l), _( "<b>Completed:</b>" ) );
+        gtk_label_set_markup (GTK_LABEL(l), _( "<b>Times Completed:</b>" ) );
         gtk_box_pack_start (GTK_BOX(hbox), l, FALSE, FALSE, 0);
         l = p->completed_lb = gtk_label_new (NULL);
         gtk_box_pack_start (GTK_BOX(hbox), l, FALSE, FALSE, 0);
@@ -708,7 +773,7 @@ peer_page_new ( TrTorrent * gtor )
   ret = vbox;
   p->gtor = gtor;
   p->model = m;
-  p->store = GTK_TREE_STORE(m);
+  p->store = GTK_LIST_STORE(m);
   g_object_set_data_full (G_OBJECT(ret), "peer-data", p, g_free);
   return ret;
 }
@@ -717,6 +782,19 @@ peer_page_new ( TrTorrent * gtor )
 *****  INFO TAB
 ****/
 
+static void
+refresh_time_lb( GtkWidget * l, time_t t )
+{
+    const char * never = _( "Never" );
+    if( !t )
+        gtk_label_set_text( GTK_LABEL( l ), never );
+    else {
+        char * str = gtr_localtime( t );
+        gtk_label_set_text( GTK_LABEL( l ), str );
+        g_free( str );
+    }
+}
+
 static GtkWidget*
 info_page_new (tr_torrent * tor)
 {
@@ -725,20 +803,23 @@ info_page_new (tr_torrent * tor)
   GtkWidget *l, *w, *fr;
   char *pch;
   char sizeStr[128];
+  char countStr[128];
   char buf[256];
   GtkTextBuffer * b;
   const tr_info * info = tr_torrentInfo(tor);
 
   hig_workarea_add_section_title (t, &row, _("Details"));
 
+    g_snprintf( countStr, sizeof( countStr ),
+                ngettext( "%'d Piece", "%'d Pieces", info->pieceCount ),
+                info->pieceCount );
     tr_strlsize( sizeStr, info->pieceSize, sizeof(sizeStr) );
     g_snprintf( buf, sizeof( buf ),
-                /* %1$'d is number of pieces
+                /* %1$s is number of pieces;
                    %2$s is how big each piece is */
-                ngettext( "%1$'d Piece @ %2$s",
-                          "%1$'d Pieces @ %2$s",
-                          info->pieceCount ),
-                info->pieceCount, sizeStr );
+                _( "%1$s @ %2$s" ),
+              countStr, sizeStr );
+
     l = gtk_label_new (buf);
     hig_workarea_add_row (t, &row, _("Pieces:"), l, NULL);
 
@@ -769,17 +850,17 @@ info_page_new (tr_torrent * tor)
   hig_workarea_add_section_title (t, &row, _("Origins"));
   
     l = gtk_label_new (*info->creator ? info->creator : _("Unknown"));
+    gtk_label_set_ellipsize( GTK_LABEL( l ), PANGO_ELLIPSIZE_END );
     hig_workarea_add_row (t, &row, _("Creator:"), l, NULL);
 
-    pch = rfc822date ((guint64)info->dateCreated * 1000u);
-    l = gtk_label_new (pch);
+    l = gtk_label_new( NULL );
+    refresh_time_lb( l, info->dateCreated );
     hig_workarea_add_row (t, &row, _("Date:"), l, NULL); 
-    g_free (pch);
 
   hig_workarea_add_section_divider (t, &row);
   hig_workarea_add_section_title (t, &row, _("Location"));
 
-    l = g_object_new( GTK_TYPE_LABEL, "label", tr_torrentGetFolder( tor ), "selectable", TRUE,
+    l = g_object_new( GTK_TYPE_LABEL, "label", tr_torrentGetDownloadDir( tor ), "selectable", TRUE,
                                       "ellipsize", PANGO_ELLIPSIZE_END, NULL );
     hig_workarea_add_row (t, &row, _( "Destination folder:" ), l, NULL); 
 
@@ -857,21 +938,15 @@ refresh_activity (GtkWidget * top)
   tr_strlratio( buf, stat->ratio, sizeof( buf ) );
   gtk_label_set_text( GTK_LABEL( a->ratio_lb ), buf );
 
-  tr_strlspeed( buf, stat->swarmspeed, sizeof(buf) );
+  tr_strlspeed( buf, stat->swarmSpeed, sizeof(buf) );
   gtk_label_set_text (GTK_LABEL(a->swarm_lb), buf );
 
   gtk_label_set_text (GTK_LABEL(a->err_lb),
                       *stat->errorString ? stat->errorString : _("None"));
 
-  pch = stat->startDate ? rfc822date( stat->startDate )
-                        : g_strdup_printf( _( "Unknown" ) );
-  gtk_label_set_text (GTK_LABEL(a->date_added_lb), pch);
-  g_free (pch);
+  refresh_time_lb( a->date_added_lb, stat->addedDate );
 
-  pch = stat->activityDate ? rfc822date( stat->activityDate )
-                           : g_strdup_printf( _( "Unknown" ) );
-  gtk_label_set_text (GTK_LABEL(a->last_activity_lb), pch);
-  g_free (pch);
+  refresh_time_lb( a->last_activity_lb, stat->activityDate );
 
 #ifdef SHOW_PIECES
   if (GDK_IS_DRAWABLE (a->availability_da->window))
@@ -972,15 +1047,6 @@ dl_speed_toggled_cb (GtkToggleButton *tb, gpointer gtor)
 }
 
 
-#if 0
-static void
-seeding_cap_toggled_cb (GtkToggleButton *tb, gpointer gtor)
-{
-  tr_torrent_set_seeding_cap_enabled (TR_TORRENT(gtor), 
-                                      gtk_toggle_button_get_active(tb));
-}
-#endif
-
 static void
 sensitize_from_check_cb (GtkToggleButton *toggle, gpointer w)
 {
@@ -1006,20 +1072,11 @@ dl_speed_spun_cb (GtkSpinButton *spin, gpointer gtor)
   setSpeedLimit( spin, gtor, TR_DOWN );
 }
 
-#if 0
-static void
-seeding_ratio_spun_cb (GtkSpinButton *spin, gpointer gtor)
-{
-  tr_torrent_set_seeding_cap_ratio (TR_TORRENT(gtor),
-                                    gtk_spin_button_get_value(spin));
-}
-#endif
-
 static void
 max_peers_spun_cb( GtkSpinButton * spin, gpointer gtor )
 {
   const uint16_t n = gtk_spin_button_get_value( spin );
-  tr_torrentSetMaxConnectedPeers( tr_torrent_handle( gtor ), n );
+  tr_torrentSetPeerLimit( tr_torrent_handle( gtor ), n );
 }
 
 static GtkWidget*
@@ -1065,24 +1122,12 @@ options_page_new ( TrTorrent * gtor )
   hig_workarea_add_section_divider (t, &row);
   hig_workarea_add_section_title (t, &row, _("Peer Connections"));
 
-    maxConnectedPeers = tr_torrentGetMaxConnectedPeers( tor );
+    maxConnectedPeers = tr_torrentGetPeerLimit( tor );
     w = gtk_spin_button_new_with_range( 1, 3000, 5 );
     gtk_spin_button_set_value( GTK_SPIN_BUTTON( w ), maxConnectedPeers );
     hig_workarea_add_row( t, &row, _( "_Maximum peers:" ), w, w );
     g_signal_connect( w, "value-changed", G_CALLBACK( max_peers_spun_cb ), gtor );
 
-#if 0
-    tb = gtk_check_button_new_with_mnemonic (_("_Stop seeding at ratio:"));
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(tb), gtor->seeding_cap_enabled);
-    g_signal_connect (tb, "toggled", G_CALLBACK(seeding_cap_toggled_cb), gtor);
-    a = (GtkAdjustment*) gtk_adjustment_new (gtor->seeding_cap, 0.0, G_MAXDOUBLE, 1, 1, 1);
-    w = gtk_spin_button_new (a, 1, 1);
-    g_signal_connect (w, "value-changed", G_CALLBACK(seeding_ratio_spun_cb), gtor);
-    g_signal_connect (tb, "toggled", G_CALLBACK(sensitize_from_check_cb), w);
-    sensitize_from_check_cb (GTK_TOGGLE_BUTTON(tb), w);
-    hig_workarea_add_row_w (t, &row, tb, w, NULL);
-#endif
-
   hig_workarea_finish (t, &row);
   return t;
 }
@@ -1102,6 +1147,15 @@ struct tracker_page
 {
     TrTorrent * gtor;
 
+    GtkTreeView * view;
+    GtkListStore * store;
+    GtkTreeSelection * sel;
+
+    GtkWidget * add_button;
+    GtkWidget * remove_button;
+    GtkWidget * save_button;
+    GtkWidget * revert_button;
+
     GtkWidget * last_scrape_time_lb;
     GtkWidget * last_scrape_response_lb;
     GtkWidget * next_scrape_countdown_lb;
@@ -1112,11 +1166,12 @@ struct tracker_page
     GtkWidget * manual_announce_countdown_lb;
 };
 
-GtkWidget*
+static GtkWidget*
 tracker_page_new( TrTorrent * gtor )
 {
     GtkWidget * t;
     GtkWidget * l;
+    GtkWidget * w;
     int row = 0;
     const char * s;
     struct tracker_page * page = g_new0( struct tracker_page, 1 );
@@ -1125,6 +1180,12 @@ tracker_page_new( TrTorrent * gtor )
     page->gtor = gtor;
 
     t = hig_workarea_create( );
+    hig_workarea_add_section_title( t, &row, _( "Trackers" ) );
+
+        w = tracker_list_new( gtor );
+        hig_workarea_add_wide_control( t, &row, w );
+
+    hig_workarea_add_section_divider( t, &row );
     hig_workarea_add_section_title( t, &row, _( "Scrape" ) );
 
         s = _( "Last scrape at:" );
@@ -1171,36 +1232,27 @@ tracker_page_new( TrTorrent * gtor )
         page->manual_announce_countdown_lb = l;
         hig_workarea_add_row( t, &row, s, l, NULL );
 
-    g_object_set_data_full( G_OBJECT( t ), TRACKER_PAGE, page, g_free );
     hig_workarea_finish( t, &row );
+    g_object_set_data_full( G_OBJECT( t ), TRACKER_PAGE, page, g_free );
     return t;
 }
 
 static void
-refresh_countdown_lb( GtkWidget * l, time_t t )
+refresh_countdown_lb( GtkWidget * w, time_t t,
+                      const char * countdown_done )
 {
     const time_t now = time( NULL );
+    GtkLabel * l = GTK_LABEL( w );
 
-    if( !t || ( t < now ) )
-        gtk_label_set_text( GTK_LABEL( l ), _( "Never" ) );
+    if( t == 1 )
+        gtk_label_set_text( l, _( "In progress" ) );
+    else if( t < now )
+        gtk_label_set_text( l, countdown_done );
     else {
         char buf[1024];
         const int seconds = t - now;
         tr_strltime( buf, seconds, sizeof( buf ) );
-        gtk_label_set_text( GTK_LABEL( l ), buf );
-    }
-}
-
-static void
-refresh_time_lb( GtkWidget * l, time_t t )
-{
-    const char * never = _( "Never" );
-    if( !t )
-        gtk_label_set_text( GTK_LABEL( l ), never );
-    else {
-        char * str = rfc822date( (guint64)t * 1000 );
-        gtk_label_set_text( GTK_LABEL( l ), str );
-        g_free( str );
+        gtk_label_set_text( l, buf );
     }
 }
 
@@ -1213,30 +1265,30 @@ refresh_tracker( GtkWidget * w )
     const tr_stat * torStat = tr_torrent_stat( page->gtor );
 
     l = page->last_scrape_time_lb;
-    t = torStat->tracker_stat.lastScrapeTime;
+    t = torStat->lastScrapeTime;
     refresh_time_lb( l, t );
 
     l = page->last_scrape_response_lb;
-    gtk_label_set_text( GTK_LABEL( l ), torStat->tracker_stat.scrapeResponse );
+    gtk_label_set_text( GTK_LABEL( l ), torStat->scrapeResponse );
 
     l = page->next_scrape_countdown_lb;
-    t = torStat->tracker_stat.nextScrapeTime;
-    refresh_countdown_lb( l, t );
+    t = torStat->nextScrapeTime;
+    refresh_countdown_lb( l, t, _( "Never" ) );
 
     l = page->last_announce_time_lb;
-    t = torStat->tracker_stat.lastAnnounceTime;
+    t = torStat->lastAnnounceTime;
     refresh_time_lb( l, t );
 
     l = page->last_announce_response_lb;
-    gtk_label_set_text( GTK_LABEL( l ), torStat->tracker_stat.announceResponse );
+    gtk_label_set_text( GTK_LABEL( l ), torStat->announceResponse );
 
     l = page->next_announce_countdown_lb;
-    t = torStat->tracker_stat.nextAnnounceTime;
-    refresh_countdown_lb( l, t );
+    t = torStat->nextAnnounceTime;
+    refresh_countdown_lb( l, t, _( "Never" ) );
 
     l = page->manual_announce_countdown_lb;
-    t = torStat->tracker_stat.nextManualAnnounceTime;
-    refresh_countdown_lb( l, t );
+    t = torStat->manualAnnounceTime;
+    refresh_countdown_lb( l, t, _( "Now" ) );
 }
 
 /****
