@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  * 
- * $Id: makemeta-ui.c 5765 2008-05-07 04:06:07Z charles $
+ * $Id: makemeta-ui.c 6358 2008-07-18 18:14:29Z charles $
  */
 
 #include <string.h>
@@ -20,18 +20,19 @@
 
 #include "hig.h"
 #include "makemeta-ui.h"
+#include "tracker-list.h"
 #include "util.h"
 
 #define UPDATE_INTERVAL_MSEC 200
 
-#define BUILDER_KEY "builder"
 #define UI_KEY "ui"
 
 typedef struct
 {
+    GtkWidget * filename_entry;
     GtkWidget * size_lb;
     GtkWidget * pieces_lb;
-    GtkWidget * announce_entry;
+    GtkWidget * announce_list;
     GtkWidget * comment_entry;
     GtkWidget * progressbar;
     GtkWidget * private_check;
@@ -97,7 +98,7 @@ refresh_cb ( gpointer user_data )
         switch( ui->builder->result )
         {
             case TR_MAKEMETA_OK:
-                txt = g_strdup( _( "Torrent created" ) );
+                txt = g_strdup( _( "Torrent created!" ) );
                 break;
 
             case TR_MAKEMETA_URL:
@@ -145,6 +146,9 @@ response_cb( GtkDialog* d, int response, gpointer user_data )
     char *tmp;
     char buf[1024];
     guint tag;
+    tr_tracker_info * trackers = NULL;
+    int i;
+    int trackerCount = 0;
 
     if( response != GTK_RESPONSE_ACCEPT )
     {
@@ -177,14 +181,21 @@ response_cb( GtkDialog* d, int response, gpointer user_data )
     gtk_progress_bar_set_text( GTK_PROGRESS_BAR(ui->progressbar), buf );
     g_free( tmp );
 
+    trackers = tracker_list_get_trackers( ui->announce_list, &trackerCount );
+
     tr_makeMetaInfo( ui->builder,
                      NULL, 
-                     gtk_entry_get_text( GTK_ENTRY( ui->announce_entry ) ),
+                     trackers, trackerCount,
                      gtk_entry_get_text( GTK_ENTRY( ui->comment_entry ) ),
                      gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( ui->private_check ) ) );
 
     tag = g_timeout_add (UPDATE_INTERVAL_MSEC, refresh_cb, ui);
     g_object_set_data_full (G_OBJECT(d), "tag", GUINT_TO_POINTER(tag), remove_tag);
+
+    /* cleanup */
+    for( i=0; i<trackerCount; ++i )
+        g_free( trackers[i].announce );
+    g_free( trackers );
 }
 
 /***
@@ -200,14 +211,14 @@ refreshFromBuilder( MakeMetaUI * ui )
     const char * filename = builder ? builder->top : NULL;
 
     if( !filename )
-        g_snprintf( buf, sizeof( buf ), _( "No files selected" ) );
+        g_snprintf( buf, sizeof( buf ), _( "No source selected" ) );
     else
         g_snprintf( buf, sizeof(buf), "%s.torrent (%d%%)", filename, 0 );
     gtk_progress_bar_set_text( GTK_PROGRESS_BAR( ui->progressbar ), buf );
     refreshButtons( ui );
 
     if( !filename )
-        g_snprintf( buf, sizeof( buf ), _( "<i>No files selected</i>" ) );
+        g_snprintf( buf, sizeof( buf ), _( "<i>No source selected</i>" ) );
     else {
         tr_strlsize( sizeStr, builder->totalSize, sizeof(sizeStr) );
         g_snprintf( buf, sizeof( buf ),
@@ -222,65 +233,92 @@ refreshFromBuilder( MakeMetaUI * ui )
     if( !filename )
         *buf = '\0';
     else {
+        char countStr[512];
+        g_snprintf( countStr, sizeof( countStr ),
+                    ngettext( "%'d Piece", "%'d Pieces", builder->pieceCount ),
+                    builder->pieceCount );
         tr_strlsize( sizeStr, builder->pieceSize, sizeof(sizeStr) );
         g_snprintf( buf, sizeof( buf ),
-                    /* %1$'s is number of pieces;
+                    /* %1$s is number of pieces;
                        %2$s is how big each piece is */
-                    ngettext( "<i>%1$'d Piece @ %2$s</i>",
-                              "<i>%1$'d Pieces @ %2$s</i>",
-                              builder->pieceCount ),
-                    builder->pieceCount, sizeStr );
+                    _( "%1$s @ %2$s" ), 
+                    countStr,
+                    sizeStr );
     }
     gtk_label_set_markup ( GTK_LABEL(ui->pieces_lb), buf );
 }
 
 static void
-onSelectionChanged( GtkFileChooser * chooser, gpointer gui )
+onSourceActivated( GtkEditable * editable, gpointer gui )
 {
+    const char * filename = gtk_entry_get_text( GTK_ENTRY( editable ) );
     MakeMetaUI * ui = gui;
-    char * filename = gtk_file_chooser_get_filename( chooser );
-    tr_metainfo_builder * builder = tr_metaInfoBuilderCreate( ui->handle, filename );
-
-    g_object_set_data_full( G_OBJECT( chooser ),
-                            BUILDER_KEY,
-                            builder,
-                            (GDestroyNotify)tr_metaInfoBuilderFree );
-    ui->builder = builder;
 
+    if( ui->builder )
+        tr_metaInfoBuilderFree( ui->builder );
+    ui->builder = tr_metaInfoBuilderCreate( ui->handle, filename );
     refreshFromBuilder( ui );
+}
 
-    g_free( filename );
+static gboolean
+onSourceLostFocus( GtkWidget * w, GdkEventFocus * focus UNUSED, gpointer gui )
+{
+    onSourceActivated( GTK_EDITABLE( w ), gui );
+    return FALSE;
 }
 
 static void
-onFileModeToggled( GtkToggleButton * t, gpointer w )
+onChooseClicked( GtkButton              * button,
+                 gpointer                 gui,
+                 const char             * title,
+                 GtkFileChooserAction     chooserAction )
 {
-    const gboolean active = gtk_toggle_button_get_active( t );
-    gtk_widget_set_sensitive( w, active );
-    if( active )
+    GtkWidget * top = gtk_widget_get_toplevel( GTK_WIDGET( button ) );
+    GtkWidget * d = gtk_file_chooser_dialog_new( title,
+                                                 GTK_WINDOW( top ),
+                                                 chooserAction,
+				                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				                 GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT,
+				                 NULL );
+    if( gtk_dialog_run( GTK_DIALOG( d ) ) == GTK_RESPONSE_ACCEPT )
     {
-        MakeMetaUI * ui = g_object_get_data( G_OBJECT( w ), UI_KEY );
-        ui->builder = g_object_get_data( G_OBJECT( w ), BUILDER_KEY );
-        refreshFromBuilder( ui );
+        MakeMetaUI * ui = gui;
+        char * filename = gtk_file_chooser_get_filename( GTK_FILE_CHOOSER( d ) );
+        gtk_entry_set_text( GTK_ENTRY( ui->filename_entry ), filename );
+        onSourceActivated( GTK_EDITABLE( ui->filename_entry ), gui );
+        g_free( filename );
     }
+
+    gtk_widget_destroy( d );
 }
-    
+
+static void
+onChooseDirectoryClicked( GtkButton * b, gpointer gui )
+{
+    onChooseClicked( b, gui, _( "Choose Directory" ), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER );
+}
+
+static void
+onChooseFileClicked( GtkButton * b, gpointer gui )
+{
+    onChooseClicked( b, gui, _( "Choose File" ), GTK_FILE_CHOOSER_ACTION_OPEN );
+}
+
 GtkWidget*
 make_meta_ui( GtkWindow * parent, tr_handle * handle )
 {
-    GSList * group;
     int row = 0;
-    GtkWidget *l, *d, *t, *w, *h;
+    GtkWidget *d, *t, *w, *h, *h2, *v, *focusMe, *extras;
     GtkBox * main_vbox;
     MakeMetaUI * ui = g_new0 ( MakeMetaUI, 1 );
     ui->handle = handle;
 
     d = gtk_dialog_new_with_buttons( _("New Torrent"),
                                      parent,
-                                     GTK_DIALOG_DESTROY_WITH_PARENT,
-                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                     GTK_DIALOG_DESTROY_WITH_PARENT|GTK_DIALOG_NO_SEPARATOR,
                                      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                      GTK_STOCK_NEW, GTK_RESPONSE_ACCEPT,
+                                     GTK_STOCK_STOP, GTK_RESPONSE_CANCEL,
                                      NULL );
     g_signal_connect( d, "response", G_CALLBACK(response_cb), ui );
     g_object_set_data_full( G_OBJECT(d), "ui", ui, freeMetaUI );
@@ -289,63 +327,79 @@ make_meta_ui( GtkWindow * parent, tr_handle * handle )
 
     t = hig_workarea_create ();
 
-    hig_workarea_add_section_title (t, &row, _( "Content" ));
+    hig_workarea_add_section_title (t, &row, _( "Source" ));
 
-        l = gtk_radio_button_new_with_mnemonic( NULL, _( "_Single File:" ) );
-        w = gtk_file_chooser_button_new( NULL, GTK_FILE_CHOOSER_ACTION_OPEN );
-        g_object_set_data( G_OBJECT( w ), UI_KEY, ui );
-        hig_workarea_add_row_w( t, &row, l, w, NULL );
-        group = gtk_radio_button_get_group( GTK_RADIO_BUTTON( l ) );
-        g_signal_connect( l, "toggled", G_CALLBACK(onFileModeToggled), w );
-        g_signal_connect( w, "selection-changed", G_CALLBACK(onSelectionChanged), ui );
-
-        l = gtk_radio_button_new_with_mnemonic( group, _( "_Folder:" ) );
-        w = gtk_file_chooser_button_new( NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER );
-        g_object_set_data( G_OBJECT( w ), UI_KEY, ui );
-        hig_workarea_add_row_w( t, &row, l, w, NULL );
-        g_signal_connect( l, "toggled", G_CALLBACK(onFileModeToggled), w );
-        g_signal_connect( w, "selection-changed", G_CALLBACK(onSelectionChanged), ui );
-        gtk_widget_set_sensitive( w, FALSE );
-
-        h = gtk_hbox_new( FALSE, GUI_PAD_SMALL );
+        h = gtk_hbox_new( FALSE, GUI_PAD );
+        v = gtk_vbox_new( FALSE, GUI_PAD_SMALL );
+        w = ui->filename_entry = gtk_entry_new( );
+        g_signal_connect( w, "activate", G_CALLBACK( onSourceActivated ), ui );
+        g_signal_connect( w, "focus-out-event", G_CALLBACK( onSourceLostFocus ), ui );
+        gtk_box_pack_start( GTK_BOX( v ), w, FALSE, FALSE, 0 );
+        h2 = gtk_hbox_new( FALSE, GUI_PAD_SMALL );
         w = ui->size_lb = gtk_label_new (NULL);
-        gtk_label_set_markup ( GTK_LABEL(w), _( "<i>No files selected</i>" ) );
-        gtk_box_pack_start( GTK_BOX(h), w, FALSE, FALSE, 0 );
+        gtk_label_set_markup ( GTK_LABEL(w), _( "<i>No source selected</i>" ) );
+        gtk_box_pack_start( GTK_BOX(h2), w, FALSE, FALSE, GUI_PAD_SMALL );
         w = ui->pieces_lb = gtk_label_new (NULL);
-        gtk_box_pack_end( GTK_BOX(h), w, FALSE, FALSE, 0 );
-        w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
+        gtk_box_pack_end( GTK_BOX(h2), w, FALSE, FALSE, GUI_PAD_SMALL );
+        w = gtk_alignment_new( 0.0f, 0.0f, 0.0f, 0.0f );
         gtk_widget_set_size_request (w, 2 * GUI_PAD_BIG, 0);
-        gtk_box_pack_start_defaults ( GTK_BOX(h), w );
-        hig_workarea_add_row (t, &row, "", h, NULL);
-        
+        gtk_box_pack_start_defaults ( GTK_BOX(h2), w );
+        gtk_box_pack_start( GTK_BOX( v ), h2, FALSE, FALSE, 0 );
+        gtk_box_pack_start_defaults( GTK_BOX( h ), v );
+        v = gtk_vbox_new( FALSE, GUI_PAD_SMALL );
+        w = tr_button_new_from_stock( GTK_STOCK_DIRECTORY, _( "F_older" ) );
+        focusMe = w;
+        g_signal_connect( w, "clicked", G_CALLBACK( onChooseDirectoryClicked ), ui );
+        gtk_box_pack_start_defaults( GTK_BOX( v ), w );
+        w = tr_button_new_from_stock( GTK_STOCK_FILE, _( "_File" ) );
+        g_signal_connect( w, "clicked", G_CALLBACK( onChooseFileClicked ), ui );
+        gtk_box_pack_start_defaults( GTK_BOX( v ), w );
+        gtk_box_pack_start( GTK_BOX( h ), v, FALSE, FALSE, 0 );
+        hig_workarea_add_wide_control( t, &row, h );
 
     hig_workarea_add_section_divider( t, &row );
-    hig_workarea_add_section_title (t, &row, _("Details"));
+    hig_workarea_add_section_title( t, &row, _( "Trackers" ) );
 
-        w = ui->private_check = hig_workarea_add_wide_checkbutton( t, &row, _( "_Private to this tracker" ), FALSE );
+        w = tracker_list_new( NULL );
+        ui->announce_list = w;
+        hig_workarea_add_wide_control( t, &row, w );
 
-        w = ui->announce_entry = gtk_entry_new( );
-        gtk_entry_set_text(GTK_ENTRY(w), "http://");
-        hig_workarea_add_row (t, &row, _( "Announce _URL:" ), w, NULL );
+    hig_workarea_add_section_divider( t, &row );
+    w = extras = gtk_expander_new_with_mnemonic( _( "<b>E_xtras</b>" ) );
+    gtk_expander_set_use_markup( GTK_EXPANDER( w ), TRUE );
+    hig_workarea_add_section_title_widget( t, &row, w );
 
+        {
+        int row2 = 0;
+        GtkWidget * t2 = hig_workarea_create( );
         w = ui->comment_entry = gtk_entry_new( );
-        hig_workarea_add_row (t, &row, _( "Commen_t:" ), w, NULL );
+        hig_workarea_add_row( t2, &row2, _( "Commen_t:" ), w, NULL );
+        w = hig_workarea_add_wide_checkbutton( t2, &row2, _( "_Private torrent" ), FALSE );
+        ui->private_check = w;
+        hig_workarea_finish( t2, &row2 );
+        gtk_container_add( GTK_CONTAINER( extras ), t2 );
+        }
 
     hig_workarea_finish( t, &row );
     gtk_box_pack_start_defaults( main_vbox, t );
 
     w = gtk_frame_new( NULL );
     gtk_frame_set_shadow_type( GTK_FRAME( w ), GTK_SHADOW_NONE );
-    gtk_container_set_border_width( GTK_CONTAINER( w ), GUI_PAD_BIG );
-    
-        ui->progressbar = gtk_progress_bar_new( );
-        gtk_progress_bar_set_text( GTK_PROGRESS_BAR( ui->progressbar), _( "No files selected" ) );
-        gtk_container_add( GTK_CONTAINER( w ), ui->progressbar );
+    gtk_container_set_border_width( GTK_CONTAINER( w ), GUI_PAD );
+    gtk_container_add( GTK_CONTAINER( w ), gtk_hseparator_new( ) );
+    gtk_box_pack_start( main_vbox, w, FALSE, FALSE, 0 );
 
-    gtk_box_pack_start( main_vbox, w, FALSE, FALSE, GUI_PAD_BIG );
+    w = gtk_frame_new( NULL );
+    gtk_frame_set_shadow_type( GTK_FRAME( w ), GTK_SHADOW_NONE );
+    gtk_container_set_border_width( GTK_CONTAINER( w ), GUI_PAD );
+    ui->progressbar = gtk_progress_bar_new( );
+    gtk_progress_bar_set_text( GTK_PROGRESS_BAR( ui->progressbar), _( "No source selected" ) );
+    gtk_container_add( GTK_CONTAINER( w ), ui->progressbar );
+    gtk_box_pack_start( main_vbox, w, FALSE, FALSE, 0 );
 
-    gtk_window_set_default_size( GTK_WINDOW(d), 400u, 0u );
+    gtk_window_set_default_size( GTK_WINDOW(d), 500, 0 );
     gtk_widget_show_all( GTK_DIALOG(d)->vbox );
     setIsBuilding( ui, FALSE );
+    gtk_widget_grab_focus( focusMe );
     return d;
 }
