@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  * 
- * $Id: msgwin.c 5491 2008-04-02 23:18:45Z charles $
+ * $Id: msgwin.c 6402 2008-07-25 14:56:28Z charles $
  */
 
 #include <errno.h>
@@ -97,13 +97,15 @@ doSave( GtkWindow      * parent,
             gtk_tree_model_get( model, &iter,
                                 COL_TR_MSG, &node,
                                 -1 );
-            date = rfc822date( (guint64)node->when * 1000u );
+            date = gtr_localtime( node->when );
             switch( node->level ) {
                 case TR_MSG_DBG: levelStr = "debug"; break;
                 case TR_MSG_ERR: levelStr = "error"; break;
                 default:         levelStr = "     "; break;
             }
-            fprintf( fp, "%s\t%s\t%s\t%s\n", date, levelStr, node->name, node->message );
+            fprintf( fp, "%s\t%s\t%s\t%s\n", date, levelStr,
+                     ( node->name ? node->name : "" ),
+                     ( node->message ? node->message : "" ) );
 
             g_free( date );
         }
