@@ -7,14 +7,19 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  * 
- * $Id:$
+ * $Id: stats.c 6390 2008-07-23 13:42:30Z charles $
  */
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include "hig.h"
 #include "stats.h"
-#include "tr_core.h"
+#include "tr-core.h"
+
+enum
+{
+    TR_RESPONSE_CLEAR = 1
+};
 
 struct stat_ui
 {
@@ -40,28 +45,29 @@ static void
 setLabelFromRatio( GtkWidget * w, double d )
 {
     char buf[128];
-    if( ( (int)d == TR_RATIO_NA ) )
-        g_strlcpy( buf, _("None"), sizeof(buf) );
-    else
-        g_snprintf( buf, sizeof(buf), "%.1f", d );
+    tr_strlratio( buf, d, sizeof( buf ) );
     setLabel( w, buf );
 }
 
 static gboolean
 updateStats( gpointer gdata )
 {
+    const char * fmt;
     char buf[128];
 
     struct stat_ui * ui = gdata;
     tr_session_stats one, all;
-    tr_getSessionStats( ui->core->handle, &one );
-    tr_getCumulativeSessionStats( ui->core->handle, &all );
+    tr_sessionGetStats( tr_core_handle( ui->core ), &one );
+    tr_sessionGetCumulativeStats( tr_core_handle( ui->core ), &all );
 
     setLabel( ui->one_up_lb, tr_strlsize( buf, one.uploadedBytes, sizeof(buf) ) );
     setLabel( ui->one_down_lb, tr_strlsize( buf, one.downloadedBytes, sizeof(buf) ) );
     setLabel( ui->one_time_lb, tr_strltime( buf, one.secondsActive, sizeof(buf) ) );
     setLabelFromRatio( ui->one_ratio_lb, one.ratio );
-    setLabel( ui->all_sessions_lb, g_strdup_printf( _("Started %d times"), (int)all.sessionCount ) );
+
+    fmt = ngettext( "Started %'d time", "Started %'d times", (int)all.sessionCount );
+    g_snprintf( buf, sizeof(buf), fmt, (int)all.sessionCount );
+    setLabel( ui->all_sessions_lb, buf );
     setLabel( ui->all_up_lb, tr_strlsize( buf, all.uploadedBytes, sizeof(buf) ) );
     setLabel( ui->all_down_lb, tr_strlsize( buf, all.downloadedBytes, sizeof(buf) ) );
     setLabel( ui->all_time_lb, tr_strltime( buf, all.secondsActive, sizeof(buf) ) );
@@ -71,10 +77,27 @@ updateStats( gpointer gdata )
 }
 
 static void
-dialogResponse( GtkDialog * dialog, gint response UNUSED, gpointer unused UNUSED )
+dialogDestroyed( gpointer p, GObject * dialog UNUSED )
 {
-    g_source_remove( GPOINTER_TO_UINT( g_object_get_data( G_OBJECT(dialog), "TrTimer" ) ) );
-    gtk_widget_destroy( GTK_WIDGET( dialog ) );
+    g_source_remove( GPOINTER_TO_UINT( p ) );
+}
+
+static void
+dialogResponse( GtkDialog * dialog, gint response, gpointer gdata )
+{
+    struct stat_ui * ui = gdata;
+
+    if( response == TR_RESPONSE_CLEAR )
+    {
+        tr_handle * handle = tr_core_handle( ui->core );
+        tr_sessionClearStats( handle );
+        updateStats( ui );
+    }
+
+    if( response == GTK_RESPONSE_CLOSE )
+    {
+        gtk_widget_destroy( GTK_WIDGET( dialog ) );
+    }
 }
 
 GtkWidget*
@@ -89,15 +112,21 @@ stats_dialog_create( GtkWindow * parent, TrCore * core )
 
     d = gtk_dialog_new_with_buttons( _("Statistics"),
                                      parent,
-                                     GTK_DIALOG_DESTROY_WITH_PARENT,
+                                     GTK_DIALOG_DESTROY_WITH_PARENT|GTK_DIALOG_NO_SEPARATOR,
+                                     GTK_STOCK_DELETE, TR_RESPONSE_CLEAR,
                                      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                      NULL );
+    gtk_dialog_set_default_response( GTK_DIALOG( d ),
+                                     GTK_RESPONSE_CLOSE );
+    gtk_dialog_set_alternative_button_order( GTK_DIALOG( d ),
+                                             GTK_RESPONSE_CLOSE,
+                                             TR_RESPONSE_CLEAR,
+                                             -1 );
     t = hig_workarea_create( );
     gtk_box_pack_start_defaults( GTK_BOX(GTK_DIALOG(d)->vbox), t );
     ui->core = core;
 
     hig_workarea_add_section_title( t, &row, _( "Current Session" ) );
-    hig_workarea_add_section_spacer( t, row, 4 );
         l = ui->one_up_lb = gtk_label_new( NULL );
         hig_workarea_add_row( t, &row, _("Uploaded:"), l, NULL );
         l = ui->one_down_lb = gtk_label_new( NULL );
@@ -107,9 +136,8 @@ stats_dialog_create( GtkWindow * parent, TrCore * core )
         l = ui->one_time_lb = gtk_label_new( NULL );
         hig_workarea_add_row( t, &row, _("Duration:"), l, NULL );
     hig_workarea_add_section_divider( t, &row );
-    hig_workarea_add_section_title( t, &row, _("Cumulative") );
-    hig_workarea_add_section_spacer( t, row, 5 );
-        l = ui->all_sessions_lb = gtk_label_new( _("Program started %d times") );
+    hig_workarea_add_section_title( t, &row, _("Total") );
+        l = ui->all_sessions_lb = gtk_label_new( _("Started %'d time") );
         hig_workarea_add_label_w( t, row++, l );
         l = ui->all_up_lb = gtk_label_new( NULL );
         hig_workarea_add_row( t, &row, _("Uploaded:"), l, NULL );
@@ -124,8 +152,8 @@ stats_dialog_create( GtkWindow * parent, TrCore * core )
 
     updateStats( ui );
     g_object_set_data_full( G_OBJECT(d), "data", ui, g_free );
-    g_signal_connect( d, "response", G_CALLBACK(dialogResponse), NULL );
+    g_signal_connect( d, "response", G_CALLBACK(dialogResponse), ui );
     i = g_timeout_add( 1000, updateStats, ui );
-    g_object_set_data( G_OBJECT(d), "TrTimer", GUINT_TO_POINTER(i) );
+    g_object_weak_ref( G_OBJECT( d ), dialogDestroyed, GUINT_TO_POINTER( i ) );
     return d;
 }
