@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  * 
- * $Id: torrent-cell-renderer.c 6184 2008-06-13 21:19:04Z charles $
+ * $Id: torrent-cell-renderer.c 6758 2008-09-16 16:49:10Z charles $
  */
 
 #include "assert.h"
@@ -84,7 +84,7 @@ getProgressString( const tr_info * info, const tr_stat * torStat )
         GString * gstr = g_string_new( str );
         g_string_append( gstr, " - " );
         if( eta == TR_ETA_NOT_AVAIL )
-            g_string_append( gstr, _( "Not available" ) );
+            g_string_append( gstr, _( "Data not fully available" ) );
          else if( eta == TR_ETA_UNKNOWN )
             g_string_append( gstr, _( "Stalled" ) );
         else {
@@ -199,8 +199,8 @@ getStatusString( const tr_stat * torStat )
                 ngettext( "Downloading from %1$'d of %2$'d connected peer",
                           "Downloading from %1$'d of %2$'d connected peers",
                           torStat->peersConnected ),
-                torStat->peersSendingToUs,
-                torStat->peersConnected );
+                torStat->peersSendingToUs + torStat->webseedsSendingToUs,
+                torStat->peersConnected + torStat->webseedsSendingToUs );
             break;
 
         case TR_STATUS_SEED:
@@ -453,7 +453,8 @@ torrent_cell_renderer_render( GtkCellRenderer      * cell,
         my_cell.height = p->bar_height;
         if( 1 )
         {
-            g_object_set( p->progress_renderer, "value", (int)(torStat->percentDone*100.0), 
+            const double percent = MAX( 0.0, torStat->percentDone );
+            g_object_set( p->progress_renderer, "value", (int)(percent*100.0), 
                                                 "text", "",
                                                 NULL );
             gtk_cell_renderer_render( p->progress_renderer,
