@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: util.h 6182 2008-06-13 21:12:49Z charles $
+ * $Id: util.h 6402 2008-07-25 14:56:28Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -29,10 +29,9 @@
 #include <stdarg.h>
 
 /* macro to shut up "unused parameter" warnings */
+#ifndef UNUSED
 #define UNUSED G_GNUC_UNUSED
-
-/* NULL-safe version of strcmp */
-int tr_strcmp( const char*, const char * );
+#endif
 
 /* return number of items in array */
 #define ALEN(a) ((signed)G_N_ELEMENTS(a))
@@ -52,8 +51,7 @@ char* tr_strlratio( char * buf, double ratio, size_t buflen );
 /* return a human-readable string for the time given in seconds. */
 char* tr_strltime( char * buf, int secs, size_t buflen );
 
-char *
-rfc822date (guint64 epoch_msec);
+char* gtr_localtime( time_t time );
 
 /* create a directory and any missing parent directories */
 gboolean
@@ -79,19 +77,21 @@ decode_uri( const char * uri );
 GSList *
 checkfilenames( int argc, char ** argv );
 
-/* retrieve the global download directory */
-char *
-getdownloaddir( void );
-
 void gtr_open_file( const char * path );
 
-guint gtr_inhibit_hibernation( void );
-
-void gtr_uninhibit_hibernation( guint );
+gboolean gtr_dbus_add_torrent( const char * filename );
 
 char* gtr_get_help_url( void );
+
 #ifdef GTK_MAJOR_VERSION
 
+GtkWidget * tr_button_new_from_stock( const char * stock,
+                                      const char * mnemonic );
+
+void addTorrentErrorDialog( GtkWidget  * window_or_child,
+                            int          err,
+                            const char * filename );
+
 /* create an error dialog, if wind is NULL or mapped then show dialog now,
    otherwise show it when wind becomes mapped */
 void
