@@ -1,29 +1,17 @@
-/******************************************************************************
- * $Id: bencode.c 5772 2008-05-08 03:25:21Z charles $
+/*
+ * This file Copyright (C) 2008 Charles Kerr <charles@rebelbase.com>
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * This file is licensed by the GPL version 2.  Works owned by the
+ * Transmission project are granted a special exemption to clause 2(b)
+ * so that the bulk of its code can remain under the MIT license. 
+ * This exemption does not extend to derived works not owned by
+ * the Transmission project.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- *****************************************************************************/
+ * $Id: bencode.c 6592 2008-08-20 01:40:27Z charles $
+ */
 
 #include <assert.h>
-#include <ctype.h> /* isdigit, isprint */
+#include <ctype.h> /* isdigit, isprint, isspace */
 #include <errno.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -31,8 +19,11 @@
 
 #include <event.h> /* evbuffer */
 
+#include "ConvertUTF.h"
+
 #include "transmission.h"
 #include "bencode.h"
+#include "json.h"
 #include "list.h"
 #include "ptrarray.h"
 #include "utils.h" /* tr_new(), tr_free() */
@@ -44,7 +35,7 @@
 int
 tr_bencIsType( const tr_benc * val, int type )
 {
-    return ( ( val != NULL ) && ( val->type == type ) );
+    return ( ( val ) && ( val->type == type ) );
 }
 
 static int
@@ -58,6 +49,14 @@ isSomething( const tr_benc * val )
     return isContainer(val) || tr_bencIsInt(val) || tr_bencIsString(val);
 }
 
+static void
+tr_bencInit( tr_benc * val, int type )
+{
+    memset( val, 0, sizeof( *val ) );
+    val->type = type;
+}
+
+
 /***
 ****  tr_bencParse()
 ****  tr_bencLoad()
@@ -179,14 +178,14 @@ getNode( tr_benc * top, tr_ptrArray * parentStack, int type )
 {
     tr_benc * parent;
 
-    assert( top != NULL );
-    assert( parentStack != NULL );
+    assert( top );
+    assert( parentStack );
 
     if( tr_ptrArrayEmpty( parentStack ) )
         return top;
 
     parent = tr_ptrArrayBack( parentStack );
-    assert( parent != NULL );
+    assert( parent );
 
     /* dictionary keys must be strings */
     if( ( parent->type == TYPE_DICT )
@@ -203,16 +202,16 @@ getNode( tr_benc * top, tr_ptrArray * parentStack, int type )
  * easier to read, but was vulnerable to a smash-stacking
  * attack via maliciously-crafted bencoded data. (#667)
  */
-int
-tr_bencParse( const void     * buf_in,
-              const void     * bufend_in,
-              tr_benc        * top,
-              const uint8_t ** setme_end )
+static int
+tr_bencParseImpl( const void     * buf_in,
+                  const void     * bufend_in,
+                  tr_benc        * top,
+                  tr_ptrArray    * parentStack,
+                  const uint8_t ** setme_end )
 {
     int err;
     const uint8_t * buf = buf_in;
     const uint8_t * bufend = bufend_in;
-    tr_ptrArray * parentStack = tr_ptrArrayNew( );
 
     tr_bencInit( top, 0 );
 
@@ -267,8 +266,11 @@ tr_bencParse( const void     * buf_in,
                 return TR_ERROR;
 
             node = tr_ptrArrayBack( parentStack );
-            if( tr_bencIsDict( node ) && ( node->val.l.count % 2 ) )
-                return TR_ERROR; /* odd # of children in dict */
+            if( tr_bencIsDict( node ) && ( node->val.l.count % 2 ) ) {
+                /* odd # of children in dict */
+                tr_bencFree( &node->val.l.vals [ --node->val.l.count ] );
+                return TR_ERROR;
+            }
 
             tr_ptrArrayPop( parentStack );
             if( tr_ptrArrayEmpty( parentStack ) )
@@ -286,8 +288,10 @@ tr_bencParse( const void     * buf_in,
                 return err;
 
             node = getNode( top, parentStack, TYPE_STR );
-            if( !node )
+            if( !node ) {
+                tr_free( str );
                 return TR_ERROR;
+            }
 
             tr_bencInitStr( node, str, str_len, 0 );
             buf = end;
@@ -303,9 +307,26 @@ tr_bencParse( const void     * buf_in,
 
     err = !isSomething( top ) || !tr_ptrArrayEmpty( parentStack );
 
-    if( !err && ( setme_end != NULL ) )
+    if( !err && setme_end )
         *setme_end = buf;
 
+    return err;
+}
+
+int
+tr_bencParse( const void     * buf,
+              const void     * end,
+              tr_benc        * top,
+              const uint8_t ** setme_end )
+{
+    int err;
+    tr_ptrArray * parentStack = tr_ptrArrayNew( );
+
+    top->type = 0; /* not initialized yet */
+    err = tr_bencParseImpl( buf, end, top, parentStack, setme_end );
+    if( err )
+        tr_bencFree( top ); 
+
     tr_ptrArrayFree( parentStack, NULL );
     return err;
 }
@@ -328,13 +349,13 @@ tr_bencLoad( const void  * buf_in,
 ****
 ***/
 
-tr_benc *
-tr_bencDictFind( tr_benc * val, const char * key )
+static int
+dictIndexOf( tr_benc * val, const char * key )
 {
     int len, ii;
 
     if( !tr_bencIsDict( val ) )
-        return NULL;
+        return -1;
 
     len = strlen( key );
     
@@ -346,10 +367,17 @@ tr_bencDictFind( tr_benc * val, const char * key )
         {
             continue;
         }
-        return &val->val.l.vals[ii+1];
+        return ii;
     }
 
-    return NULL;
+    return -1;
+}
+
+tr_benc *
+tr_bencDictFind( tr_benc * val, const char * key )
+{
+    const int i = dictIndexOf( val, key );
+    return i<0 ? NULL : &val->val.l.vals[i+1];
 }
 
 tr_benc*
@@ -465,6 +493,23 @@ tr_bencDictFindStr( tr_benc * dict, const char * key, const char ** setme )
     return found;
 }
 
+int
+tr_bencDictFindRaw( tr_benc         * dict,
+                    const char      * key,
+                    const uint8_t  ** setme_raw, 
+                    size_t          * setme_len )
+{
+    int found = FALSE;
+    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_STR );
+    if( child ) {
+        *setme_raw = (uint8_t*) child->val.s.s;
+        *setme_len = child->val.s.i;
+        found = TRUE;
+    }
+    return found;
+}
+
+
 /***
 ****
 ***/
@@ -487,9 +532,8 @@ tr_bencInitRaw( tr_benc * val, const void * src, size_t byteCount )
 {
     tr_bencInit( val, TYPE_STR );
     val->val.s.i = byteCount;
-    val->val.s.s = tr_new( char, byteCount );
+    val->val.s.s = tr_memdup( src, byteCount );
     val->val.s.nofree = 0;
-    memcpy( val->val.s.s, src, byteCount );
 }
 
 int
@@ -591,10 +635,12 @@ tr_bencDictAdd( tr_benc * dict, const char * key )
     tr_benc * keyval, * itemval;
 
     assert( tr_bencIsDict( dict ) );
+    if( dict->val.l.count + 2 > dict->val.l.alloc )
+        makeroom( dict, 2 );
     assert( dict->val.l.count + 2 <= dict->val.l.alloc );
 
     keyval = dict->val.l.vals + dict->val.l.count++;
-    tr_bencInitStr( keyval, (char*)key, -1, 1 );
+    tr_bencInitStrDup( keyval, key );
 
     itemval = dict->val.l.vals + dict->val.l.count++;
     tr_bencInit( itemval, TYPE_INT );
@@ -619,7 +665,7 @@ tr_benc*
 tr_bencDictAddDouble( tr_benc * dict, const char * key, double d )
 {
     char buf[128];
-    snprintf( buf, sizeof( buf ), "%f", d );
+    tr_snprintf( buf, sizeof( buf ), "%f", d );
     return tr_bencDictAddStr( dict, key, buf );
 }
 tr_benc*
@@ -644,6 +690,26 @@ tr_bencDictAddRaw( tr_benc * dict, const char * key, const void * src, size_t le
     return child;
 }
 
+int
+tr_bencDictRemove( tr_benc * dict, const char * key )
+{
+    int i = dictIndexOf( dict, key );
+    if( i >= 0 )
+    {
+        const int n = dict->val.l.count;
+        tr_bencFree( &dict->val.l.vals[i] );
+        tr_bencFree( &dict->val.l.vals[i+1] );
+        if( i + 2 < n )
+        {
+            dict->val.l.vals[i]   = dict->val.l.vals[n-2];
+            dict->val.l.vals[i+1] = dict->val.l.vals[n-1];
+        }
+        dict->val.l.count -= 2;
+    }
+    return i >= 0; /* return true if found */
+}
+
+
 /***
 ****  BENC WALKING
 ***/
@@ -767,7 +833,7 @@ struct WalkFuncs
  * attack via maliciously-crafted bencoded data. (#667)
  */
 static void
-bencWalk( const tr_benc   * top,
+bencWalk( const tr_benc      * top,
           struct WalkFuncs   * walkFuncs,
           void               * user_data )
 {
@@ -799,7 +865,7 @@ bencWalk( const tr_benc   * top,
             continue;
         }
 
-        switch( val->type )
+        if( val ) switch( val->type )
         {
             case TYPE_INT:
                 walkFuncs->intFunc( val, user_data );
@@ -878,7 +944,7 @@ tr_bencSave( const tr_benc * top, int * len )
     walkFuncs.containerEndFunc = saveContainerEndFunc;
     bencWalk( top, &walkFuncs, out );
     
-    if( len != NULL )
+    if( len )
         *len = EVBUFFER_LENGTH( out );
     ret = tr_strndup( (char*) EVBUFFER_DATA( out ), EVBUFFER_LENGTH( out ) );
     evbuffer_free( out );
@@ -907,7 +973,7 @@ freeContainerBeginFunc( const tr_benc * val, void * freeme )
 void
 tr_bencFree( tr_benc * val )
 {
-    if( val != NULL )
+    if( val && val->type )
     {
         tr_ptrArray * freeme = tr_ptrArrayNew( );
         struct WalkFuncs walkFuncs;
@@ -1010,99 +1076,183 @@ tr_bencPrint( const tr_benc * val )
 
 struct ParentState
 {
-    int type;
-    int index;
+    int bencType;
+    int childIndex;
+    int childCount;
 };
  
-struct phpWalk
+struct jsonWalk
 {
     tr_list * parents;
     struct evbuffer * out;
 };
 
 static void
-phpChildFunc( struct phpWalk * data )
+jsonIndent( struct jsonWalk * data )
+{
+    const int width = tr_list_size( data->parents ) * 4;
+    evbuffer_add_printf( data->out, "\n%*.*s", width, width, " " );
+}
+
+static void
+jsonChildFunc( struct jsonWalk * data )
 {
     if( data->parents )
     {
         struct ParentState * parentState = data->parents->data;
 
-        if( parentState->type == TYPE_LIST )
-            evbuffer_add_printf( data->out, "i:%d;", parentState->index++ );
+        switch( parentState->bencType )
+        {
+            case TYPE_DICT: {
+                const int i = parentState->childIndex++;
+                if( ! ( i % 2 ) )
+                    evbuffer_add_printf( data->out, ": " );
+                else {
+                    evbuffer_add_printf( data->out, ", " );
+                    jsonIndent( data );
+                }
+                break;
+            }
+
+            case TYPE_LIST: {
+                ++parentState->childIndex;
+                evbuffer_add_printf( data->out, ", " );
+                jsonIndent( data );
+                break;
+            }
+
+            default:
+                break;
+        }
     }
 }
 
 static void
-phpPushParent( struct phpWalk * data, int type )
+jsonPushParent( struct jsonWalk * data, const tr_benc * benc )
 {
     struct ParentState * parentState = tr_new( struct ParentState, 1 );
-    parentState->type = type;
-    parentState->index = 0;
+    parentState->bencType = benc->type;
+    parentState->childIndex = 0;
+    parentState->childCount = benc->val.l.count;
     tr_list_prepend( &data->parents, parentState );
 }
 
 static void
-phpPopParent( struct phpWalk * data )
+jsonPopParent( struct jsonWalk * data )
 {
     tr_free( tr_list_pop_front( &data->parents ) );
 }
 
 static void
-phpIntFunc( const tr_benc * val, void * vdata )
+jsonIntFunc( const tr_benc * val, void * vdata )
 {
-    struct phpWalk * data = vdata;
-    phpChildFunc( data );
-    evbuffer_add_printf( data->out, "i:%"PRId64";", val->val.i );
+    struct jsonWalk * data = vdata;
+    evbuffer_add_printf( data->out, "%"PRId64, val->val.i );
+    jsonChildFunc( data );
 }
 static void
-phpStringFunc( const tr_benc * val, void * vdata )
+jsonStringFunc( const tr_benc * val, void * vdata )
 {
-    struct phpWalk * data = vdata;
-    phpChildFunc( data );
-    evbuffer_add_printf( data->out, "s:%d:\"%s\";", val->val.s.i, val->val.s.s );
+    struct jsonWalk * data = vdata;
+    const unsigned char *it, *end;
+    evbuffer_add_printf( data->out, "\"" );
+    for( it=(const unsigned char*)val->val.s.s, end=it+val->val.s.i; it!=end; ++it )
+    {
+        switch( *it ) {
+            case '/' : evbuffer_add_printf( data->out, "\\/" ); break;
+            case '\b': evbuffer_add_printf( data->out, "\\b" ); break;
+            case '\f': evbuffer_add_printf( data->out, "\\f" ); break;
+            case '\n': evbuffer_add_printf( data->out, "\\n" ); break;
+            case '\r': evbuffer_add_printf( data->out, "\\r" ); break;
+            case '\t': evbuffer_add_printf( data->out, "\\t" ); break;
+            case '"' : evbuffer_add_printf( data->out, "\\\"" ); break;
+            case '\\': evbuffer_add_printf( data->out, "\\\\" ); break;
+            default:
+                if( isascii( *it ) ) {
+                    /*fprintf( stderr, "[%c]\n", *it );*/
+                    evbuffer_add_printf( data->out, "%c", *it );
+                } else {
+                    const UTF8 * tmp = it;
+                    UTF32 buf = 0;
+                    UTF32 * u32 = &buf;
+                    ConvertUTF8toUTF32( &tmp, end, &u32, &buf+1, 0 );
+                    evbuffer_add_printf( data->out, "\\u%04x", buf );
+                    it = tmp - 1;
+                    /*fprintf( stderr, "[\\u%04x]\n", buf );*/
+                }
+        }
+    }
+    evbuffer_add_printf( data->out, "\"" );
+    jsonChildFunc( data );
 }
 static void
-phpDictBeginFunc( const tr_benc * val, void * vdata )
+jsonDictBeginFunc( const tr_benc * val, void * vdata )
 {
-    struct phpWalk * data = vdata;
-    phpChildFunc( data );
-    phpPushParent( data, TYPE_DICT );
-    evbuffer_add_printf( data->out, "a:%d:{", val->val.l.count/2 );
+    struct jsonWalk * data = vdata;
+    jsonPushParent( data, val );
+    evbuffer_add_printf( data->out, "{" );
+    if( val->val.l.count )
+        jsonIndent( data );
 }
 static void
-phpListBeginFunc( const tr_benc * val, void * vdata )
+jsonListBeginFunc( const tr_benc * val, void * vdata )
 {
-    struct phpWalk * data = vdata;
-    phpChildFunc( data );
-    phpPushParent( data, TYPE_LIST );
-    evbuffer_add_printf( data->out, "a:%d:{", val->val.l.count );
+    const int nChildren = tr_bencListSize( val );
+    struct jsonWalk * data = vdata;
+    jsonPushParent( data, val );
+    evbuffer_add_printf( data->out, "[" );
+    if( nChildren )
+        jsonIndent( data );
 }
 static void
-phpContainerEndFunc( const tr_benc * val UNUSED, void * vdata )
+jsonContainerEndFunc( const tr_benc * val, void * vdata )
 {
-    struct phpWalk * data = vdata;
-    phpPopParent( data );
-    evbuffer_add_printf( data->out, "}" );
+    size_t i;
+    struct jsonWalk * data = vdata;
+    char * str;
+    int emptyContainer = FALSE;
+
+    /* trim out the trailing comma, if any */
+    str = (char*) EVBUFFER_DATA( data->out );
+    for( i=EVBUFFER_LENGTH( data->out )-1; i>0; --i ) {
+        if( isspace( str[i] ) ) continue;
+        if( str[i]==',' )
+            EVBUFFER_LENGTH( data->out ) = i;
+        if( str[i]=='{' || str[i]=='[' )
+            emptyContainer = TRUE;
+        break;
+    }
+
+    jsonPopParent( data );
+    if( !emptyContainer )
+        jsonIndent( data );
+    if( tr_bencIsDict( val ) )
+        evbuffer_add_printf( data->out, "}" );
+    else /* list */
+        evbuffer_add_printf( data->out, "]" );
+    jsonChildFunc( data );
 }
 char*
-tr_bencSaveAsSerializedPHP( const tr_benc * top, int * len )
+tr_bencSaveAsJSON( const tr_benc * top, int * len )
 {
     char * ret;
     struct WalkFuncs walkFuncs;
-    struct phpWalk data;
+    struct jsonWalk data;
 
     data.out = evbuffer_new( );
     data.parents = NULL;
 
-    walkFuncs.intFunc = phpIntFunc;
-    walkFuncs.stringFunc = phpStringFunc;
-    walkFuncs.dictBeginFunc = phpDictBeginFunc;
-    walkFuncs.listBeginFunc = phpListBeginFunc;
-    walkFuncs.containerEndFunc = phpContainerEndFunc;
+    walkFuncs.intFunc = jsonIntFunc;
+    walkFuncs.stringFunc = jsonStringFunc;
+    walkFuncs.dictBeginFunc = jsonDictBeginFunc;
+    walkFuncs.listBeginFunc = jsonListBeginFunc;
+    walkFuncs.containerEndFunc = jsonContainerEndFunc;
 
     bencWalk( top, &walkFuncs, &data );
     
-    if( len != NULL )
+    if( EVBUFFER_LENGTH( data.out ) )
+        evbuffer_add_printf( data.out, "\n" );
+    if( len )
         *len = EVBUFFER_LENGTH( data.out );
     ret = tr_strndup( (char*) EVBUFFER_DATA( data.out ), EVBUFFER_LENGTH( data.out ) );
     evbuffer_free( data.out );
@@ -1113,12 +1263,10 @@ tr_bencSaveAsSerializedPHP( const tr_benc * top, int * len )
 ****
 ***/
 
-int
-tr_bencSaveFile( const char * filename,  const tr_benc * b )
+static int
+saveFile( const char * filename, const char * content, size_t len )
 {
     int err = TR_OK;
-    int len;
-    char * content = tr_bencSave( b, &len );
     FILE * out = NULL;
 
     out = fopen( filename, "wb+" );
@@ -1137,12 +1285,21 @@ tr_bencSaveFile( const char * filename,  const tr_benc * b )
 
     if( !err )
         tr_dbg( "tr_bencSaveFile saved \"%s\"", filename );
-    tr_free( content );
     if( out )
         fclose( out );
     return err;
 }
 
+int
+tr_bencSaveFile( const char * filename,  const tr_benc * b )
+{
+    int len;
+    char * content = tr_bencSave( b, &len );
+    const int err = saveFile( filename, content, len );
+    tr_free( content );
+    return err;
+}
+
 int
 tr_bencLoadFile( const char * filename, tr_benc * b )
 {
@@ -1154,3 +1311,25 @@ tr_bencLoadFile( const char * filename, tr_benc * b )
     tr_free( content );
     return ret;
 }
+
+int
+tr_bencSaveJSONFile( const char * filename, const tr_benc * b )
+{
+    int len;
+    char * content = tr_bencSaveAsJSON( b, &len );
+    const int err = saveFile( filename, content, len );
+    tr_free( content );
+    return err;
+}
+
+int
+tr_bencLoadJSONFile( const char * filename, tr_benc * b )
+{
+    int ret;
+    size_t contentLen;
+    uint8_t * content = tr_loadFile( filename, &contentLen );
+    ret = content ? tr_jsonParse( content, contentLen, b, NULL )
+                  : TR_ERROR_IO_OTHER;
+    tr_free( content );
+    return ret;
+}
