@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: clients.h 4404 2008-01-01 17:20:20Z charles $
+ * $Id: clients.h 5723 2008-04-29 16:57:16Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -22,4 +22,6 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
-char * tr_clientForId( const uint8_t * );
+void tr_clientForId( char       * buf,
+                     size_t       buflen,
+                     const void * peer_id );
