@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: completion.h 5682 2008-04-24 15:25:01Z charles $
+ * $Id: completion.h 6092 2008-06-09 00:59:33Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -55,7 +55,8 @@ void             tr_cpPieceRem( tr_completion *, tr_piece_index_t piece );
 int              tr_cpBlockIsComplete( const tr_completion *, tr_block_index_t block );
 void             tr_cpBlockAdd( tr_completion *, tr_block_index_t block );
 tr_errno         tr_cpBlockBitfieldSet( tr_completion *, struct tr_bitfield * );
-int              tr_cpMissingBlocksInPiece( const tr_completion * cp, tr_piece_index_t piece );
+int              tr_cpMissingBlocksInPiece( const tr_completion * cp,
+                                            tr_piece_index_t piece );
 
 
 const struct tr_bitfield * tr_cpPieceBitfield( const tr_completion* );
