@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: crypto.c 4597 2008-01-10 18:52:46Z charles $
+ * $Id: crypto.c 6479 2008-08-11 04:40:28Z charles $
  */
 
 #include <sys/types.h> /* for event.h, as well as netinet/in.h on some platforms */
@@ -50,6 +50,7 @@ tr_sha1( uint8_t    * setme,
             break;
         SHA1_Update( &sha, content, content_len );
     }
+    va_end( vl );
     SHA1_Final( setme, &sha );
 }
 
@@ -241,7 +242,7 @@ tr_cryptoSetTorrentHash( tr_crypto     * crypto,
 {
     crypto->torrentHashIsSet = hash ? 1 : 0;
 
-    if( hash != NULL )
+    if( hash )
         memcpy( crypto->torrentHash, hash, SHA_DIGEST_LENGTH );
     else
         memset( crypto->torrentHash, 0, SHA_DIGEST_LENGTH );
@@ -250,7 +251,7 @@ tr_cryptoSetTorrentHash( tr_crypto     * crypto,
 const uint8_t*
 tr_cryptoGetTorrentHash( const tr_crypto * crypto )
 {
-    assert( crypto != NULL );
+    assert( crypto );
     assert( crypto->torrentHashIsSet );
 
     return crypto->torrentHash;
@@ -259,7 +260,7 @@ tr_cryptoGetTorrentHash( const tr_crypto * crypto )
 int
 tr_cryptoHasTorrentHash( const tr_crypto * crypto )
 {
-    assert( crypto != NULL );
+    assert( crypto );
 
     return crypto->torrentHashIsSet ? 1 : 0;
 }
