@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: fdlimit.c 4855 2008-01-28 21:05:48Z charles $
+ * $Id: fdlimit.c 6489 2008-08-11 19:05:00Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -40,17 +40,17 @@
 #include <sys/resource.h> /* getrlimit */
 #endif
 #include <unistd.h>
-#include <libgen.h> /* basename, dirname */
+#include <libgen.h> /* dirname */
 #include <fcntl.h> /* O_LARGEFILE */
 
 #include <event.h>
 #include <evutil.h>
 
 #include "transmission.h"
-#include "trcompat.h"
+#include "fdlimit.h"
 #include "list.h"
 #include "net.h"
-#include "platform.h"
+#include "platform.h" /* tr_lock */
 #include "utils.h"
 
 #if SIZEOF_VOIDP==8
@@ -59,34 +59,7 @@
 #define TR_UINT_TO_PTR(i) ((void*)((uint32_t)i))
 #endif
 
-/**
-***
-**/
-
-static void
-myDebug( const char * file, int line, const char * fmt, ... )
-{
-    FILE * fp = tr_getLog( );
-    if( fp != NULL )
-    {
-        va_list args;
-        char s[64];
-        struct evbuffer * buf = evbuffer_new( );
-        char * myfile = tr_strdup( file );
-
-        evbuffer_add_printf( buf, "[%s] ", tr_getLogTimeStr( s, sizeof(s) ) );
-        va_start( args, fmt );
-        evbuffer_add_vprintf( buf, fmt, args );
-        va_end( args );
-        evbuffer_add_printf( buf, " (%s:%d)\n", basename(myfile), line );
-        fwrite( EVBUFFER_DATA(buf), 1, EVBUFFER_LENGTH(buf), fp );
-
-        tr_free( myfile );
-        evbuffer_free( buf );
-    }
-}
-
-#define dbgmsg(fmt...) myDebug(__FILE__, __LINE__, ##fmt )
+#define dbgmsg(fmt...) tr_deepLog( __FILE__, __LINE__, NULL, ##fmt )
 
 /**
 ***
@@ -127,19 +100,29 @@ static struct tr_fd_s * gFd = NULL;
 ****
 ***/
 
-static int
-TrOpenFile( int i, const char * filename, int write )
+static tr_errno
+TrOpenFile( int           i,
+            const char  * folder,
+            const char  * torrentFile,
+            int           write )
 {
     struct tr_openfile * file = &gFd->open[i];
     int flags;
+    char filename[MAX_PATH_LENGTH];
+    struct stat sb;
+
+    /* confirm the parent folder exists */
+    if( stat( folder, &sb ) || !S_ISDIR( sb.st_mode ) )
+        return TR_ERROR_IO_PARENT;
 
     /* create subfolders, if any */
+    tr_buildPath ( filename, sizeof(filename), folder, torrentFile, NULL );
     if( write ) {
         char * tmp = tr_strdup( filename );
-        const int err = tr_mkdirp( dirname(tmp), 0777 );
+        const int err = tr_mkdirp( dirname(tmp), 0777 ) ? errno : 0;
         tr_free( tmp );
         if( err )
-            return tr_ioErrorFromErrno( );
+            return tr_ioErrorFromErrno( err );
     }
 
     /* open the file */
@@ -153,8 +136,8 @@ TrOpenFile( int i, const char * filename, int write )
     file->fd = open( filename, flags, 0666 );
     if( file->fd == -1 ) {
         const int err = errno;
-        tr_err( "Couldn't open '%s': %s", filename, strerror(err) );
-        return TR_ERROR_IO_OTHER;
+        tr_err( _( "Couldn't open \"%1$s\": %2$s" ), filename, tr_strerror(err) );
+        return tr_ioErrorFromErrno( err );
     }
 
     return TR_OK;
@@ -187,14 +170,19 @@ fileIsCheckedOut( const struct tr_openfile * o )
 }
 
 int
-tr_fdFileCheckout( const char * filename, int write )
+tr_fdFileCheckout( const char * folder,
+                   const char * torrentFile, 
+                   int          write )
 {
     int i, winner = -1;
     struct tr_openfile * o;
+    char filename[MAX_PATH_LENGTH];
 
-    assert( filename && *filename );
+    assert( folder && *folder );
+    assert( torrentFile && *torrentFile );
     assert( write==0 || write==1 );
 
+    tr_buildPath ( filename, sizeof(filename), folder, torrentFile, NULL );
     dbgmsg( "looking for file '%s', writable %c", filename, write?'y':'n' );
 
     tr_lockLock( gFd->lock );
@@ -269,14 +257,14 @@ tr_fdFileCheckout( const char * filename, int write )
     o = &gFd->open[winner];
     if( !fileIsOpen( o ) )
     {
-        const int err = TrOpenFile( winner, filename, write );
+        const tr_errno err = TrOpenFile( winner, folder, torrentFile, write );
         if( err ) {
             tr_lockUnlock( gFd->lock );
             return err;
         }
 
         dbgmsg( "opened '%s' in slot %d, write %c", filename, winner, write?'y':'n' );
-        strlcpy( o->filename, filename, sizeof( o->filename ) );
+        tr_strlcpy( o->filename, filename, sizeof( o->filename ) );
         o->isWritable = write;
     }
 
@@ -372,7 +360,7 @@ tr_fdSocketCreate( int type, int isReserved )
 
     if( isReserved || ( gFd->normal < getSocketMax( gFd ) ) )
         if( ( s = socket( AF_INET, type, 0 ) ) < 0 )
-            tr_err( "Couldn't create socket (%s)", strerror( sockerrno ) );
+            tr_err( _( "Couldn't create socket: %s" ), tr_strerror( sockerrno ) );
 
     if( s > -1 )
     {
@@ -398,8 +386,8 @@ tr_fdSocketAccept( int b, struct in_addr * addr, tr_port_t * port )
     unsigned int len;
     struct sockaddr_in sock;
 
-    assert( addr != NULL );
-    assert( port != NULL );
+    assert( addr );
+    assert( port );
 
     tr_lockLock( gFd->lock );
     if( gFd->normal < getSocketMax( gFd ) )
@@ -471,7 +459,7 @@ tr_fdInit( int globalPeerLimit )
                (rlim_t)(globalPeerLimit + NOFILE_BUFFER) );
         setrlimit( RLIMIT_NOFILE, &rlim );
         gFd->normalMax = rlim.rlim_cur - NOFILE_BUFFER;
-        tr_dbg( "setrlimit( RLIMIT_NOFILE, %d )", rlim.rlim_cur );
+        tr_dbg( "setrlimit( RLIMIT_NOFILE, %d )", (int)rlim.rlim_cur );
     }
 #else
     gFd->normalMax = globalPeerLimit;
