@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: makemeta.h 5358 2008-03-24 17:18:08Z charles $
+ * $Id: makemeta.h 5989 2008-06-01 05:36:23Z charles $
  */
 
 #ifndef TR_MAKEMETA_H
@@ -54,7 +54,8 @@ typedef struct tr_metainfo_builder
     ***  and cleaned up by tr_metaInfoBuilderFree()
     **/
 
-    char * announce;
+    tr_tracker_info  * trackers;
+    int trackerCount;
     char * comment;
     char * outputFile;
     int isPrivate;
@@ -95,7 +96,7 @@ void
 tr_metaInfoBuilderFree( tr_metainfo_builder* );
 
 /**
- * 'outputFile' if NULL, builder->top + ".torrent" will be used.
+ * @brief create a new .torrent file
  *
  * This is actually done in a worker thread, not the main thread!
  * Otherwise the client's interface would lock up while this runs.
@@ -103,13 +104,21 @@ tr_metaInfoBuilderFree( tr_metainfo_builder* );
  * It is the caller's responsibility to poll builder->isDone
  * from time to time!  When the worker thread sets that flag,
  * the caller must pass the builder to tr_metaInfoBuilderFree().
+ *
+ * @param outputFile if NULL, builder->top + ".torrent" will be used.
+
+ * @param trackers An array of trackers, sorted by tier from first to last.
+ *                 NOTE: only the `tier' and `announce' fields are used.
+ *
+ * @param trackerCount size of the `trackers' array
  */
 void
-tr_makeMetaInfo( tr_metainfo_builder  * builder,
-                 const char           * outputFile,
-                 const char           * announce,
-                 const char           * comment,
-                 int                    isPrivate );
+tr_makeMetaInfo( tr_metainfo_builder     * builder,
+                 const char              * outputFile,
+                 const tr_tracker_info   * trackers,
+                 int                       trackerCount,
+                 const char              * comment,
+                 int                       isPrivate );
 
 
 #endif
