@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: metainfo.c 5889 2008-05-21 20:03:31Z charles $
+ * $Id: metainfo.c 6503 2008-08-13 14:19:26Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -39,7 +39,6 @@
 #include "crypto.h" /* tr_sha1 */
 #include "metainfo.h"
 #include "platform.h"
-#include "trcompat.h" /* strlcpy */
 #include "utils.h"
 
 /***********************************************************************
@@ -132,7 +131,7 @@ getTorrentFilename( const tr_handle  * handle,
 {
     const char * dir = tr_getTorrentDir( handle );
     char base[MAX_PATH_LENGTH];
-    snprintf( base, sizeof( base ), "%s.%16.16s.torrent", inf->name, inf->hashString );
+    tr_snprintf( base, sizeof( base ), "%s.%16.16s.torrent", inf->name, inf->hashString );
     tr_buildPath( buf, buflen, dir, base, NULL );
 }
 
@@ -151,7 +150,7 @@ getTorrentOldFilename( const tr_handle * handle,
     else
     {
         char base[1024];
-        snprintf( base, sizeof(base), "%s-%s", info->hashString, handle->tag );
+        tr_snprintf( base, sizeof(base), "%s-%s", info->hashString, handle->tag );
         tr_buildPath( name, len, torDir, base, NULL );
     }
 }
@@ -224,6 +223,26 @@ announceToScrape( const char * announce )
     return scrape;
 }
 
+static void
+geturllist( tr_info * inf, tr_benc * meta )
+{
+    benc_val_t * urls;
+
+    if( tr_bencDictFindList( meta, "url-list", &urls ) )
+    {
+        int i;
+        const char * url;
+        const int n = tr_bencListSize( urls );
+
+        inf->webseedCount = 0;
+        inf->webseeds = tr_new0( char*, n );
+
+        for( i=0; i<n; ++i )
+            if( tr_bencGetStr( tr_bencListChild( urls, i ), &url ) )
+                inf->webseeds[inf->webseedCount++] = tr_strdup( url );
+    }
+}
+
 static int
 getannounce( tr_info * inf, tr_benc * meta )
 {
@@ -261,7 +280,7 @@ getannounce( tr_info * inf, tr_benc * meta )
 
         /* did we use any of the tiers? */
         if( !trackerCount ) {
-            tr_inf( _( "Invalid metadata entry \"%s\"" ), "announce-list" );
+            tr_nerr( inf->name, _( "Invalid metadata entry \"%s\"" ), "announce-list" );
             tr_free( trackers );
             trackers = NULL;
         }
@@ -282,6 +301,9 @@ getannounce( tr_info * inf, tr_benc * meta )
     inf->trackers = trackers;
     inf->trackerCount = trackerCount;
 
+    if( !inf->trackerCount )
+        tr_nerr( inf->name, _( "Invalid metadata entry \"%s\"" ), "announce" );
+
     return inf->trackerCount ? TR_OK : TR_ERROR;
 }
 
@@ -349,9 +371,9 @@ tr_metainfoParse( const tr_handle  * handle,
     if( !tr_bencIsInt( val ) )
     {
         if( val )
-            tr_err( _( "Invalid metadata entry \"%s\"" ), "piece length" );
+            tr_nerr( inf->name, _( "Invalid metadata entry \"%s\"" ), "piece length" );
         else
-            tr_err( _( "Missing metadata entry \"%s\"" ), "piece length" );
+            tr_nerr( inf->name, _( "Missing metadata entry \"%s\"" ), "piece length" );
         goto fail;
     }
     inf->pieceSize = val->val.i;
@@ -361,14 +383,14 @@ tr_metainfoParse( const tr_handle  * handle,
     if( !tr_bencIsString( val ) )
     {
         if( val )
-            tr_err( _( "Invalid metadata entry \"%s\"" ), "pieces" );
+            tr_nerr( inf->name, _( "Invalid metadata entry \"%s\"" ), "pieces" );
         else
-            tr_err( _( "Missing metadata entry \"%s\"" ), "pieces" );
+            tr_nerr( inf->name, _( "Missing metadata entry \"%s\"" ), "pieces" );
         goto fail;
     }
     if( val->val.s.i % SHA_DIGEST_LENGTH )
     {
-        tr_err( _( "Invalid metadata entry \"%s\"" ), "pieces" );
+        tr_nerr( inf->name, _( "Invalid metadata entry \"%s\"" ), "pieces" );
         goto fail;
     }
     inf->pieceCount = val->val.s.i / SHA_DIGEST_LENGTH;
@@ -382,17 +404,16 @@ tr_metainfoParse( const tr_handle  * handle,
 
     /* get file or top directory name */
     val = tr_bencDictFindFirst( beInfo, "name.utf-8", "name", NULL );
-    if( parseFiles( inf, tr_bencDictFindFirst( beInfo,
-                                               "name.utf-8", "name", NULL ),
+    if( parseFiles( inf, val,
                     tr_bencDictFind( beInfo, "files" ),
                     tr_bencDictFind( beInfo, "length" ) ) )
     {
         goto fail;
     }
 
-    if( !inf->fileCount || !inf->totalSize )
+    if( !inf->fileCount || !inf->totalSize || !inf->pieceSize )
     {
-        tr_err( _( "Torrent is corrupt" ) ); /* the content is missing! */
+        tr_nerr( inf->name, _( "Torrent is corrupt" ) ); /* the content is missing! */
         goto fail;
     }
 
@@ -401,7 +422,7 @@ tr_metainfoParse( const tr_handle  * handle,
     if( (uint64_t) inf->pieceCount !=
         ( inf->totalSize + inf->pieceSize - 1 ) / inf->pieceSize )
     {
-        tr_err( _( "Torrent is corrupt" ) ); /* size of hashes and files don't match */
+        tr_nerr( inf->name, _( "Torrent is corrupt" ) ); /* size of hashes and files don't match */
         goto fail;
     }
 
@@ -409,6 +430,9 @@ tr_metainfoParse( const tr_handle  * handle,
     if( getannounce( inf, meta ) )
         goto fail;
 
+    /* get the url-list */
+    geturllist( inf, meta );
+
     /* filename of Transmission's copy */
     getTorrentFilename( handle, inf, buf, sizeof( buf ) );
     tr_free( inf->torrent );
@@ -426,9 +450,13 @@ void tr_metainfoFree( tr_info * inf )
     tr_file_index_t ff;
     int i;
 
+    for( i=0; i<inf->webseedCount; ++i )
+        tr_free( inf->webseeds[i] );
+
     for( ff=0; ff<inf->fileCount; ++ff )
         tr_free( inf->files[ff].name );
 
+    tr_free( inf->webseeds );
     tr_free( inf->pieces );
     tr_free( inf->files );
     tr_free( inf->comment );
@@ -559,18 +587,18 @@ parseFiles( tr_info * inf, tr_benc * name,
             if( getfile( &inf->files[ii].name, inf->name, path ) )
             {
                 if( path )
-                    tr_err( _( "Invalid metadata entry \"%s\"" ), "path" );
+                    tr_nerr( inf->name, _( "Invalid metadata entry \"%s\"" ), "path" );
                 else
-                    tr_err( _( "Missing metadata entry \"%s\"" ), "path" );
+                    tr_nerr( inf->name, _( "Missing metadata entry \"%s\"" ), "path" );
                 return TR_EINVALID;
             }
             length = tr_bencDictFind( item, "length" );
             if( !tr_bencIsInt( length ) )
             {
                 if( length )
-                    tr_err( _( "Invalid metadata entry \"%s\"" ), "length" );
+                    tr_nerr( inf->name, _( "Invalid metadata entry \"%s\"" ), "length" );
                 else
-                    tr_err( _( "Missing metadata entry \"%s\"" ), "length" );
+                    tr_nerr( inf->name, _( "Missing metadata entry \"%s\"" ), "length" );
                 return TR_EINVALID;
             }
             inf->files[ii].length = length->val.i;
@@ -599,7 +627,7 @@ parseFiles( tr_info * inf, tr_benc * name,
     }
     else
     {
-        tr_err( _( "Invalid or missing metadata entries \"length\" and \"files\"" ) );
+        tr_nerr( inf->name, _( "Invalid or missing metadata entries \"length\" and \"files\"" ) );
     }
 
     return TR_OK;
