@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-mgr.h 5647 2008-04-19 15:07:59Z charles $
+ * $Id: peer-mgr.h 6469 2008-08-10 15:15:51Z charles $
  */
 
 #ifndef TR_PEER_MGR_H
@@ -59,10 +59,11 @@ void tr_peerMgrAddIncoming( tr_peerMgr      * manager,
                             uint16_t          port,
                             int               socket );
 
-tr_pex * tr_peerMgrCompactToPex( const void  * compact,
-                                 size_t        compactLen,
-                                 const char  * added_f,
-                                 size_t      * pexCount );
+tr_pex * tr_peerMgrCompactToPex( const void     * compact,
+                                 size_t           compactLen,
+                                 const uint8_t  * added_f,
+                                 size_t           added_f_len,
+                                 size_t         * setme_pex_count );
                              
 void tr_peerMgrAddPex( tr_peerMgr     * manager,
                        const uint8_t  * torrentHash,
@@ -105,6 +106,8 @@ void tr_peerMgrTorrentStats( const tr_peerMgr * manager,
                              const uint8_t    * torrentHash,
                              int              * setmePeersKnown,
                              int              * setmePeersConnected,
+                             int              * setmeSeedsConnected,
+                             int              * setmeWebseedsSendingToUs,
                              int              * setmePeersSendingToUs,
                              int              * setmePeersGettingFromUs,
                              int              * setmePeersFrom ); /* <-- array of TR_PEER_FROM__MAX */
@@ -113,6 +116,9 @@ struct tr_peer_stat * tr_peerMgrPeerStats( const tr_peerMgr  * manager,
                                            const uint8_t     * torrentHash,
                                            int               * setmeCount );
 
+float* tr_peerMgrWebSpeeds( const tr_peerMgr * manager,
+                            const uint8_t    * torrentHash );
+
                              
 struct tr_bitfield *
 tr_peerMgrGenerateAllowedSet( const uint32_t         setCount,
