@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2008 Charles Kerr <charles@rebelbase.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: ptrarray.c 4875 2008-01-31 02:24:42Z charles $
+ * $Id: ptrarray.c 6489 2008-08-11 19:05:00Z charles $
  */
 
 #include <assert.h>
@@ -45,10 +45,8 @@ tr_ptrArrayDup( tr_ptrArray* in )
     tr_ptrArray * out;
 
     out = tr_new( tr_ptrArray, 1 );
-    out->n_items = in->n_items;
-    out->n_alloc = in->n_items;
-    out->items = tr_new( void*, out->n_alloc );
-    memcpy( out->items, in->items, out->n_items * sizeof(void*) );
+    out->n_items = out->n_alloc = in->n_items;
+    out->items = tr_memdup( in->items, out->n_items * sizeof(void*) );
 
     return out;
 }
@@ -58,9 +56,9 @@ tr_ptrArrayForeach( tr_ptrArray * t, PtrArrayForeachFunc func )
 {
     int i;
 
-    assert( t != NULL );
-    assert( t->items != NULL );
-    assert( func != NULL );
+    assert( t );
+    assert( t->items );
+    assert( func );
 
     for( i=0; i<t->n_items; ++i )
         func( t->items[i] );
@@ -69,10 +67,10 @@ tr_ptrArrayForeach( tr_ptrArray * t, PtrArrayForeachFunc func )
 void
 tr_ptrArrayFree( tr_ptrArray * t, PtrArrayForeachFunc func )
 {
-    assert( t != NULL );
-    assert( t->items != NULL );
+    assert( t );
+    assert( t->items );
 
-    if( func != NULL )
+    if( func )
         tr_ptrArrayForeach( t, func );
 
     tr_free( t->items );
@@ -86,10 +84,16 @@ tr_ptrArrayPeek( tr_ptrArray * t, int * size )
     return t->items;
 }
 
+void**
+tr_ptrArrayBase( tr_ptrArray * t )
+{
+    return t->items;
+}
+
 void*
 tr_ptrArrayNth( tr_ptrArray* t, int i )
 {
-    assert( t != NULL  );
+    assert( t );
     assert( i >= 0 );
     assert( i < t->n_items );
 
@@ -178,7 +182,7 @@ tr_ptrArrayErase( tr_ptrArray * t, int begin, int end )
 ***
 **/
 
-int
+static int
 tr_ptrArrayLowerBound( const tr_ptrArray * t,
                        const void        * ptr,
                        int                 compare( const void *,const void * ),
