@@ -7,14 +7,14 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id:$
+ * $Id: torrent-ctor.c 6425 2008-08-01 16:43:22Z charles $
  */
 
 #include <libgen.h> /* basename */
 #include "transmission.h"
 #include "bencode.h"
 #include "platform.h"
-#include "trcompat.h" /* strlcpy */
+#include "session.h" /* tr_sessionFindTorrentFile */
 #include "utils.h"
 
 #define DEFAULT_MAX_CONNECTED_PEERS 50
@@ -23,20 +23,25 @@ struct optional_args
 {
     unsigned int isSet_paused : 1;
     unsigned int isSet_connected : 1;
-    unsigned int isSet_destination : 1;
+    unsigned int isSet_downloadDir : 1;
 
     unsigned int isPaused : 1;
-    uint16_t maxConnectedPeers;
-    char destination[MAX_PATH_LENGTH];
+    uint16_t peerLimit;
+    char downloadDir[MAX_PATH_LENGTH];
 };
 
+/** Opaque class used when instantiating torrents.
+  * @ingroup tr_ctor */
 struct tr_ctor
 {
     const tr_handle * handle;
     unsigned int saveInOurTorrentsDir : 1;
+    unsigned int doDelete : 1;
 
     unsigned int isSet_metainfo : 1;
-    benc_val_t metainfo;
+    unsigned int isSet_delete : 1;
+    tr_benc metainfo;
+    char * sourceFile;
 
     struct optional_args optionalArgs[2];
 };
@@ -45,6 +50,13 @@ struct tr_ctor
 ****
 ***/
 
+static void
+setSourceFile( tr_ctor * ctor, const char * sourceFile )
+{
+    tr_free( ctor->sourceFile );
+    ctor->sourceFile = tr_strdup( sourceFile );
+}
+
 static void
 clearMetainfo( tr_ctor * ctor )
 {
@@ -52,6 +64,8 @@ clearMetainfo( tr_ctor * ctor )
         ctor->isSet_metainfo = 0;
         tr_bencFree( &ctor->metainfo );
     }
+
+    setSourceFile( ctor, NULL );
 }
 
 int
@@ -66,6 +80,12 @@ tr_ctorSetMetainfo( tr_ctor        * ctor,
     return err;
 }
 
+const char*
+tr_ctorGetSourceFile( const tr_ctor * ctor )
+{
+    return ctor->sourceFile;
+}
+
 int
 tr_ctorSetMetainfoFromFile( tr_ctor        * ctor,
                             const char     * filename )
@@ -82,11 +102,13 @@ tr_ctorSetMetainfoFromFile( tr_ctor        * ctor,
         err = 1;
     }
 
+    setSourceFile( ctor, filename );
+
     /* if no `name' field was set, then set it from the filename */
     if( ctor->isSet_metainfo ) {
-        benc_val_t * info = tr_bencDictFindType( &ctor->metainfo, "info", TYPE_DICT );
-        if( info != NULL ) {
-            benc_val_t * name = tr_bencDictFindFirst( info, "name.utf-8", "name", NULL );
+        tr_benc * info = tr_bencDictFindType( &ctor->metainfo, "info", TYPE_DICT );
+        if( info ) {
+            tr_benc * name = tr_bencDictFindFirst( info, "name.utf-8", "name", NULL );
             if( name == NULL )
                 name = tr_bencDictAdd( info, "name" );
             if( name->type!=TYPE_STR || !name->val.s.s || !*name->val.s.s ) {
@@ -105,20 +127,39 @@ int
 tr_ctorSetMetainfoFromHash( tr_ctor        * ctor,
                             const char     * hashString )
 {
-    int err = -1;
-    char basename[2048];
-    char filename[MAX_PATH_LENGTH];
+    int err;
+    const char * filename;
 
-    if( err && ( ctor->handle->tag != NULL ) ) {
-        snprintf( basename, sizeof(basename), "%s-%s", hashString, ctor->handle->tag );
-        tr_buildPath( filename, sizeof(filename), tr_getTorrentsDirectory(), basename, NULL );
-        err = tr_ctorSetMetainfoFromFile( ctor, filename );
-    }
+    if(( filename = tr_sessionFindTorrentFile( ctor->handle, hashString )))
+        err = tr_ctorSetMetainfoFromFile( ctor, filename ); 
+    else
+        err = TR_ERROR;
 
-    if( err ) {
-        tr_buildPath( filename, sizeof(filename), tr_getTorrentsDirectory(), hashString, NULL );
-        err = tr_ctorSetMetainfoFromFile( ctor, filename );
-    }
+    return err;
+}
+
+/***
+****
+***/
+
+void
+tr_ctorSetDeleteSource( tr_ctor  * ctor,
+                        uint8_t    deleteSource )
+{
+    ctor->doDelete = deleteSource ? 1 : 0;
+    ctor->isSet_delete = 1;
+}
+
+int
+tr_ctorGetDeleteSource( const tr_ctor  * ctor,
+                        uint8_t        * setme )
+{
+    int err = 0;
+
+    if( !ctor->isSet_delete )
+        err = 1;
+    else if( setme )
+        *setme = ctor->doDelete ? 1 : 0;
 
     return err;
 }
@@ -151,83 +192,88 @@ tr_ctorSetPaused( tr_ctor        * ctor,
 }
 
 void
-tr_ctorSetMaxConnectedPeers( tr_ctor        * ctor,
-                             tr_ctorMode      mode,
-                             uint16_t         maxConnectedPeers )
+tr_ctorSetPeerLimit( tr_ctor        * ctor,
+                     tr_ctorMode      mode,
+                     uint16_t         peerLimit )
 {
     struct optional_args * args = &ctor->optionalArgs[mode];
     args->isSet_connected = 1;
-    args->maxConnectedPeers = maxConnectedPeers;
+    args->peerLimit = peerLimit;
 }
 
 void
-tr_ctorSetDestination( tr_ctor        * ctor,
+tr_ctorSetDownloadDir( tr_ctor        * ctor,
                        tr_ctorMode      mode,
                        const char     * directory )
 {
     struct optional_args * args = &ctor->optionalArgs[mode];
-    args->isSet_destination = 1;
-    strlcpy( args->destination, directory, sizeof( args->destination ) );
+    if( directory ) {
+        args->isSet_downloadDir = 1;
+        tr_strlcpy( args->downloadDir, directory, sizeof( args->downloadDir ) );
+    } else {
+        args->isSet_downloadDir = 0;
+        *args->downloadDir = '\0';
+    }
 }
 
 int
-tr_ctorGetMaxConnectedPeers( const tr_ctor  * ctor,
-                             tr_ctorMode      mode,
-                             uint16_t       * setmeCount )
+tr_ctorGetPeerLimit( const tr_ctor  * ctor,
+                     tr_ctorMode      mode,
+                     uint16_t       * setmeCount )
 {
     int err = 0;
     const struct optional_args * args = &ctor->optionalArgs[mode];
 
-    if( args->isSet_connected )
-        *setmeCount = args->maxConnectedPeers;
-    else
+    if( !args->isSet_connected )
         err = 1;
+    else if( setmeCount )
+        *setmeCount = args->peerLimit;
 
     return err;
 }
 
 int
-tr_ctorGetIsPaused( const tr_ctor  * ctor,
-                    tr_ctorMode      mode,
-                    uint8_t        * setmeIsPaused )
+tr_ctorGetPaused( const tr_ctor  * ctor,
+                  tr_ctorMode      mode,
+                  uint8_t        * setmeIsPaused )
 {
     int err = 0;
     const struct optional_args * args = &ctor->optionalArgs[mode];
 
-    if( args->isSet_paused )
+    if( !args->isSet_paused )
+        err = 1;
+    else if( setmeIsPaused )
         *setmeIsPaused = args->isPaused ? 1 : 0;
-    else
-        err = 1;
 
     return err;
 }
 
 int
-tr_ctorGetDestination( const tr_ctor  * ctor,
+tr_ctorGetDownloadDir( const tr_ctor  * ctor,
                        tr_ctorMode      mode,
-                       const char    ** setmeDestination )
+                       const char    ** setmeDownloadDir )
 {
     int err = 0;
     const struct optional_args * args = &ctor->optionalArgs[mode];
 
-    if( args->isSet_destination )
-        *setmeDestination = args->destination;
-    else
+    if( !args->isSet_downloadDir )
         err = 1;
+    else if( setmeDownloadDir )
+        *setmeDownloadDir = args->downloadDir;
 
     return err;
 }
 
 int
-tr_ctorGetMetainfo( const tr_ctor              * ctor,
-                    const struct benc_val_s   ** setme )
+tr_ctorGetMetainfo( const tr_ctor    * ctor,
+                    const tr_benc   ** setme )
 {
     int err = 0;
 
-    if( ctor->isSet_metainfo )
-        *setme = &ctor->metainfo;
-    else
+    if( !ctor->isSet_metainfo )
         err = 1;
+    else if( setme )
+        *setme = &ctor->metainfo;
 
     return err;
 }
@@ -241,8 +287,9 @@ tr_ctorNew( const tr_handle * handle )
 {
     tr_ctor * ctor = tr_new0( struct tr_ctor, 1 );
     ctor->handle = handle;
-    tr_ctorSetMaxConnectedPeers( ctor, TR_FALLBACK, DEFAULT_MAX_CONNECTED_PEERS );
+    tr_ctorSetPeerLimit( ctor, TR_FALLBACK, DEFAULT_MAX_CONNECTED_PEERS );
     tr_ctorSetPaused( ctor, TR_FALLBACK, FALSE );
+    tr_ctorSetDownloadDir( ctor, TR_FALLBACK, handle->downloadDir );
     tr_ctorSetSave( ctor, TRUE );
     return ctor;
 }
