@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: torrent.c 6181 2008-06-13 21:08:16Z charles $
+ * $Id: torrent.c 6469 2008-08-10 15:15:51Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -42,7 +42,6 @@
 #include "ratecontrol.h"
 #include "torrent.h"
 #include "tracker.h"
-#include "trcompat.h" /* for strlcpy */
 #include "trevent.h"
 #include "utils.h"
 #include "verify.h"
@@ -54,19 +53,49 @@
 ***/
 
 int
-tr_torrentExists( tr_handle       * handle,
+tr_torrentId( const tr_torrent * tor )
+{
+    return tor->uniqueId;
+}
+
+tr_torrent*
+tr_torrentFindFromId( tr_handle * handle, int id )
+{
+    tr_torrent * tor = NULL;
+
+    while(( tor = tr_torrentNext( handle, tor )))
+        if( tor->uniqueId == id )
+            return tor;
+
+    return NULL;
+}
+
+tr_torrent*
+tr_torrentFindFromHashString( tr_handle * handle, const char * str )
+{
+    tr_torrent * tor = NULL;
+
+    while(( tor = tr_torrentNext( handle, tor )))
+        if( !strcmp( str, tor->info.hashString ) )
+            return tor;
+
+    return NULL;
+}
+
+int
+tr_torrentExists( const tr_handle * handle,
                   const uint8_t   * torrentHash )
 {
-    return tr_torrentFindFromHash( handle, torrentHash ) != NULL;
+    return tr_torrentFindFromHash( (tr_handle*)handle, torrentHash ) != NULL;
 }
 
 tr_torrent*
 tr_torrentFindFromHash( tr_handle      * handle,
                         const uint8_t  * torrentHash )
 {
-    tr_torrent * tor;
+    tr_torrent * tor = NULL;
 
-    for( tor = handle->torrentList; tor; tor = tor->next )
+    while(( tor = tr_torrentNext( handle, tor )))
         if( !memcmp( tor->info.hash, torrentHash, SHA_DIGEST_LENGTH ) )
             return tor;
 
@@ -77,9 +106,9 @@ tr_torrent*
 tr_torrentFindFromObfuscatedHash( tr_handle      * handle,
                                   const uint8_t  * obfuscatedTorrentHash )
 {
-    tr_torrent * tor;
+    tr_torrent * tor = NULL;
 
-    for( tor = handle->torrentList; tor; tor = tor->next )
+    while(( tor = tr_torrentNext( handle, tor )))
         if( !memcmp( tor->obfuscatedHash, obfuscatedTorrentHash, SHA_DIGEST_LENGTH ) )
             return tor;
 
@@ -158,7 +187,7 @@ onTrackerResponse( void * tracker UNUSED, void * vevent, void * user_data )
             size_t i, n;
             tr_pex * pex = tr_peerMgrCompactToPex( event->compact,
                                                    event->compactLen,
-                                                   NULL, &n );
+                                                   NULL, 0, &n );
             if( event->allAreSeeds )
                 tr_tordbg( tor, "Got %d seeds from tracker", (int)n );
             else
@@ -178,13 +207,13 @@ onTrackerResponse( void * tracker UNUSED, void * vevent, void * user_data )
         case TR_TRACKER_WARNING:
             tr_torerr( tor, _( "Tracker warning: \"%s\"" ), event->text );
             tor->error = TR_ERROR_TC_WARNING;
-            strlcpy( tor->errorString, event->text, sizeof(tor->errorString) );
+            tr_strlcpy( tor->errorString, event->text, sizeof(tor->errorString) );
             break;
 
         case TR_TRACKER_ERROR:
             tr_torerr( tor, _( "Tracker error: \"%s\"" ), event->text );
             tor->error = TR_ERROR_TC_ERROR;
-            strlcpy( tor->errorString, event->text, sizeof(tor->errorString) );
+            tr_strlcpy( tor->errorString, event->text, sizeof(tor->errorString) );
             break;
 
         case TR_TRACKER_ERROR_CLEAR:
@@ -203,7 +232,7 @@ onTrackerResponse( void * tracker UNUSED, void * vevent, void * user_data )
 static int
 getBytePiece( const tr_info * info, uint64_t byteOffset )
 {
-    assert( info != NULL );
+    assert( info );
     assert( info->pieceSize != 0 );
 
     return byteOffset / info->pieceSize;
@@ -212,10 +241,10 @@ getBytePiece( const tr_info * info, uint64_t byteOffset )
 static void
 initFilePieces ( tr_info * info, tr_file_index_t fileIndex )
 {
-    tr_file * file = &info->files[fileIndex];
+    tr_file * file;
     uint64_t firstByte, lastByte;
 
-    assert( info != NULL );
+    assert( info );
     assert( fileIndex < info->fileCount );
 
     file = &info->files[fileIndex];
@@ -278,17 +307,18 @@ tr_torrentInitFilePieces( tr_torrent * tor )
     tr_file_index_t ff;
     tr_piece_index_t pp;
     uint64_t offset = 0;
+    tr_info * inf = &tor->info;
 
-    assert( tor != NULL );
+    assert( inf );
 
-    for( ff=0; ff<tor->info.fileCount; ++ff ) {
-      tor->info.files[ff].offset = offset;
-      offset += tor->info.files[ff].length;
-      initFilePieces( &tor->info, ff );
+    for( ff=0; ff<inf->fileCount; ++ff ) {
+      inf->files[ff].offset = offset;
+      offset += inf->files[ff].length;
+      initFilePieces( inf, ff );
     }
 
-    for( pp=0; pp<tor->info.pieceCount; ++pp )
-        tor->info.pieces[pp].priority = calculatePiecePriority( tor, pp, -1 );
+    for( pp=0; pp<inf->pieceCount; ++pp )
+        inf->pieces[pp].priority = calculatePiecePriority( tor, pp, -1 );
 }
 
 int
@@ -297,7 +327,7 @@ tr_torrentPromoteTracker( tr_torrent * tor, int pos )
     int i;
     int tier;
 
-    assert( tor != NULL );
+    assert( tor );
     assert( ( 0 <= pos ) && ( pos < tor->info.trackerCount ) );
 
     /* the tier of the tracker we're promoting */
@@ -359,6 +389,22 @@ randomizeTiers( tr_info * info )
 
 static void torrentStart( tr_torrent * tor, int reloadProgress );
 
+/**
+ * Decide on a block size.  constraints:
+ * (1) most clients decline requests over 16 KiB
+ * (2) pieceSize must be a multiple of block size
+ */
+static uint32_t
+getBlockSize( uint32_t pieceSize )
+{
+    uint32_t b = pieceSize;
+    while( b > MAX_BLOCK_SIZE )
+        b /= 2u;
+    if( !b || ( pieceSize % b ) ) /* not cleanly divisible */
+        return 0;
+    return b;
+}
+
 static void
 torrentRealInit( tr_handle     * h,
                  tr_torrent    * tor,
@@ -368,21 +414,16 @@ torrentRealInit( tr_handle     * h,
     uint64_t loaded;
     uint64_t t;
     tr_info * info = &tor->info;
+    static int nextUniqueId = 1;
    
     tr_globalLock( h );
 
     tor->handle   = h;
+    tor->uniqueId = nextUniqueId++;
 
     randomizeTiers( info );
 
-    /**
-     * Decide on a block size.  constraints:
-     * (1) most clients decline requests over 16 KiB
-     * (2) pieceSize must be a multiple of block size
-     */
-    tor->blockSize = info->pieceSize;
-    while( tor->blockSize > MAX_BLOCK_SIZE )
-        tor->blockSize /= 2;
+    tor->blockSize = getBlockSize( info->pieceSize );
 
     tor->lastPieceSize = info->totalSize % info->pieceSize;
 
@@ -424,7 +465,7 @@ torrentRealInit( tr_handle     * h,
 
     tor->upload         = tr_rcInit();
     tor->download       = tr_rcInit();
-    tor->swarmspeed     = tr_rcInit();
+    tor->swarmSpeed     = tr_rcInit();
 
     tr_sha1( tor->obfuscatedHash, "req2", 4,
                                   info->hash, SHA_DIGEST_LENGTH,
@@ -433,7 +474,7 @@ torrentRealInit( tr_handle     * h,
     tr_peerMgrAddTorrent( h->peerMgr, tor );
 
     if( !h->isPortSet )
-        tr_setBindPort( h, TR_DEFAULT_PORT );
+        tr_sessionSetPeerPort( h, TR_DEFAULT_PORT );
 
     assert( !tor->downloadedCur );
     assert( !tor->uploadedCur );
@@ -442,17 +483,19 @@ torrentRealInit( tr_handle     * h,
 
     tor->checkedPieces = tr_bitfieldNew( tor->info.pieceCount );
     tr_torrentUncheck( tor );
+
+    tor->addedDate = time( NULL ); /* this is a default value to be
+                                      overwritten by the resume file */
     loaded = tr_torrentLoadResume( tor, ~0, ctor );
     
     doStart = tor->isRunning;
     tor->isRunning = 0;
 
     if( !(loaded & TR_FR_SPEEDLIMIT ) ) {
-        int limit, enabled;
-        tr_getGlobalSpeedLimit( tor->handle, TR_UP, &enabled, &limit );
-        tr_torrentSetSpeedLimit( tor, TR_UP, limit );
-        tr_getGlobalSpeedLimit( tor->handle, TR_DOWN, &enabled, &limit );
-        tr_torrentSetSpeedLimit( tor, TR_DOWN, limit );
+        tr_torrentSetSpeedLimit( tor, TR_UP,
+                tr_sessionGetSpeedLimit( tor->handle, TR_UP ) );
+        tr_torrentSetSpeedLimit( tor, TR_DOWN,
+                tr_sessionGetSpeedLimit( tor->handle, TR_DOWN ) );
     }
 
     tor->cpStatus = tr_cpGetStatus( tor->completion );
@@ -460,9 +503,17 @@ torrentRealInit( tr_handle     * h,
     tor->tracker = tr_trackerNew( tor );
     tor->trackerSubscription = tr_trackerSubscribe( tor->tracker, onTrackerResponse, tor );
 
-    tor->next = h->torrentList;
-    h->torrentList = tor;
-    h->torrentCount++;
+    {
+        tr_torrent * it = NULL;
+        tr_torrent * last = NULL;
+        while(( it = tr_torrentNext( h, it )))
+            last = it;
+        if( !last )
+            h->torrentList = tor;
+        else
+            last->next = tor;
+        ++h->torrentCount;
+    }
 
     tr_globalUnlock( h );
 
@@ -482,19 +533,6 @@ torrentRealInit( tr_handle     * h,
         torrentStart( tor, FALSE );
 }
 
-static int
-hashExists( const tr_handle   * h,
-            const uint8_t     * hash )
-{
-    const tr_torrent * tor;
-
-    for( tor=h->torrentList; tor; tor=tor->next )
-        if( !memcmp( hash, tor->info.hash, SHA_DIGEST_LENGTH ) )
-            return TRUE;
-
-    return FALSE;
-}
-
 int
 tr_torrentParse( const tr_handle  * handle,
                  const tr_ctor    * ctor,
@@ -515,7 +553,10 @@ tr_torrentParse( const tr_handle  * handle,
     err = tr_metainfoParse( handle, setmeInfo, metainfo );
     doFree = !err && ( setmeInfo == &tmp );
 
-    if( !err && hashExists( handle, setmeInfo->hash ) )
+    if( !err && !getBlockSize( setmeInfo->pieceSize ) )
+        err = TR_EINVALID;
+
+    if( !err && tr_torrentExists( handle, setmeInfo->hash ) )
         err = TR_EDUPLICATE;
 
     if( doFree )
@@ -550,20 +591,20 @@ tr_torrentNew( tr_handle      * handle,
 **/
 
 void
-tr_torrentSetFolder( tr_torrent * tor, const char * path )
+tr_torrentSetDownloadDir( tr_torrent * tor, const char * path )
 {
-    if( !path || !tor->destination || strcmp( path, tor->destination ) )
+    if( !path || !tor->downloadDir || strcmp( path, tor->downloadDir ) )
     {
-        tr_free( tor->destination );
-        tor->destination = tr_strdup( path );
+        tr_free( tor->downloadDir );
+        tor->downloadDir = tr_strdup( path );
         tr_torrentSaveResume( tor );
     }
 }
 
 const char*
-tr_torrentGetFolder( const tr_torrent * tor )
+tr_torrentGetDownloadDir( const tr_torrent * tor )
 {
-    return tor->destination;
+    return tor->downloadDir;
 }
 
 void
@@ -589,21 +630,21 @@ tr_torrentAllowsPex( const tr_torrent * tor )
 }
 
 static void
-tr_manualUpdateImpl( void * vtor )
+tr_torrentManualUpdateImpl( void * vtor )
 {
     tr_torrent * tor = vtor;
     if( tor->isRunning )
         tr_trackerReannounce( tor->tracker );
 }
 void
-tr_manualUpdate( tr_torrent * tor )
+tr_torrentManualUpdate( tr_torrent * tor )
 {
-    tr_runInEventThread( tor->handle, tr_manualUpdateImpl, tor );
+    tr_runInEventThread( tor->handle, tr_torrentManualUpdateImpl, tor );
 }
 int
 tr_torrentCanManualUpdate( const tr_torrent * tor )
 {
-    return ( tor != NULL )
+    return ( tor )
         && ( tor->isRunning )
         && ( tr_trackerCanManualAnnounce( tor->tracker ) );
 }
@@ -661,6 +702,8 @@ tr_torrentStat( tr_torrent * tor )
 {
     tr_stat * s;
     struct tr_tracker * tc;
+    const tr_tracker_info * ti;
+    int usableSeeds = 0;
 
     if( !tor )
         return NULL;
@@ -670,25 +713,31 @@ tr_torrentStat( tr_torrent * tor )
     tor->lastStatTime = time( NULL );
 
     s = &tor->stats;
+    s->id = tor->uniqueId;
     s->status = tr_torrentGetStatus( tor );
     s->error  = tor->error;
     memcpy( s->errorString, tor->errorString,
             sizeof( s->errorString ) );
 
     tc = tor->tracker;
-    s->tracker = tr_trackerGetAddress( tor->tracker );
-
-    tr_trackerStat( tor->tracker, &s->tracker_stat );
-
+    ti = tr_trackerGetAddress( tor->tracker );
+    s->announceURL = ti ? ti->announce : NULL;
+    s->scrapeURL   = ti ? ti->scrape   : NULL;
+    tr_trackerStat( tc, s );
+    tr_trackerGetCounts( tc, &s->timesCompleted,
+                             &s->leechers, 
+                             &s->seeders );
     tr_peerMgrTorrentStats( tor->handle->peerMgr,
                             tor->info.hash,
                             &s->peersKnown,
                             &s->peersConnected,
+                               &usableSeeds,
+                            &s->webseedsSendingToUs,
                             &s->peersSendingToUs,
                             &s->peersGettingFromUs,
                              s->peersFrom );
 
-    s->manualAnnounceTime = tr_trackerGetManualAnnounceTime( tor->tracker );
+    usableSeeds += tor->info.webseedCount;
 
     s->percentComplete = tr_cpPercentComplete ( tor->completion );
 
@@ -696,20 +745,18 @@ tr_torrentStat( tr_torrent * tor )
     s->leftUntilDone = tr_cpLeftUntilDone( tor->completion );
     s->sizeWhenDone = tr_cpSizeWhenDone( tor->completion );
 
-    s->recheckProgress =
-        1.0 - (tr_torrentCountUncheckedPieces( tor ) / (double) tor->info.pieceCount);
+    s->recheckProgress = s->status == TR_STATUS_CHECK
+        ? 1.0 - (tr_torrentCountUncheckedPieces( tor ) / (double) tor->info.pieceCount)
+        : 0.0;
 
     tr_torrentGetRates( tor, &s->rateDownload, &s->rateUpload );
-   
-    tr_trackerGetCounts( tc,
-                         &s->completedFromTracker,
-                         &s->leechers, 
-                         &s->seeders );
 
-    s->swarmspeed = tr_rcRate( tor->swarmspeed );
+    s->swarmSpeed = tr_rcRate( tor->swarmSpeed );
     
-    s->startDate = tor->startDate;
     s->activityDate = tor->activityDate;
+    s->addedDate    = tor->addedDate;
+    s->doneDate     = tor->doneDate;
+    s->startDate    = tor->startDate;
 
     s->corruptEver     = tor->corruptCur    + tor->corruptPrev;
     s->downloadedEver  = tor->downloadedCur + tor->downloadedPrev;
@@ -718,6 +765,15 @@ tr_torrentStat( tr_torrent * tor )
     s->haveUnchecked   = tr_cpHaveTotal( tor->completion ) - s->haveValid;
 
 
+    if( usableSeeds > 0 )
+    {
+        s->desiredAvailable = s->leftUntilDone;
+    }
+    else if( !s->leftUntilDone || !s->peersConnected )
+    {
+        s->desiredAvailable = 0;
+    }
+    else
     {
         tr_piece_index_t i;
         tr_bitfield * peerPieces = tr_peerMgrGetAvailable( tor->handle->peerMgr,
@@ -760,7 +816,7 @@ fileBytesCompleted ( const tr_torrent * tor, tr_file_index_t fileIndex )
     const uint64_t lastBlockOffset  = (file->offset + lastOffset) % tor->blockSize;
     uint64_t haveBytes = 0;
 
-    assert( tor != NULL );
+    assert( tor );
     assert( fileIndex < tor->info.fileCount );
     assert( file->offset + file->length <= tor->info.totalSize );
     assert( ( firstBlock < tor->blockCount ) || (!file->length && file->offset==tor->info.totalSize) );
@@ -822,12 +878,20 @@ tr_torrentFilesFree( tr_file_stat * files, tr_file_index_t fileCount UNUSED )
 ****
 ***/
 
+float*
+tr_torrentWebSpeeds( const tr_torrent * tor )
+{
+    return tor ? tr_peerMgrWebSpeeds( tor->handle->peerMgr, tor->info.hash )
+               : NULL;
+}
+    
+
 tr_peer_stat *
 tr_torrentPeers( const tr_torrent * tor, int * peerCount )
 {
     tr_peer_stat * ret = NULL;
 
-    if( tor != NULL )
+    if( tor )
         ret = tr_peerMgrPeerStats( tor->handle->peerMgr,
                                    tor->info.hash, peerCount );
 
@@ -840,7 +904,8 @@ tr_torrentPeersFree( tr_peer_stat * peers, int peerCount UNUSED )
     tr_free( peers );
 }
 
-void tr_torrentAvailability( const tr_torrent * tor, int8_t * tab, int size )
+void
+tr_torrentAvailability( const tr_torrent * tor, int8_t * tab, int size )
 {
     tr_peerMgrTorrentAvailability( tor->handle->peerMgr,
                                    tor->info.hash,
@@ -876,7 +941,7 @@ tr_torrentSetHasPiece( tr_torrent * tor, tr_piece_index_t pieceIndex, int has )
 {
     tr_torrentLock( tor );
 
-    assert( tor != NULL );
+    assert( tor );
     assert( pieceIndex < tor->info.pieceCount );
 
     if( has )
@@ -887,14 +952,6 @@ tr_torrentSetHasPiece( tr_torrent * tor, tr_piece_index_t pieceIndex, int has )
     tr_torrentUnlock( tor );
 }
 
-void
-tr_torrentRemoveSaved( tr_torrent * tor )
-{
-    tr_metainfoRemoveSaved( tor->handle, &tor->info );
-
-    tr_torrentRemoveResume( tor );
-}
-
 /***
 ****
 ***/
@@ -906,7 +963,7 @@ freeTorrent( tr_torrent * tor )
     tr_handle * h = tor->handle;
     tr_info * inf = &tor->info;
 
-    assert( tor != NULL );
+    assert( tor );
     assert( !tor->isRunning );
 
     tr_globalLock( h );
@@ -917,7 +974,7 @@ freeTorrent( tr_torrent * tor )
 
     tr_rcClose( tor->upload );
     tr_rcClose( tor->download );
-    tr_rcClose( tor->swarmspeed );
+    tr_rcClose( tor->swarmSpeed );
 
     tr_trackerUnsubscribe( tor->tracker, tor->trackerSubscription );
     tr_trackerFree( tor->tracker );
@@ -925,7 +982,7 @@ freeTorrent( tr_torrent * tor )
 
     tr_bitfieldFree( tor->checkedPieces );
 
-    tr_free( tor->destination );
+    tr_free( tor->downloadDir );
 
     if( tor == h->torrentList )
         h->torrentList = tor->next;
@@ -939,8 +996,6 @@ freeTorrent( tr_torrent * tor )
     assert( h->torrentCount >= 1 );
     h->torrentCount--;
 
-    tr_torinf( tor, _( "Closing torrent; %d left" ), h->torrentCount );
-
     tr_metainfoFree( inf );
     tr_free( tor );
 
@@ -951,32 +1006,6 @@ freeTorrent( tr_torrent * tor )
 ***  Start/Stop Callback
 **/
 
-static void
-fireActiveChange( tr_torrent * tor, int isRunning )
-{
-    assert( tor != NULL );
-
-    if( tor->active_func != NULL )
-        (tor->active_func)( tor, isRunning, tor->active_func_user_data );
-}
-
-void
-tr_torrentSetActiveCallback( tr_torrent             * tor,
-                             tr_torrent_active_func   func,
-                             void                   * user_data )
-{
-    assert( tor != NULL );
-    tor->active_func = func;
-    tor->active_func_user_data = user_data;
-}
-
-void
-tr_torrentClearActiveCallback( tr_torrent * torrent )
-{
-    tr_torrentSetActiveCallback( torrent, NULL, NULL );
-}
-
-
 static void
 checkAndStartImpl( void * vtor )
 {
@@ -985,12 +1014,11 @@ checkAndStartImpl( void * vtor )
     tr_globalLock( tor->handle );
 
     tor->isRunning  = 1;
-    fireActiveChange( tor, tor->isRunning );
     *tor->errorString = '\0';
     tr_torrentResetTransferStats( tor );
     tor->cpStatus = tr_cpGetStatus( tor->completion );
     tr_torrentSaveResume( tor );
-    tor->startDate = tr_date( );
+    tor->startDate = time( NULL );
     tr_trackerStart( tor->tracker );
     tr_peerMgrStartTorrent( tor->handle->peerMgr, tor->info.hash );
 
@@ -1022,7 +1050,8 @@ torrentStart( tr_torrent * tor, int reloadProgress )
 void
 tr_torrentStart( tr_torrent * tor )
 {
-    torrentStart( tor, TRUE );
+    if( tor )
+        torrentStart( tor, TRUE );
 }
 
 static void
@@ -1058,13 +1087,12 @@ stopTorrent( void * vtor )
     tr_verifyRemove( tor );
     tr_peerMgrStopTorrent( tor->handle->peerMgr, tor->info.hash );
     tr_trackerStop( tor->tracker );
-    fireActiveChange( tor, 0 );
 
     for( i=0; i<tor->info.fileCount; ++i )
     {
         char path[MAX_PATH_LENGTH];
         const tr_file * file = &tor->info.files[i];
-        tr_buildPath( path, sizeof(path), tor->destination, file->name, NULL );
+        tr_buildPath( path, sizeof(path), tor->downloadDir, file->name, NULL );
         tr_fdFileClose( path );
     }
 }
@@ -1072,14 +1100,17 @@ stopTorrent( void * vtor )
 void
 tr_torrentStop( tr_torrent * tor )
 {
-    tr_globalLock( tor->handle );
+    if( tor )
+    {
+        tr_globalLock( tor->handle );
 
-    tor->isRunning = 0;
-    if( !tor->isDeleting )
-        tr_torrentSaveResume( tor );
-    tr_runInEventThread( tor->handle, stopTorrent, tor );
+        tor->isRunning = 0;
+        if( !tor->isDeleting )
+            tr_torrentSaveResume( tor );
+        tr_runInEventThread( tor->handle, stopTorrent, tor );
 
-    tr_globalUnlock( tor->handle );
+        tr_globalUnlock( tor->handle );
+    }
 }
 
 static void
@@ -1089,15 +1120,17 @@ closeTorrent( void * vtor )
     tr_torrentSaveResume( tor );
     tor->isRunning = 0;
     stopTorrent( tor );
-    if( tor->isDeleting )
-        tr_torrentRemoveSaved( tor );
+    if( tor->isDeleting ) {
+        tr_metainfoRemoveSaved( tor->handle, &tor->info );
+        tr_torrentRemoveResume( tor );
+    }
     freeTorrent( tor );
 }
 
 void
-tr_torrentClose( tr_torrent * tor )
+tr_torrentFree( tr_torrent * tor )
 {
-    if( tor != NULL )
+    if( tor )
     {
         tr_handle * handle = tor->handle;
         tr_globalLock( handle );
@@ -1110,10 +1143,10 @@ tr_torrentClose( tr_torrent * tor )
 }
 
 void
-tr_torrentDelete( tr_torrent * tor )
+tr_torrentRemove( tr_torrent * tor )
 {
     tor->isDeleting = 1;
-    tr_torrentClose( tor );
+    tr_torrentFree( tor );
 }
 
 
@@ -1139,11 +1172,11 @@ getCompletionString( int type )
 static void
 fireStatusChange( tr_torrent * tor, cp_status_t status )
 {
-    assert( tor != NULL );
+    assert( tor );
     assert( status==TR_CP_INCOMPLETE || status==TR_CP_DONE || status==TR_CP_COMPLETE );
 
-    if( tor->status_func != NULL )
-        (tor->status_func)( tor, status, tor->status_func_user_data );
+    if( tor->status_func )
+        tor->status_func( tor, status, tor->status_func_user_data );
 }
 
 void
@@ -1151,7 +1184,7 @@ tr_torrentSetStatusCallback( tr_torrent             * tor,
                              tr_torrent_status_func   func,
                              void                   * user_data )
 {
-    assert( tor != NULL );
+    assert( tor );
     tor->status_func = func;
     tor->status_func_user_data = user_data;
 }
@@ -1186,8 +1219,12 @@ tr_torrentRecheckCompleteness( tr_torrent * tor )
         fireStatusChange( tor, cpStatus );
 
         if( recentChange && ( cpStatus == TR_CP_COMPLETE ) )
+        {
             tr_trackerCompleted( tor->tracker );
 
+            tor->doneDate = time( NULL );
+        }
+
         tr_torrentSaveResume( tor );
     }
 
@@ -1212,7 +1249,7 @@ tr_torrentInitFilePriority( tr_torrent      * tor,
     tr_piece_index_t i;
     tr_file * file;
 
-    assert( tor != NULL );
+    assert( tor );
     assert( fileIndex < tor->info.fileCount );
     assert( priority==TR_PRI_LOW || priority==TR_PRI_NORMAL || priority==TR_PRI_HIGH );
 
@@ -1244,7 +1281,7 @@ tr_torrentGetFilePriority( const tr_torrent *  tor, tr_file_index_t file )
     tr_priority_t ret;
 
     tr_torrentLock( tor );
-    assert( tor != NULL );
+    assert( tor );
     assert( file < tor->info.fileCount );
     ret = tor->info.files[file].priority;
     tr_torrentUnlock( tor );
@@ -1370,14 +1407,14 @@ tr_torrentSetFileDLs ( tr_torrent      * tor,
 ***/
 
 void
-tr_torrentSetMaxConnectedPeers( tr_torrent  * tor,
-                                uint16_t      maxConnectedPeers )
+tr_torrentSetPeerLimit( tr_torrent  * tor,
+                        uint16_t      maxConnectedPeers )
 {
     tor->maxConnectedPeers = maxConnectedPeers;
 }
 
 uint16_t
-tr_torrentGetMaxConnectedPeers( const tr_torrent  * tor )
+tr_torrentGetPeerLimit( const tr_torrent  * tor )
 {
     return tor->maxConnectedPeers;
 }
@@ -1416,6 +1453,8 @@ tr_torrentReqIsValid( const tr_torrent * tor,
     else if( tr_pieceOffset( tor, index, offset, length ) > tor->info.totalSize )
         err = 4;
 
+if( err ) fprintf( stderr, "index %lu offset %lu length %lu err %d\n", (unsigned long)index, (unsigned long)offset, (unsigned long)length, err );
+
     return !err;
 }
 
@@ -1505,7 +1544,7 @@ tr_torrentGetMTimes( const tr_torrent * tor, int * setme_n )
         char fname[MAX_PATH_LENGTH];
         struct stat sb;
         tr_buildPath( fname, sizeof(fname),
-                      tor->destination, tor->info.files[i].name, NULL );
+                      tor->downloadDir, tor->info.files[i].name, NULL );
         if ( !stat( fname, &sb ) ) {
 #ifdef SYS_DARWIN
             m[i] = sb.st_mtimespec.tv_sec;
@@ -1518,3 +1557,79 @@ tr_torrentGetMTimes( const tr_torrent * tor, int * setme_n )
     *setme_n = n;
     return m;
 }
+
+/***
+****
+***/
+
+void
+tr_torrentSetAnnounceList( tr_torrent             * tor,
+                           const tr_tracker_info  * trackers,
+                           int                      trackerCount )
+{
+    tr_benc metainfo;
+
+    /* save to the .torrent file */
+    if( !tr_bencLoadFile( tor->info.torrent, &metainfo ) )
+    {
+        int i;
+        int prevTier = -1;
+        tr_benc * tier = NULL;
+        tr_benc * announceList;
+        tr_info tmpInfo;
+
+        /* remove the old fields */
+        tr_bencDictRemove( &metainfo, "announce" );
+        tr_bencDictRemove( &metainfo, "announce-list" );
+
+        /* add the new fields */
+        tr_bencDictAddStr( &metainfo, "announce", trackers[0].announce );
+        announceList = tr_bencDictAddList( &metainfo, "announce-list", 0 );
+        for( i=0; i<trackerCount; ++i ) {
+            if( prevTier != trackers[i].tier ) {
+                prevTier = trackers[i].tier;
+                tier = tr_bencListAddList( announceList, 0 );
+            }
+            tr_bencListAddStr( tier, trackers[i].announce );
+        }
+
+        /* try to parse it back again, to make sure it's good */
+        memset( &tmpInfo, 0, sizeof( tr_info ) );
+        if( !tr_metainfoParse( tor->handle, &tmpInfo, &metainfo ) )
+        {
+            /* if it's good, save it and use it */
+            tr_metainfoFree( &tor->info );
+            tor->info = tmpInfo;
+            tr_torrentInitFilePieces( tor );
+            tr_bencSaveFile( tor->info.torrent, &metainfo );
+        }
+
+        /* cleanup */ 
+        tr_bencFree( &metainfo );
+    }
+}
+
+/**
+***
+**/
+
+/** @deprecated this method will be removed in 1.40 */
+void
+tr_torrentSetAddedDate( tr_torrent * tor, time_t t )
+{
+    tor->addedDate = t;
+}
+
+/** @deprecated this method will be removed in 1.40 */
+void
+tr_torrentSetActivityDate( tr_torrent * tor, time_t t )
+{
+    tor->activityDate = t;
+}
+
+/** @deprecated this method will be removed in 1.40 */
+void
+tr_torrentSetDoneDate( tr_torrent * tor, time_t t )
+{
+    tor->doneDate = t;
+}
