@@ -7,11 +7,10 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: tracker.c 5881 2008-05-21 17:25:56Z charles $
+ * $Id: tracker.c 6759 2008-09-16 16:50:21Z charles $
  */
 
 #include <assert.h>
-#include <stdio.h> /* snprintf */
 #include <stdlib.h>
 #include <string.h> /* strcmp, strchr */
 
@@ -21,11 +20,10 @@
 #include "bencode.h"
 #include "completion.h"
 #include "net.h"
-#include "port-forwarding.h"
 #include "publish.h"
+#include "resume.h"
 #include "torrent.h"
 #include "tracker.h"
-#include "trcompat.h" /* strlcpy */
 #include "trevent.h"
 #include "utils.h"
 #include "web.h"
@@ -37,14 +35,6 @@ enum
     /* seconds between tracker pulses */
     PULSE_INTERVAL_MSEC = 1000,
 
-    /* maximum number of concurrent tracker socket connections */
-    MAX_TRACKER_SOCKETS = 16,
-
-    /* maximum number of concurrent tracker socket connections during shutdown.
-     * all the peer connections should be gone by now, so we can hog more 
-     * connections to send `stop' messages to the trackers */
-    MAX_TRACKER_SOCKETS_DURING_SHUTDOWN = 64,
-
     /* unless the tracker says otherwise, rescrape this frequently */
     DEFAULT_SCRAPE_INTERVAL_SEC = (60 * 15),
 
@@ -54,6 +44,12 @@ enum
     /* unless the tracker says otherwise, this is the announce min_interval */
     DEFAULT_ANNOUNCE_MIN_INTERVAL_SEC = (60 * 2),
 
+    /* how long to wait before a rescrape the first time we get an error */
+    FIRST_SCRAPE_RETRY_INTERVAL_SEC = 30,
+
+    /* how long to wait before a reannounce the first time we get an error */
+    FIRST_ANNOUNCE_RETRY_INTERVAL_SEC = 30,
+
     /* the value of the 'numwant' argument passed in tracker requests. */
     NUMWANT = 80,
 
@@ -71,21 +67,22 @@ struct tr_tracker
 
     uint8_t randOffset;
 
-    tr_session * session;
+    /* sent as the "key" argument in tracker requests
+       to verify us if our IP address changes.
+       This is immutable for the life of the tracker object. */
+    char key_param[KEYLEN+1];
 
     /* these are set from the latest scrape or tracker response */
     int announceIntervalSec;
     int announceMinIntervalSec;
     int scrapeIntervalSec;
     int retryScrapeIntervalSec;
+    int retryAnnounceIntervalSec;
 
     /* index into the torrent's tr_info.trackers array */
     int trackerIndex;
 
-    /* sent as the "key" argument in tracker requests
-       to verify us if our IP address changes.
-       This is immutable for the life of the tracker object. */
-    char key_param[KEYLEN+1];
+    tr_session * session;
 
     tr_publisher_t * publisher;
 
@@ -125,15 +122,20 @@ struct tr_tracker
 ***/
 
 static const tr_tracker_info *
-getCurrentAddressFromTorrent( const tr_tracker * t, const tr_torrent * tor )
+getCurrentAddressFromTorrent( tr_tracker * t, const tr_torrent * tor )
 {
+    /* user might have removed trackers,
+     * so check to make sure our current index is in-bounds */
+    if( t->trackerIndex >= tor->info.trackerCount )
+        t->trackerIndex = 0;
+
     assert( t->trackerIndex >= 0 );
     assert( t->trackerIndex < tor->info.trackerCount );
     return tor->info.trackers + t->trackerIndex;
 }
     
 static const tr_tracker_info *
-getCurrentAddress( const tr_tracker * t )
+getCurrentAddress( tr_tracker * t )
 {
     const tr_torrent * torrent;
     if(( torrent = tr_torrentFindFromHash( t->session, t->hash )))
@@ -142,7 +144,7 @@ getCurrentAddress( const tr_tracker * t )
 }
 
 static int
-trackerSupportsScrape( const tr_tracker * t, const tr_torrent * tor )
+trackerSupportsScrape( tr_tracker * t, const tr_torrent * tor )
 {
     const tr_tracker_info * info = getCurrentAddressFromTorrent( t, tor );
     return info && info->scrape;
@@ -152,7 +154,7 @@ trackerSupportsScrape( const tr_tracker * t, const tr_torrent * tor )
 ****
 ***/
 
-tr_tracker *
+static tr_tracker *
 findTracker( tr_session * session, const uint8_t * hash )
 {
     tr_torrent * torrent = tr_torrentFindFromHash( session, hash );
@@ -289,7 +291,7 @@ onStoppedResponse( tr_session    * session,
                    long            responseCode  UNUSED,
                    const void    * response      UNUSED,
                    size_t          responseLen   UNUSED,
-                   void          * torrent_hash )
+                   void          * torrent_hash  )
 {
     dbgmsg( NULL, "got a response to some `stop' message" );
     tr_free( torrent_hash );
@@ -313,14 +315,15 @@ onTrackerResponse( tr_session    * session,
     if( !t ) /* tracker's been closed */
         return;
 
-    dbgmsg( t->name, "tracker response: %d", responseCode );
-    tr_ndbg( t->name, "tracker response: %d", responseCode );
+    dbgmsg( t->name, "tracker response: %ld", responseCode );
+    tr_ndbg( t->name, "tracker response: %ld", responseCode );
     t->lastAnnounceResponse = responseCode;
 
     if( responseCode == HTTP_OK )
     {
         tr_benc benc;
-        const int bencLoaded = !tr_bencLoad( response, responseLen, &benc, 0 );
+        const int bencLoaded = !tr_bencLoad( response, responseLen,
+                                             &benc, NULL );
         publishErrorClear( t );
         if( bencLoaded && tr_bencIsDict( &benc ) )
         {
@@ -330,9 +333,9 @@ onTrackerResponse( tr_session    * session,
             const char * str;
 
             success = TRUE;
+            t->retryAnnounceIntervalSec = FIRST_SCRAPE_RETRY_INTERVAL_SEC;
 
             if(( tr_bencDictFindStr( &benc, "failure reason", &str ))) {
-               // publishErrorMessageAndStop( t, str );
                 publishMessage( t, str, TR_TRACKER_ERROR );
                 success = FALSE;
             }
@@ -359,6 +362,9 @@ onTrackerResponse( tr_session    * session,
             if(( tr_bencDictFindInt( &benc, "incomplete", &i )))
                 t->leecherCount = incomplete = i;
 
+            if(( tr_bencDictFindInt( &benc, "downloaded", &i )))
+                t->timesDownloaded = i;
+
             if(( tmp = tr_bencDictFind( &benc, "peers" )))
             {
                 const int allAreSeeds = incomplete == 0;
@@ -395,9 +401,14 @@ onTrackerResponse( tr_session    * session,
         const int interval = t->announceIntervalSec + t->randOffset;
         const time_t now = time ( NULL );
         dbgmsg( t->name, "request succeeded. reannouncing in %d seconds", interval );
-        t->scrapeAt = now + t->scrapeIntervalSec + t->randOffset;
+        if( t->scrapeAt <= now )
+            t->scrapeAt = now + t->scrapeIntervalSec + t->randOffset;
         t->reannounceAt = now + interval;
         t->manualAnnounceAllowedAt = now + t->announceMinIntervalSec;
+
+        /* #319: save the .resume file after an announce so that, in case
+         * of a crash, our stats still match up with the tracker's stats */
+        tr_torrentSaveResume( tr_torrentFindFromHash( t->session, t->hash ) );
     }
     else if( 300<=responseCode && responseCode<=399 )
     {
@@ -424,7 +435,8 @@ onTrackerResponse( tr_session    * session,
          * incapable of performing the request.  So we pause a bit and
          * try again. */
         t->manualAnnounceAllowedAt = ~(time_t)0;
-        t->reannounceAt = time( NULL ) + 60;
+        t->reannounceAt = time( NULL ) + t->retryAnnounceIntervalSec;
+        t->retryAnnounceIntervalSec *= 2;
     }
     else
     {
@@ -453,13 +465,14 @@ onScrapeResponse( tr_session   * session,
         return;
 
     dbgmsg( t->name, "scrape response: %ld\n", responseCode );
-    tr_ndbg( t->name, "scrape response: %d", responseCode );
+    tr_ndbg( t->name, "scrape response: %ld", responseCode );
     t->lastScrapeResponse = responseCode;
 
     if( responseCode == HTTP_OK )
     {
         tr_benc benc, *files;
-        const int bencLoaded = !tr_bencLoad( response, responseLen, &benc, 0 );
+        const int bencLoaded = !tr_bencLoad( response, responseLen,
+                                             &benc, NULL );
         if( bencLoaded && tr_bencDictFindDict( &benc, "files", &files ) )
         {
             int i;
@@ -488,12 +501,16 @@ onScrapeResponse( tr_session   * session,
                     if(( tr_bencDictFindInt( flags, "min_request_interval", &itmp )))
                         t->scrapeIntervalSec = i;
 
-                success = TRUE;
+                /* as per ticket #1045, safeguard against trackers returning
+                 * a very low min_request_interval... */
+                if( t->scrapeIntervalSec < DEFAULT_SCRAPE_INTERVAL_SEC )
+                    t->scrapeIntervalSec = DEFAULT_SCRAPE_INTERVAL_SEC;
 
                 tr_ndbg( t->name, "Scrape successful.  Rescraping in %d seconds.",
                          t->scrapeIntervalSec );
 
-                t->retryScrapeIntervalSec = 30;
+                success = TRUE;
+                t->retryScrapeIntervalSec = FIRST_SCRAPE_RETRY_INTERVAL_SEC;
             }
         }
 
@@ -526,7 +543,7 @@ onScrapeResponse( tr_session   * session,
     else
     {
         const int interval = t->retryScrapeIntervalSec + t->randOffset;
-        dbgmsg( t->name, "Tracker responded to scrape with %d.  Retrying in %d seconds.",
+        dbgmsg( t->name, "Tracker responded to scrape with %ld.  Retrying in %d seconds.",
                    responseCode,  interval );
         t->retryScrapeIntervalSec *= 2;
         t->scrapeAt = time( NULL ) + interval;
@@ -549,9 +566,9 @@ enum
 
 struct tr_tracker_request
 {
-    char * url;
-    int reqtype; /* TR_REQ_* */
     uint8_t torrent_hash[SHA_DIGEST_LENGTH];
+    int reqtype; /* TR_REQ_* */
+    char * url;
     tr_web_done_func * done_func;
     tr_session * session;
 };
@@ -564,7 +581,7 @@ freeRequest( struct tr_tracker_request * req )
 }
 
 static void
-buildTrackerRequestURI( const tr_tracker  * t,
+buildTrackerRequestURI( tr_tracker        * t,
                         const tr_torrent  * torrent,
                         const char        * eventName,
                         struct evbuffer   * buf )
@@ -588,7 +605,7 @@ buildTrackerRequestURI( const tr_tracker  * t,
         strchr(ann, '?') ? '&' : '?',
         t->escaped,
         t->peer_id,
-        tr_sharedGetPublicPort( t->session->shared ),
+        tr_sessionGetPeerPort( t->session ),
         torrent->uploadedCur,
         torrent->downloadedCur,
         torrent->corruptCur,
@@ -602,7 +619,7 @@ buildTrackerRequestURI( const tr_tracker  * t,
 }
 
 static struct tr_tracker_request*
-createRequest( tr_session * session, const tr_tracker * tracker, int reqtype )
+createRequest( tr_session * session, tr_tracker * tracker, int reqtype )
 {
     static const char* strings[] = { "started", "completed", "stopped", "", "err" };
     const tr_torrent * torrent = tr_torrentFindFromHash( session, tracker->hash );
@@ -627,7 +644,7 @@ createRequest( tr_session * session, const tr_tracker * tracker, int reqtype )
 }
 
 static struct tr_tracker_request*
-createScrape( tr_session * session, const tr_tracker * tracker )
+createScrape( tr_session * session, tr_tracker * tracker )
 {
     const tr_tracker_info * a = getCurrentAddress( tracker );
     struct tr_tracker_request * req;
@@ -650,12 +667,11 @@ createScrape( tr_session * session, const tr_tracker * tracker )
 
 struct tr_tracker_handle
 {
-    unsigned int isShuttingDown : 1;
     int runningCount;
     tr_timer * pulseTimer;
 };
 
-static int pulse( void * vsession );
+static int trackerPulse( void * vsession );
 
 static void
 ensureGlobalsExist( tr_session * session )
@@ -663,35 +679,21 @@ ensureGlobalsExist( tr_session * session )
     if( session->tracker == NULL )
     {
         session->tracker = tr_new0( struct tr_tracker_handle, 1 );
-        session->tracker->pulseTimer = tr_timerNew( session, pulse, session, PULSE_INTERVAL_MSEC );
+        session->tracker->pulseTimer = tr_timerNew( session, trackerPulse, session, PULSE_INTERVAL_MSEC );
         dbgmsg( NULL, "creating tracker timer" );
     }
 }
 
 void
-tr_trackerShuttingDown( tr_session * session )
+tr_trackerSessionClose( tr_session * session )
 {
-    if( session->tracker )
-        session->tracker->isShuttingDown = 1;
-}
-
-static int
-maybeFreeGlobals( tr_session * session )
-{
-    int globalsExist = session->tracker != NULL;
-
-    if( globalsExist
-        && ( session->tracker->runningCount < 1 )
-        && ( session->torrentList== NULL ) )
+    if( session && session->tracker )
     {
         dbgmsg( NULL, "freeing tracker timer" );
         tr_timerFree( &session->tracker->pulseTimer );
         tr_free( session->tracker );
         session->tracker = NULL;
-        globalsExist = FALSE;
     }
-
-    return globalsExist;
 }
 
 /***
@@ -702,7 +704,6 @@ static void
 invokeRequest( void * vreq )
 {
     struct tr_tracker_request * req = vreq;
-    uint8_t * hash;
     tr_tracker * t = findTracker( req->session, req->torrent_hash );
 
     if( t )
@@ -712,12 +713,13 @@ invokeRequest( void * vreq )
         if( req->reqtype == TR_REQ_SCRAPE )
         {
             t->lastScrapeTime = now;
-            t->scrapeAt = 0;
+            t->scrapeAt = 1;
         }
         else
         {
             t->lastAnnounceTime = now;
-            t->reannounceAt = 0;
+            t->reannounceAt = 1;
+            t->manualAnnounceAllowedAt = 1;
             t->scrapeAt = req->reqtype == TR_REQ_STOPPED
                         ? now + t->scrapeIntervalSec + t->randOffset
                         : 0;
@@ -726,35 +728,30 @@ invokeRequest( void * vreq )
 
     ++req->session->tracker->runningCount;
 
-    hash = tr_new0( uint8_t, SHA_DIGEST_LENGTH );
-    memcpy( hash, req->torrent_hash, SHA_DIGEST_LENGTH );
-    tr_webRun( req->session, req->url, req->done_func, hash );
+    tr_webRun( req->session, req->url, NULL, req->done_func,
+               tr_memdup( req->torrent_hash, SHA_DIGEST_LENGTH ) );
 
     freeRequest( req );
 }
 
-static void ensureGlobalsExist( tr_session * );
-
 static void
-enqueueScrape( tr_session * session, const tr_tracker * tracker )
+enqueueScrape( tr_session * session, tr_tracker * tracker )
 {
     struct tr_tracker_request * req;
-    ensureGlobalsExist( session );
     req = createScrape( session, tracker );
     tr_runInEventThread( session, invokeRequest, req );
 }
 
 static void
-enqueueRequest( tr_session * session, const tr_tracker * tracker, int reqtype )
+enqueueRequest( tr_session * session, tr_tracker * tracker, int reqtype )
 {
     struct tr_tracker_request * req;
-    ensureGlobalsExist( session );
     req = createRequest( session, tracker, reqtype );
     tr_runInEventThread( session, invokeRequest, req );
 }
 
 static int
-pulse( void * vsession )
+trackerPulse( void * vsession )
 {
     tr_session * session = vsession;
     struct tr_tracker_handle * th = session->tracker;
@@ -772,13 +769,20 @@ pulse( void * vsession )
     {
         tr_tracker * t = tor->tracker;
 
-        if( t->scrapeAt && trackerSupportsScrape( t, tor ) && ( now >= t->scrapeAt ) ) {
-            t->scrapeAt = 0;
+        if( ( t->scrapeAt > 1 ) &&
+            ( t->scrapeAt <= now ) &&
+            ( trackerSupportsScrape( t, tor ) ) )
+        {
+            t->scrapeAt = 1;
             enqueueScrape( session, t );
         }
 
-        if( t->reannounceAt && t->isRunning && ( now >= t->reannounceAt ) ) {
-            t->reannounceAt = 0;
+        if( ( t->reannounceAt > 1 ) && 
+            ( t->reannounceAt <= now ) &&
+            ( t->isRunning ) )
+        {
+            t->reannounceAt = 1;
+            t->manualAnnounceAllowedAt = 1;
             enqueueRequest( session, t, TR_REQ_REANNOUNCE );
         }
     }
@@ -786,7 +790,18 @@ pulse( void * vsession )
     if( th->runningCount )
         dbgmsg( NULL, "tracker pulse after upkeep... %d running", th->runningCount );
 
-    return maybeFreeGlobals( session );
+    /* free the tracker manager if no torrents are left */
+    if(    ( session->tracker )
+        && ( session->tracker->runningCount < 1 )
+        && ( session->torrentList == NULL ) )
+    {
+        tr_trackerSessionClose( session );
+    }
+
+    /* if there are still running torrents (as indicated by
+     * the existence of the tracker manager) then keep the
+     * trackerPulse() timer alive */
+    return session->tracker != NULL;
 }
 
 static void
@@ -796,7 +811,7 @@ onReqDone( tr_session * session )
     {
         --session->tracker->runningCount;
         dbgmsg( NULL, "decrementing runningCount to %d", session->tracker->runningCount );
-        pulse( session );
+        trackerPulse( session );
     }
 }
 
@@ -831,7 +846,7 @@ escape( char * out, const uint8_t * in, int in_len ) /* rfc2396 */
         if( is_rfc2396_alnum(*in) )
             *out++ = (char) *in++;
         else 
-            out += snprintf( out, 4, "%%%02X", (unsigned int)*in++ );
+            out += tr_snprintf( out, 4, "%%%02X", (unsigned int)*in++ );
     *out = '\0';
 }
 
@@ -841,11 +856,14 @@ tr_trackerNew( const tr_torrent * torrent )
     const tr_info * info = &torrent->info;
     tr_tracker * t;
 
+    ensureGlobalsExist( torrent->handle );
+
     t = tr_new0( tr_tracker, 1 );
     t->publisher = tr_publisherNew( );
     t->session                  = torrent->handle;
     t->scrapeIntervalSec        = DEFAULT_SCRAPE_INTERVAL_SEC;
-    t->retryScrapeIntervalSec   = 60;
+    t->retryScrapeIntervalSec   = FIRST_SCRAPE_RETRY_INTERVAL_SEC;
+    t->retryAnnounceIntervalSec = FIRST_ANNOUNCE_RETRY_INTERVAL_SEC;
     t->announceIntervalSec      = DEFAULT_ANNOUNCE_INTERVAL_SEC;
     t->announceMinIntervalSec   = DEFAULT_ANNOUNCE_MIN_INTERVAL_SEC;
     t->timesDownloaded          = -1;
@@ -909,7 +927,7 @@ tr_trackerUnsubscribe( tr_tracker        * t,
 }
 
 const tr_tracker_info *
-tr_trackerGetAddress( const tr_tracker   * t )
+tr_trackerGetAddress( tr_tracker   * t )
 {
     return getCurrentAddress( t );
 }
@@ -987,8 +1005,8 @@ tr_trackerChangeMyPort( tr_tracker * t )
 }
 
 void
-tr_trackerStat( const tr_tracker       * t,
-                struct tr_tracker_stat * setme)
+tr_trackerStat( const tr_tracker * t,
+                struct tr_stat   * setme)
 {
     assert( t );
     assert( setme );
@@ -997,23 +1015,23 @@ tr_trackerStat( const tr_tracker       * t,
     setme->nextScrapeTime = t->scrapeAt;
     setme->lastAnnounceTime = t->lastAnnounceTime;
     setme->nextAnnounceTime = t->reannounceAt;
-    setme->nextManualAnnounceTime = t->manualAnnounceAllowedAt;
+    setme->manualAnnounceTime = t->manualAnnounceAllowedAt;
 
     if( t->lastScrapeResponse == -1 ) /* never been scraped */
         *setme->scrapeResponse = '\0';
     else
-        snprintf( setme->scrapeResponse,
-                  sizeof( setme->scrapeResponse ),
-                  "%s (%ld)",
-                  tr_webGetResponseStr( t->lastScrapeResponse ),
-                  t->lastScrapeResponse );
+        tr_snprintf( setme->scrapeResponse,
+                     sizeof( setme->scrapeResponse ),
+                     "%s (%ld)",
+                     tr_webGetResponseStr( t->lastScrapeResponse ),
+                     t->lastScrapeResponse );
 
     if( t->lastAnnounceResponse == -1 ) /* never been announced */
         *setme->announceResponse = '\0';
     else
-        snprintf( setme->announceResponse,
-                  sizeof( setme->announceResponse ),
-                  "%s (%ld)",
-                  tr_webGetResponseStr( t->lastAnnounceResponse ),
-                  t->lastAnnounceResponse );
+        tr_snprintf( setme->announceResponse,
+                     sizeof( setme->announceResponse ),
+                     "%s (%ld)",
+                     tr_webGetResponseStr( t->lastAnnounceResponse ),
+                     t->lastAnnounceResponse );
 }
