@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: utils.c 5890 2008-05-21 20:56:10Z charles $
+ * $Id: utils.c 6768 2008-09-16 17:50:16Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -23,7 +23,7 @@
  *****************************************************************************/
 
 #include <assert.h>
-#include <ctype.h> /* isalpha */
+#include <ctype.h> /* isalpha, tolower */
 #include <errno.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -45,7 +45,6 @@
 #endif
 
 #include "transmission.h"
-#include "trcompat.h"
 #include "utils.h"
 #include "platform.h"
 
@@ -168,7 +167,7 @@ tr_getLogTimeStr( char * buf, int buflen )
 #endif
     strftime( tmp, sizeof(tmp), "%H:%M:%S", &now_tm );
     milliseconds = (int)(tv.tv_usec / 1000);
-    snprintf( buf, buflen, "%s.%03d", tmp, milliseconds );
+    tr_snprintf( buf, buflen, "%s.%03d", tmp, milliseconds );
 
     return buf;
 }
@@ -177,7 +176,7 @@ void
 tr_deepLog( const char * file, int line, const char * name, const char * fmt, ... )
 {
     FILE * fp = tr_getLog( );
-    if( fp != NULL )
+    if( fp )
     {
         va_list args;
         char timestr[64];
@@ -239,7 +238,7 @@ tr_msg( const char * file, int line, int level,
                 newmsg = tr_new0( tr_msg_list, 1 );
                 newmsg->level = level;
                 newmsg->when = time( NULL );
-                newmsg->message = tr_strdup( (char*)EVBUFFER_DATA( buf ) );
+                newmsg->message = tr_strdup( EVBUFFER_DATA( buf ) );
                 newmsg->file = file;
                 newmsg->line = line;
                 newmsg->name = tr_strdup( name );
@@ -251,7 +250,10 @@ tr_msg( const char * file, int line, int level,
             {
                 if( fp == NULL )
                     fp = stderr;
-                fprintf( fp, "%s\n", (char*)EVBUFFER_DATA(buf) );
+                if( name )
+                    fprintf( fp, "%s: %s\n", name, (char*)EVBUFFER_DATA(buf) );
+                else
+                    fprintf( fp, "%s\n", (char*)EVBUFFER_DATA(buf) );
                 fflush( fp );
             }
 
@@ -352,6 +354,47 @@ tr_compareUint32( uint32_t a, uint32_t b )
     return 0;
 }
 
+int
+tr_compareUint64( uint64_t a, uint64_t b )
+{
+    if( a < b ) return -1;
+    if( a > b ) return 1;
+    return 0;
+}
+
+int
+tr_compareDouble( double a, double b )
+{
+    if( a < b ) return -1;
+    if( a > b ) return 1;
+    return 0;
+}
+
+int
+tr_strcmp( const void * a, const void * b )
+{
+    if( a && b ) return strcmp( a, b );
+    if( a ) return 1;
+    if( b ) return -1;
+    return 0;
+}
+
+int
+tr_strcasecmp( const char * a, const char * b )
+{
+    if( !a && !b ) return 0;
+    if( !a ) return -1;
+    if( !b ) return 1;
+#ifdef HAVE_STRCASECMP
+    return strcasecmp( a, b );
+#else
+    while( *a && ( tolower( *(uint8_t*)a ) == tolower( *(uint8_t*)b ) ) )
+        ++a, ++b;
+    return tolower( *(uint8_t*)a) - tolower(*(uint8_t*)b );
+#endif
+}
+
+
 /**
 ***
 **/
@@ -470,7 +513,7 @@ tr_mkdirp( const char * path_in, int permissions )
         {
             /* Node exists but isn't a folder */
             char buf[MAX_PATH_LENGTH];
-            snprintf( buf, sizeof( buf ), _( "File \"%s\" is in the way" ), path );
+            tr_snprintf( buf, sizeof( buf ), _( "File \"%s\" is in the way" ), path );
             tr_err( _( "Couldn't create \"%1$s\": %2$s" ), path_in, buf );
             tr_free( path );
             errno = ENOTDIR;
@@ -492,10 +535,13 @@ tr_mkdirp( const char * path_in, int permissions )
 void
 tr_buildPath ( char *buf, size_t buflen, const char *first_element, ... )
 {
-    struct evbuffer * evbuf = evbuffer_new( );
+    struct evbuffer * evbuf;
     const char * element = first_element;
     va_list vl;
+
+    evbuf = evbuffer_new( );
     va_start( vl, first_element );
+
     while( element ) {
         if( EVBUFFER_LENGTH(evbuf) )
             evbuffer_add_printf( evbuf, "%c", TR_PATH_DELIMITER );
@@ -503,9 +549,11 @@ tr_buildPath ( char *buf, size_t buflen, const char *first_element, ... )
         element = (const char*) va_arg( vl, const char* );
     }
     if( EVBUFFER_LENGTH(evbuf) )
-        strlcpy( buf, (char*)EVBUFFER_DATA(evbuf), buflen );
+        tr_strlcpy( buf, (char*)EVBUFFER_DATA(evbuf), buflen );
     else
         *buf = '\0';
+
+    va_end( vl );
     evbuffer_free( evbuf );
 }
 
@@ -526,7 +574,7 @@ tr_ioErrorFromErrno( int err )
         case EFBIG:
             return TR_ERROR_IO_FILE_TOO_BIG;
         default:
-            tr_dbg( "generic i/o errno from errno: %s", tr_strerror( errno ) );
+            tr_err( "generic i/o errno from errno: %s", tr_strerror( errno ) );
             return TR_ERROR_IO_OTHER;
     }
 }
@@ -578,14 +626,22 @@ tr_errorString( int code )
 *****
 ****/
 
-char*
-tr_strdup( const char * in )
+void*
+tr_memdup( const void * in, int byteCount )
 {
-    return tr_strndup( in, in ? strlen(in) : 0 );
+    void * out = tr_new( uint8_t, byteCount );
+    memcpy( out, in, byteCount );
+    return out;
+}
+    
+char*
+tr_strdup( const void * in )
+{
+    return tr_strndup( in, in ? strlen((const char*)in) : 0 );
 }
 
 char*
-tr_strndup( const char * in, int len )
+tr_strndup( const void * in, int len )
 {
     char * out = NULL;
 
@@ -593,15 +649,34 @@ tr_strndup( const char * in, int len )
     {
         out = tr_strdup( in );
     }
-    else if( in != NULL )
+    else if( in )
     {
         out = tr_malloc( len+1 );
         memcpy( out, in, len );
         out[len] = '\0';
     }
+
     return out;
 }
 
+char*
+tr_strdup_printf( const char * fmt, ... )
+{
+    char * ret = NULL;
+    struct evbuffer * buf;
+    va_list ap;
+
+    buf = evbuffer_new( );
+    va_start( ap, fmt );
+
+    if( evbuffer_add_vprintf( buf, fmt, ap ) != -1 )
+        ret = tr_strdup( (char*)EVBUFFER_DATA( buf ) );
+
+    va_end( ap );
+    evbuffer_free( buf );
+    return ret;
+}
+
 void*
 tr_calloc( size_t nmemb, size_t size )
 {
@@ -642,21 +717,13 @@ tr_strerror( int i )
 *****
 ****/
 
-/* note that the argument is how many bits are needed, not bytes */
 tr_bitfield*
-tr_bitfieldNew( size_t bitcount )
+tr_bitfieldNew( size_t bitCount )
 {
-    tr_bitfield * ret = calloc( 1, sizeof(tr_bitfield) );
-    if( NULL == ret )
-        return NULL;
-
-    ret->len = (bitcount+7u) / 8u;
-    ret->bits = calloc( ret->len, 1 );
-    if( NULL == ret->bits ) {
-        free( ret );
-        return NULL;
-    }
-
+    tr_bitfield * ret = tr_new0( tr_bitfield, 1 );
+    ret->bitCount = bitCount;
+    ret->byteCount = (bitCount+7u) / 8u;
+    ret->bits = tr_new0( uint8_t, ret->byteCount );
     return ret;
 }
 
@@ -664,25 +731,26 @@ tr_bitfield*
 tr_bitfieldDup( const tr_bitfield * in )
 {
     tr_bitfield * ret = calloc( 1, sizeof(tr_bitfield) );
-    ret->len = in->len;
-    ret->bits = malloc( ret->len );
-    memcpy( ret->bits, in->bits, ret->len );
+    ret->bitCount = in->bitCount;
+    ret->byteCount = in->byteCount;
+    ret->bits = tr_memdup( in->bits, in->byteCount );
     return ret;
 }
 
-void tr_bitfieldFree( tr_bitfield * bitfield )
+void
+tr_bitfieldFree( tr_bitfield * bitfield )
 {
     if( bitfield )
     {
-        free( bitfield->bits );
-        free( bitfield );
+        tr_free( bitfield->bits );
+        tr_free( bitfield );
     }
 }
 
 void
 tr_bitfieldClear( tr_bitfield * bitfield )
 {
-    memset( bitfield->bits, 0, bitfield->len );
+    memset( bitfield->bits, 0, bitfield->byteCount );
 }
 
 int
@@ -690,7 +758,7 @@ tr_bitfieldIsEmpty( const tr_bitfield * bitfield )
 {
     size_t i;
 
-    for( i=0; i<bitfield->len; ++i )
+    for( i=0; i<bitfield->byteCount; ++i )
         if( bitfield->bits[i] )
             return 0;
 
@@ -700,28 +768,67 @@ tr_bitfieldIsEmpty( const tr_bitfield * bitfield )
 int
 tr_bitfieldHas( const tr_bitfield * bitfield, size_t nth )
 {
-    static const uint8_t ands[8] = { 128, 64, 32, 16, 8, 4, 2, 1 };
-    const size_t i = nth >> 3u;
-    return ( bitfield != NULL )
-        && ( bitfield->bits != NULL )
-        && ( i < bitfield->len )
-        && ( ( bitfield->bits[i] & ands[nth&7u] ) != 0 );
+    return ( tr_bitfieldTestFast( bitfield, nth ) )
+        && ( tr_bitfieldHasFast( bitfield, nth ) );
 }
 
+#if 0
+static int
+find_top_bit( uint8_t val )
+{
+    int pos = 0;
+    if ( val & 0xF0U ) pos |= 4, val >>= 4;
+    if ( val & 0xCU )  pos |= 2, val >>= 2;
+    if ( val & 0x2 )   pos |= 1;
+    return 7 - pos;
+}
+
+int
+tr_bitfieldFindTrue( const tr_bitfield  * bitfield,
+                     size_t               startBit,
+                     size_t             * setmePos )
+{
+    if( bitfield && bitfield->bits && startBit < bitfield->bitCount )
+    {
+        const uint8_t * b   = bitfield->bits + startBit/8;
+        const uint8_t * end = bitfield->bits + bitfield->byteCount;
+
+        /* If first byte already contains a set bit after startBit*/
+        if( *b & ( 0xff >> (startBit&7) ) ) {
+            *setmePos  = 8 * ( b - bitfield->bits );
+            *setmePos += find_top_bit( *b & ( 0xff >> (startBit&7) ) );
+            return 1;
+        }
+
+        /* Test bitfield for first non zero byte */
+        ++b;
+        while( (b < end) && !*b )
+            ++b;
+
+        /* If we hit the end of our bitfield, no set bit was found */
+        if( b == end )
+            return 0;
+
+        /* New bitposition is byteoff*8 */
+        *setmePos = 8 * ( b - bitfield->bits ) + find_top_bit( *b );
+
+        return 1;
+    }
+
+    return 0;
+}
+#endif
+
 int
 tr_bitfieldAdd( tr_bitfield  * bitfield, size_t nth )
 {
-    static const uint8_t ands[8] = { 128, 64, 32, 16, 8, 4, 2, 1 };
-    const size_t i = nth >> 3u;
+    assert( bitfield );
+    assert( bitfield->bits );
 
-    assert( bitfield != NULL );
-    assert( bitfield->bits != NULL );
-
-    if( i >= bitfield->len )
+    if( nth >= bitfield->bitCount )
         return -1;
 
-    bitfield->bits[i] |= ands[nth&7u];
-    assert( tr_bitfieldHas( bitfield, nth ) );
+    bitfield->bits[nth>>3u] |= (0x80 >> (nth&7u));
     return 0;
 }
 
@@ -742,17 +849,13 @@ int
 tr_bitfieldRem( tr_bitfield   * bitfield,
                 size_t          nth )
 {
-    static const uint8_t rems[8] = { 127, 191, 223, 239, 247, 251, 253, 254 };
-    const size_t i = nth >> 3u;
+    assert( bitfield );
+    assert( bitfield->bits );
 
-    assert( bitfield != NULL );
-    assert( bitfield->bits != NULL );
-
-    if( i >= bitfield->len )
+    if( nth >= bitfield->bitCount )
         return -1;
 
-    bitfield->bits[i] &= rems[nth&7u];
-    assert( !tr_bitfieldHas( bitfield, nth ) );
+    bitfield->bits[nth>>3u] &= (0xff7f >> (nth&7u));
     return 0;
 }
 
@@ -775,9 +878,9 @@ tr_bitfieldOr( tr_bitfield * a, const tr_bitfield * b )
     uint8_t *ait;
     const uint8_t *aend, *bit;
 
-    assert( a->len == b->len );
+    assert( a->bitCount == b->bitCount );
 
-    for( ait=a->bits, bit=b->bits, aend=ait+a->len; ait!=aend; )
+    for( ait=a->bits, bit=b->bits, aend=ait+a->byteCount; ait!=aend; )
         *ait++ |= *bit++;
 
     return a;
@@ -790,9 +893,9 @@ tr_bitfieldDifference( tr_bitfield * a, const tr_bitfield * b )
     uint8_t *ait;
     const uint8_t *aend, *bit;
 
-    assert( a->len == b->len );
+    assert( a->bitCount == b->bitCount );
 
-    for( ait=a->bits, bit=b->bits, aend=ait+a->len; ait!=aend; )
+    for( ait=a->bits, bit=b->bits, aend=ait+a->byteCount; ait!=aend; )
         *ait++ &= ~(*bit++);
 }
 
@@ -824,7 +927,7 @@ tr_bitfieldCountTrueBits( const tr_bitfield* b )
     if( !b )
         return 0;
 
-    for( it=b->bits, end=it+b->len; it!=end; ++it )
+    for( it=b->bits, end=it+b->byteCount; it!=end; ++it )
         ret += trueBitCount[*it];
 
     return ret;
@@ -858,8 +961,25 @@ tr_wait( uint64_t delay_milliseconds )
 ****
 ***/
 
+int
+tr_stringEndsWith( const char * str, const char * end )
+{
+    const size_t slen = strlen( str );
+    const size_t elen = strlen( end );
+    return slen>=elen && !memcmp( &str[slen-elen], end, elen );
+}
+
+int
+tr_snprintf( char * buf, size_t buflen, const char * fmt, ... )
+{
+    int len;
+    va_list args;
+    va_start( args, fmt );
+    len = evutil_vsnprintf( buf, buflen, fmt, args );
+    va_end( args );
+    return len;
+}
 
-#ifndef HAVE_STRLCPY
 
 /*
  * Copy src to string dst of size siz.  At most siz-1 characters
@@ -867,36 +987,38 @@ tr_wait( uint64_t delay_milliseconds )
  * Returns strlen(src); if retval >= siz, truncation occurred.
  */
 size_t
-strlcpy(char *dst, const char *src, size_t siz)
+tr_strlcpy(char *dst, const char *src, size_t siz)
 {
-	char *d = dst;
-	const char *s = src;
-	size_t n = siz;
+#ifdef HAVE_STRLCPY
+    return strlcpy( dst, src, siz );
+#else
+    char *d = dst;
+    const char *s = src;
+    size_t n = siz;
 
-	assert( s != NULL );
-	assert( d != NULL );
+    assert( s );
+    assert( d );
 
-	/* Copy as many bytes as will fit */
-	if (n != 0) {
-		while (--n != 0) {
-			if ((*d++ = *s++) == '\0')
-				break;
-		}
-	}
+    /* Copy as many bytes as will fit */
+    if (n != 0) {
+        while (--n != 0) {
+            if ((*d++ = *s++) == '\0')
+                break;
+        }
+    }
 
-	/* Not enough room in dst, add NUL and traverse rest of src */
-	if (n == 0) {
-		if (siz != 0)
-			*d = '\0';		/* NUL-terminate dst */
-		while (*s++)
-			;
-	}
+    /* Not enough room in dst, add NUL and traverse rest of src */
+    if (n == 0) {
+        if (siz != 0)
+            *d = '\0'; /* NUL-terminate dst */
+        while (*s++)
+            ;
+    }
 
-	return(s - src - 1);	/* count does not include NUL */
+    return(s - src - 1); /* count does not include NUL */
+#endif
 }
 
-#endif /* HAVE_STRLCPY */
-
 /***
 ****
 ***/
@@ -946,6 +1068,9 @@ tr_httpIsValidURL( const char * url )
         "<>#%<\""                    /* delims */
         "{}|\\^[]`";                 /* unwise */
 
+    if( url == NULL )
+        return FALSE;
+
     for( c=url; c && *c; ++c )
         if( !strchr( rfc2396_valid_chars, *c ) )
             return FALSE;
@@ -1011,3 +1136,69 @@ tr_httpParseURL( const char * url_in, int len,
     tr_free( tmp );
     return err;
 }
+
+#include <string.h>
+#include <openssl/sha.h>
+#include <openssl/hmac.h>
+#include <openssl/evp.h>
+#include <openssl/bio.h>
+#include <openssl/buffer.h>
+
+char *
+tr_base64_encode( const void * input, int length, int * setme_len )
+{
+    char * ret;
+    BIO * b64;
+    BIO * bmem;
+    BUF_MEM * bptr;
+
+    if( length < 1 )
+       length = strlen( input );
+
+    bmem = BIO_new( BIO_s_mem( ) );
+    b64 = BIO_new( BIO_f_base64( ) );
+    b64 = BIO_push( b64, bmem );
+    BIO_write( b64, input, length );
+    (void) BIO_flush( b64 );
+    BIO_get_mem_ptr( b64, &bptr );
+    ret = tr_strndup( bptr->data, bptr->length );
+    if( setme_len )
+        *setme_len = bptr->length;
+
+    BIO_free_all( b64 );
+    return ret;
+}
+
+char *
+tr_base64_decode( const void * input, int length, int * setme_len )
+{
+    char * ret;
+    BIO * b64;
+    BIO * bmem;
+    int retlen;
+
+    if( length < 1 )
+       length = strlen( input );
+
+    ret = tr_new0( char, length );
+    b64 = BIO_new( BIO_f_base64( ) );
+    bmem = BIO_new_mem_buf( (unsigned char*)input, length );
+    bmem = BIO_push( b64, bmem );
+    retlen = BIO_read( bmem, ret, length );
+    if( !retlen )
+    {
+        /* try again, but with the BIO_FLAGS_BASE64_NO_NL flag */
+        BIO_free_all( bmem );
+        b64 = BIO_new( BIO_f_base64( ) );
+        BIO_set_flags( b64, BIO_FLAGS_BASE64_NO_NL );
+        bmem = BIO_new_mem_buf( (unsigned char*)input, length );
+        bmem = BIO_push( b64, bmem );
+        retlen = BIO_read( bmem, ret, length );
+    }
+
+    if( setme_len )
+        *setme_len = retlen;
+
+    BIO_free_all( bmem );
+    return ret;
+}
