@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: utils.h 5759 2008-05-06 19:06:46Z charles $
+ * $Id: utils.h 6768 2008-09-16 17:50:16Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -29,13 +29,61 @@
 #include <stdarg.h>
 #include <stddef.h> /* for size_t */
 #include <stdio.h> /* FILE* */
+#include <time.h> /* time_t* */
 
-void tr_msgInit( void );
+/***
+****
+***/
+
+#ifndef FALSE
+#define FALSE 0
+#endif
+
+#ifndef TRUE
+#define TRUE 1
+#endif
+
+#ifndef UNUSED
+#ifdef __GNUC__
+#define UNUSED __attribute__((unused))
+#else
+#define UNUSED
+#endif
+#endif
+
+#ifndef TR_GNUC_PRINTF
+#ifdef __GNUC__
+#define TR_GNUC_PRINTF( fmt, args ) __attribute__((format (printf, fmt, args)))
+#else
+#define TR_GNUC_PRINTF( fmt, args )
+#endif
+#endif
+
+#ifndef TR_GNUC_NULL_TERMINATED
+#if __GNUC__ >= 4
+#define TR_GNUC_NULL_TERMINATED __attribute__((__sentinel__))
+#else
+#define TR_GNUC_NULL_TERMINATED
+#endif
+#endif
+
+#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 96)
+#define TR_GNUC_PURE __attribute__((__pure__))
+#define TR_GNUC_MALLOC __attribute__((__malloc__))
+#else
+#define TR_GNUC_PURE
+#define TR_GNUC_MALLOC
+#endif
+
+
+/***
+****
+***/
 
 #if !defined(_)
 #if defined(SYS_DARWIN)
   #define _(a) (a)
-#elif HAVE_LIBINTL
+#elif defined(HAVE_LIBINTL_H)
   #include <libintl.h>
   #define _(a) gettext (a)
 #else
@@ -54,10 +102,18 @@ void tr_msgInit( void );
 #define tr_err( a... ) tr_msg( __FILE__, __LINE__, TR_MSG_ERR, NULL, ## a )
 #define tr_inf( a... ) tr_msg( __FILE__, __LINE__, TR_MSG_INF, NULL, ## a )
 #define tr_dbg( a... ) tr_msg( __FILE__, __LINE__, TR_MSG_DBG, NULL, ## a )
-void tr_msg  ( const char * file, int line, int level, const char * torrent, const char * fmt, ... );
+
+void tr_msgInit( void );
+
+void tr_msg( const char * file, int line,
+             int level, const char * torrent,
+             const char * fmt, ... ) TR_GNUC_PRINTF( 5, 6 );
+
 FILE* tr_getLog( void );
 
-void tr_deepLog( const char * file, int line, const char * name, const char * fmt, ... ); 
+void tr_deepLog( const char * file, int line,
+                 const char * name,
+                 const char * fmt, ... ) TR_GNUC_PRINTF( 4, 5 );
 
 char* tr_getLogTimeStr( char * buf, int buflen );
 
@@ -82,13 +138,14 @@ int tr_mkdir( const char * path, int permissions );
 int tr_mkdirp( const char * path, int permissions );
 
 
-uint8_t* tr_loadFile( const char * filename, size_t * size );
+uint8_t* tr_loadFile( const char * filename, size_t * size ) TR_GNUC_MALLOC;
 
 
 /* creates a filename from a series of elements using the
    correct separator for filenames. */
 void tr_buildPath( char* buf, size_t buflen,
-                   const char * first_element, ... );
+                   const char * first_element, ... )
+                                      TR_GNUC_NULL_TERMINATED;
 
 struct timeval tr_timevalMsec( uint64_t milliseconds );
 
@@ -127,13 +184,22 @@ void tr_wait( uint64_t delay_milliseconds );
 #define tr_renew(struct_type, mem, n_structs)    \
     ((struct_type *) realloc ((mem), ((size_t) sizeof (struct_type)) * ((size_t) (n_structs))))
 
-void* tr_malloc  ( size_t );
-void* tr_malloc0 ( size_t );
-void* tr_calloc  ( size_t nmemb, size_t size );
+void* tr_malloc  ( size_t ) TR_GNUC_MALLOC;
+void* tr_malloc0 ( size_t ) TR_GNUC_MALLOC;
+void* tr_calloc  ( size_t nmemb, size_t size ) TR_GNUC_MALLOC;
 void  tr_free    ( void* );
 
-char* tr_strdup( const char * str );
-char* tr_strndup( const char * str, int len );
+char* tr_strdup( const void * str ) TR_GNUC_MALLOC;
+char* tr_strndup( const void * str, int len ) TR_GNUC_MALLOC;
+void* tr_memdup( const void * src, int byteCount ) TR_GNUC_MALLOC;
+char* tr_strdup_printf( const char * fmt, ... )  TR_GNUC_PRINTF( 1, 2 ) TR_GNUC_MALLOC;
+char* tr_base64_encode( const void * input, int inlen, int *outlen ) TR_GNUC_MALLOC;
+char* tr_base64_decode( const void * input, int inlen, int *outlen ) TR_GNUC_MALLOC;
+
+size_t tr_strlcpy( char * dst, const char * src, size_t siz );
+int tr_snprintf( char * buf, size_t buflen, const char * fmt, ... );
+
+int   tr_stringEndsWith( const char * string, const char * end );
 
 const char* tr_strerror( int );
 
@@ -154,6 +220,11 @@ void tr_set_compare( const void * a, size_t aCount,
                     
 int tr_compareUint16( uint16_t a, uint16_t b );
 int tr_compareUint32( uint32_t a, uint32_t b );
+int tr_compareUint64( uint64_t a, uint64_t b );
+int tr_compareDouble( double a, double b );
+
+int tr_strcmp( const void * a, const void * b );
+int tr_strcasecmp( const char * a, const char * b );
 
 void tr_sha1_to_hex( char * out, const uint8_t * sha1 );
 
@@ -174,14 +245,14 @@ int tr_httpParseURL( const char * url,
 struct tr_bitfield
 {
     uint8_t * bits;
-    size_t len;
+    size_t bitCount;
+    size_t byteCount;
 };
 
 typedef struct tr_bitfield tr_bitfield;
-typedef struct tr_bitfield tr_bitfield_t;
 
-tr_bitfield* tr_bitfieldNew( size_t bitcount );
-tr_bitfield* tr_bitfieldDup( const tr_bitfield* );
+tr_bitfield* tr_bitfieldNew( size_t bitcount ) TR_GNUC_MALLOC;
+tr_bitfield* tr_bitfieldDup( const tr_bitfield* ) TR_GNUC_MALLOC;
 void tr_bitfieldFree( tr_bitfield*);
 
 void tr_bitfieldClear( tr_bitfield* );
@@ -197,6 +268,18 @@ size_t tr_bitfieldCountTrueBits( const tr_bitfield* );
 
 tr_bitfield* tr_bitfieldOr( tr_bitfield*, const tr_bitfield* );
 
+/** A stripped-down version of bitfieldHas to be used
+    for speed when you're looping quickly.  This version
+    has none of tr_bitfieldHas()'s safety checks, so you
+    need to call tr_bitfieldTestFast() first before you
+    start looping. */
+#define tr_bitfieldHasFast(bitfield,nth) \
+    (( bitfield->bits[(nth)>>3u] << ((nth)&7u) & 0x80) != 0 )
+
+/** @param high the highest nth bit you're going to access */
+#define tr_bitfieldTestFast(bitfield,high) \
+    ( (bitfield) && ((bitfield)->bits) && ((high)<(bitfield)->bitCount ) )
+
 double tr_getRatio( double numerator, double denominator );
 
 #endif
