@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: web.h 5714 2008-04-28 13:29:38Z charles $
+ * $Id: web.h 6073 2008-06-07 21:26:41Z charles $
  */
 
 #ifndef TR_HTTP_H
@@ -16,20 +16,21 @@
 struct tr_handle;
 typedef struct tr_web tr_web;
 
-tr_web* tr_webInit( tr_handle * session );
+tr_web* tr_webInit( struct tr_handle * session );
 
 void tr_webClose( tr_web ** );
 
-typedef void (tr_web_done_func)( tr_handle    * session,
-                                  long          response_code,
-                                  const void  * response,
-                                  size_t        response_byte_count,
-                                  void        * user_data );
+typedef void (tr_web_done_func)( struct tr_handle    * session,
+                                 long                  response_code,
+                                 const void          * response,
+                                 size_t                response_byte_count,
+                                 void                * user_data );
 
 const char * tr_webGetResponseStr( long response_code );
                                
-void tr_webRun( tr_handle          * session,
+void tr_webRun( struct tr_handle   * session,
                 const char         * url,
+                const char         * range,
                 tr_web_done_func     done_func,
                 void               * done_func_user_data );
 
