@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: InfoWindowController.h 5139 2008-02-27 13:52:43Z livings124 $
+ * $Id: InfoWindowController.h 6219 2008-06-17 21:18:00Z livings124 $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -28,9 +28,12 @@
 #import "PiecesView.h"
 #import <transmission.h>
 
+@class TrackerTableView;
+
 @interface InfoWindowController : NSWindowController
 {
-    NSArray * fTorrents, * fPeers, * fTrackers;
+    NSArray * fTorrents;
+    NSMutableArray * fTrackers;
     
     IBOutlet NSView * fInfoView, * fActivityView, * fTrackerView, * fPeersView, * fFilesView, * fOptionsView;
     int fCurrentTabTag;
@@ -49,22 +52,26 @@
     IBOutlet NSTextView * fCommentView;
     IBOutlet NSButton * fRevealDataButton, * fRevealTorrentButton;
     
-    IBOutlet NSTableView * fTrackerTable;
+    IBOutlet TrackerTableView * fTrackerTable;
+    IBOutlet NSSegmentedControl * fTrackerAddRemoveControl;
     IBOutlet NSTextField * fAnnounceAddressField, * fAnnounceLastField, * fAnnounceResponseField, * fAnnounceNextField,
                             * fScrapeAddressField, * fScrapeLastField, * fScrapeResponseField, * fScrapeNextField;
-
-    IBOutlet NSTableView * fPeerTable;
+    
+    NSArray * fPeers, * fWebSeeds;
+    IBOutlet NSTableView * fPeerTable, * fWebSeedTable;
     IBOutlet NSTextField * fConnectedPeersField, * fDownloadingFromField, * fUploadingToField, * fKnownField,
                             * fSeedersField, * fLeechersField, * fCompletedFromTrackerField;
     IBOutlet NSTextView * fErrorMessageView;
     IBOutlet PiecesView * fPiecesView;
     IBOutlet NSSegmentedControl * fPiecesControl;
+    float fWebSeedTableHeight, fSpaceBetweenWebSeedAndPeer;
+    NSViewAnimation * fWebSeedTableAnimation;
     
     IBOutlet FileOutlineController * fFileController;
     
     IBOutlet NSPopUpButton * fRatioPopUp, * fUploadLimitPopUp, * fDownloadLimitPopUp;
     IBOutlet NSTextField * fUploadLimitField, * fDownloadLimitField, * fRatioLimitField, * fPeersConnectField,
-                        * fUploadLimitLabel, * fDownloadLimitLabel;
+                        * fUploadLimitLabel, * fDownloadLimitLabel, * fPeersConnectLabel;
     
     NSString * fInitialString;
 }
@@ -78,6 +85,13 @@
 - (void) setNextTab;
 - (void) setPreviousTab;
 
+- (void) addRemoveTracker: (id) sender;
+
+- (BOOL) shouldQuickLookFileView;
+- (NSArray *) quickLookURLs;
+- (BOOL) canQuickLook;
+- (NSRect) quickLookFrameWithURL: (NSURL*) url;
+
 - (void) setPiecesView: (id) sender;
 - (void) setPiecesViewForAvailable: (BOOL) available;
 
