@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: PrefsController.h 5633 2008-04-17 04:25:49Z livings124 $
+ * $Id: PrefsController.h 6361 2008-07-19 13:40:23Z livings124 $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -34,7 +34,7 @@
     NSUserDefaults * fDefaults;
     BOOL fHasLoaded;
     
-    IBOutlet NSView * fGeneralView, * fTransfersView, * fBandwidthView, * fPeersView, * fNetworkView;
+    IBOutlet NSView * fGeneralView, * fTransfersView, * fBandwidthView, * fPeersView, * fNetworkView, * fRemoteView;
     
     NSString * fInitialString;
     
@@ -55,7 +55,15 @@
     IBOutlet NSImageView * fPortStatusImage;
     IBOutlet NSProgressIndicator * fPortStatusProgress;
     NSTimer * fPortStatusTimer;
-    int fPublicPort, fNatStatus;
+    int fPeerPort, fNatStatus;
+    
+    IBOutlet NSTextField * fProxyAddressField, * fProxyPortField, * fProxyPasswordField;
+    IBOutlet NSPopUpButton * fProxyTypePopUp;
+    
+    IBOutlet NSTextField * fRPCPortField, * fRPCPasswordField;
+    IBOutlet NSTableView * fRPCAccessTable;
+    IBOutlet NSSegmentedControl * fRPCAddRemoveControl;
+    NSMutableArray * fRPCAccessArray;
 }
 
 - (id) initWithHandle: (tr_handle *) handle;
@@ -113,7 +121,30 @@
 
 - (void) setAutoSize: (id) sender;
 
+- (void) setProxyEnabled: (id) sender;
+- (void) setProxyAddress: (id) sender;
+- (void) setProxyPort: (id) sender;
+- (void) setProxyType: (id) sender;
+- (void) updateProxyType;
+- (void) setProxyAuthorize: (id) sender;
+- (void) setProxyUsername: (id) sender;
+- (void) setProxyPassword: (id) sender;
+- (void) updateProxyPassword;
+
+- (void) setRPCEnabled: (id) sender;
+- (void) linkWebUI: (id) sender;
+- (void) setRPCAuthorize: (id) sender;
+- (void) setRPCUsername: (id) sender;
+- (void) setRPCPassword: (id) sender;
+- (void) updateRPCPassword;
+- (void) setRPCPort: (id) sender;
+- (void) updateRPCAccessList;
+- (void) addRemoveRPCIP: (id) sender;
+
 - (void) helpForPeers: (id) sender;
 - (void) helpForNetwork: (id) sender;
+- (void) helpForRemote: (id) sender;
+
+- (void) rpcUpdatePrefs;
 
 @end
