@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: StatsWindowController.m 6189 2008-06-13 21:52:11Z livings124 $
+ * $Id: StatsWindowController.m 5929 2008-05-24 23:51:51Z livings124 $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -163,23 +163,23 @@ tr_handle * fLib;
 - (void) updateStats
 {
     tr_session_stats statsAll, statsSession;
-    tr_getCumulativeSessionStats(fLib, &statsAll);
-    tr_getSessionStats(fLib, &statsSession);
+    tr_sessionGetCumulativeStats(fLib, &statsAll);
+    tr_sessionGetStats(fLib, &statsSession);
     
     [fUploadedField setStringValue: [NSString stringForFileSize: statsSession.uploadedBytes]];
-    [fUploadedField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%u bytes", "stats -> bytes"),
+    [fUploadedField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%llu bytes", "stats -> bytes"),
                                     statsSession.uploadedBytes]];
     [fUploadedAllField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%@ total", "stats total"),
                                         [NSString stringForFileSize: statsAll.uploadedBytes]]];
-    [fUploadedAllField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%u bytes", "stats -> bytes"),
+    [fUploadedAllField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%llu bytes", "stats -> bytes"),
                                     statsAll.uploadedBytes]];
     
     [fDownloadedField setStringValue: [NSString stringForFileSize: statsSession.downloadedBytes]];
-    [fDownloadedField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%u bytes", "stats -> bytes"),
+    [fDownloadedField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%llu bytes", "stats -> bytes"),
                                     statsSession.downloadedBytes]];
     [fDownloadedAllField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%@ total", "stats total"),
                                         [NSString stringForFileSize: statsAll.downloadedBytes]]];
-    [fDownloadedAllField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%u bytes", "stats -> bytes"),
+    [fDownloadedAllField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%llu bytes", "stats -> bytes"),
                                         statsAll.downloadedBytes]];
     
     [fRatioField setStringValue: [NSString stringForRatio: statsSession.ratio]];
@@ -197,12 +197,12 @@ tr_handle * fLib;
         [fNumOpenedField setStringValue: NSLocalizedString(@"1 time", "stats window -> times opened")];
     else
         [fNumOpenedField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%llu times", "stats window -> times opened"),
-                                        statsAll.sessionCount]];
+                                            statsAll.sessionCount]];
 }
 
 - (void) performResetStats
 {
-    tr_clearSessionStats(fLib);
+    tr_sessionClearStats(fLib);
     [self updateStats];
 }
 
