@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: Torrent.h 4785 2008-01-21 19:35:00Z livings124 $
+ * $Id: Torrent.h 6325 2008-07-11 21:48:43Z livings124 $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -34,52 +34,54 @@ typedef enum
     TORRENT_FILE_DEFAULT
 } torrentFileState;
 
+#define STAT_TIME_NONE -1
+#define STAT_TIME_NOW -2
+
 @interface Torrent : NSObject
 {
-    tr_handle * fLib;
     tr_torrent * fHandle;
     const tr_info * fInfo;
     const tr_stat * fStat;
     
-    int fID;
+    BOOL fResumeOnWake;
     
-    BOOL         fResumeOnWake;
-    NSDate       * fDateAdded, * fDateCompleted, * fDateActivity;
+    BOOL fUseIncompleteFolder;
+    NSString * fDownloadFolder, * fIncompleteFolder;
     
-    BOOL        fUseIncompleteFolder;
-    NSString    * fDownloadFolder, * fIncompleteFolder;
-    
-    BOOL        fPublicTorrent;
-    NSString    * fPublicTorrentLocation;
+    BOOL fPublicTorrent;
+    NSString * fPublicTorrentLocation;
 	
     NSUserDefaults * fDefaults;
 
     NSImage * fIcon;
     
-    tr_file_stat * fileStat;
+    NSString * fNameString, * fHashString;
+    
+    tr_file_stat * fFileStat;
     NSArray * fFileList;
     
-    NSMenu * fFileMenu;
-    
-    float * fPreviousFinishedPieces;
-    NSDate * fFinishedPiecesDate;
+    NSIndexSet * fPreviousFinishedIndexes;
+    NSDate * fPreviousFinishedIndexesDate;
     
     float fRatioLimit;
     int fRatioSetting;
-    BOOL fFinishedSeeding, fWaitToStart, fError, fChecking, fStalled;
+    BOOL fFinishedSeeding, fWaitToStart, fStalled;
     
     int fOrderValue, fGroupValue;
     
+    BOOL fAddedTrackers;
+    
     NSDictionary * fQuickPauseDict;
 }
 
 - (id) initWithPath: (NSString *) path location: (NSString *) location deleteTorrentFile: (torrentFileState) torrentDelete
         lib: (tr_handle *) lib;
-- (id) initWithData: (NSData *) data location: (NSString *) location lib: (tr_handle *) lib;
+- (id) initWithTorrentStruct: (tr_torrent *) torrentStruct location: (NSString *) location lib: (tr_handle *) lib;
 - (id) initWithHistory: (NSDictionary *) history lib: (tr_handle *) lib;
 
 - (NSDictionary *) history;
 
+- (void) closeRemoveTorrentInterface;
 - (void) closeRemoveTorrent;
 
 - (void) changeIncompleteDownloadFolder: (NSString *) folder;
@@ -88,8 +90,8 @@ typedef enum
 
 - (void) getAvailability: (int8_t *) tab size: (int) size;
 - (void) getAmountFinished: (float *) tab size: (int) size;
-- (float *) getPreviousAmountFinished;
--(void) setPreviousAmountFinished: (float *) tab;
+- (NSIndexSet *) previousFinishedPieces;
+-(void) setPreviousFinishedPieces: (NSIndexSet *) indexes;
 
 - (void) update;
 
@@ -136,12 +138,24 @@ typedef enum
 - (NSImage *) icon;
 
 - (NSString *) name;
-- (BOOL) folder;
+- (BOOL) isFolder;
 - (uint64_t) size;
 - (uint64_t) sizeLeft;
-- (NSString *) trackerAddress;
+
 - (NSString *) trackerAddressAnnounce;
-- (NSArray *) allTrackers;
+- (NSDate *) lastAnnounceTime;
+- (int) nextAnnounceTime;
+- (NSString *) announceResponse;
+
+- (NSString *) trackerAddressScrape;
+- (NSDate *) lastScrapeTime;
+- (int) nextScrapeTime;
+- (NSString *) scrapeResponse;
+
+- (NSMutableArray *) allTrackers: (BOOL) separators;
+- (BOOL) updateAllTrackersForAdd: (NSMutableArray *) trackers;
+- (void) updateAllTrackersForRemove: (NSMutableArray *) trackers;
+- (BOOL) hasAddedTrackers;
 
 - (NSString *) comment;
 - (NSString *) creator;
@@ -161,16 +175,17 @@ typedef enum
 - (float) progress;
 - (float) progressDone;
 - (float) progressLeft;
+- (float) checkingProgress;
 
 - (int) eta;
 - (int) etaRatio;
-- (NSString *) etaString: (int) eta;
 
 - (float) notAvailableDesired;
 
 - (BOOL) isActive;
 - (BOOL) isSeeding;
 - (BOOL) isChecking;
+- (BOOL) isCheckingWaiting;
 - (BOOL) allDownloaded;
 - (BOOL) isComplete;
 - (BOOL) isError;
@@ -178,6 +193,9 @@ typedef enum
 
 - (NSArray *) peers;
 
+- (NSUInteger) webSeedCount;
+- (NSArray *) webSeeds;
+
 - (NSString *) progressString;
 - (NSString *) statusString;
 - (NSString *) shortStatusString;
@@ -204,6 +222,7 @@ typedef enum
 - (float) totalRate;
 - (uint64_t) haveVerified;
 - (uint64_t) haveTotal;
+- (uint64_t) totalSizeSelected;
 - (uint64_t) downloadedTotal;
 - (uint64_t) uploadedTotal;
 - (uint64_t) failedHash;
@@ -231,8 +250,6 @@ typedef enum
 - (BOOL) hasFilePriority: (int) priority forIndexes: (NSIndexSet *) indexSet;
 - (NSSet *) filePrioritiesForIndexes: (NSIndexSet *) indexSet;
 
-- (NSMenu *) fileMenu;
-
 - (NSDate *) dateAdded;
 - (NSDate *) dateCompleted;
 - (NSDate *) dateActivity;
@@ -241,10 +258,8 @@ typedef enum
 - (int) stalledMinutes;
 - (BOOL) isStalled;
 
-- (NSNumber *) stateSortKey;
+- (NSInteger) stateSortKey;
 
-- (int) torrentID;
-- (const tr_info *) torrentInfo;
-- (const tr_stat *) torrentStat;
+- (tr_torrent *) torrentStruct;
 
 @end
