@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: TorrentCell.h 5339 2008-03-23 04:36:30Z livings124 $
+ * $Id: TorrentCell.h 6252 2008-06-25 05:01:04Z livings124 $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -32,17 +32,9 @@
     NSMutableDictionary * fTitleAttributes, * fStatusAttributes;
     
     BOOL fTracking, fMouseDownControlButton, fMouseDownRevealButton, fMouseDownActionButton,
-            fHoverControl, fHoverReveal, fHoverAction, fMouseDownProgressField, fMouseDownMinimalStatusField;
+            fHoverControl, fHoverReveal, fHoverAction;
     
-    NSColor * fBarOverlayColor;
-    CTGradient * fWhiteGradient, * fGrayGradient, * fLightGrayGradient, * fBlueGradient, * fDarkBlueGradient, 
-                * fGreenGradient, * fLightGreenGradient, * fDarkGreenGradient, * fYellowGradient, * fRedGradient,
-                * fTransparentGradient;
-    
-    NSColor * fGrayColor, * fBlueColor, * fBlue1Color, * fBlue2Color, * fBlue3Color, * fBlue4Color, * fOrangeColor,
-            * fFieldBackColor;
-    
-    NSBitmapImageRep * fBitmap;
+    NSColor * fBarBorderColor, * fBluePieceColor;
 }
 
 - (NSRect) iconRectForBounds: (NSRect) bounds;
