@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: TorrentCell.m 5630 2008-04-16 12:32:07Z livings124 $
+ * $Id: TorrentCell.m 6252 2008-06-25 05:01:04Z livings124 $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -54,7 +54,8 @@
 #define PADDING_BETWEEN_BAR_AND_STATUS 2.0
 
 #define PIECES_TOTAL_PERCENT 0.6
-#define MAX_PIECES 324
+
+#define MAX_PIECES (18*18)
 
 @interface TorrentCell (Private)
 
@@ -97,60 +98,12 @@
                                 paragraphStyle, NSParagraphStyleAttributeName, nil];
         [paragraphStyle release];
         
-        //store box colors
-        fGrayColor = [[NSColor colorWithCalibratedRed: 0.9 green: 0.9 blue: 0.9 alpha: 1.0] retain];
-        fBlue1Color = [[NSColor colorWithCalibratedRed: 0.8 green: 1.0 blue: 1.0 alpha: 1.0] retain];
-        fBlue2Color = [[NSColor colorWithCalibratedRed: 0.6 green: 1.0 blue: 1.0 alpha: 1.0] retain];
-        fBlue3Color = [[NSColor colorWithCalibratedRed: 0.6 green: 0.8 blue: 1.0 alpha: 1.0] retain];
-        fBlue4Color = [[NSColor colorWithCalibratedRed: 0.4 green: 0.6 blue: 1.0 alpha: 1.0] retain];
-        fBlueColor = [[NSColor colorWithCalibratedRed: 0.0 green: 0.4 blue: 0.8 alpha: 1.0] retain];
-        fOrangeColor = [[NSColor orangeColor] retain];
-        
-        fBarOverlayColor = [[NSColor colorWithDeviceWhite: 0.0 alpha: 0.2] retain];
-        
-        fFieldBackColor = [[NSColor colorWithDeviceWhite: 0.65 alpha: 0.7] retain];
+        fBluePieceColor = [[NSColor colorWithCalibratedRed: 0.0 green: 0.4 blue: 0.8 alpha: 1.0] retain];
+        fBarBorderColor = [[NSColor colorWithDeviceWhite: 0.0 alpha: 0.2] retain];
     }
 	return self;
 }
 
-- (id) copyWithZone: (NSZone *) zone
-{
-    TorrentCell * copy = [super copyWithZone: zone];
-    
-    copy->fBitmap = nil;
-    
-    copy->fGrayGradient = [fGrayGradient retain];
-    copy->fLightGrayGradient = [fLightGrayGradient retain];
-    copy->fBlueGradient = [fBlueGradient retain];
-    copy->fDarkBlueGradient = [fDarkBlueGradient retain];
-    copy->fGreenGradient = [fGreenGradient retain];
-    copy->fLightGreenGradient = [fLightGreenGradient retain];
-    copy->fDarkGreenGradient = [fDarkGreenGradient retain];
-    copy->fYellowGradient = [fYellowGradient retain];
-    copy->fRedGradient = [fRedGradient retain];
-    copy->fTransparentGradient = [fTransparentGradient retain];
-    
-    return copy;
-}
-
-- (void) dealloc
-{
-    [fBitmap release];
-    
-    [fGrayGradient release];
-    [fLightGrayGradient release];
-    [fBlueGradient release];
-    [fDarkBlueGradient release];
-    [fGreenGradient release];
-    [fLightGreenGradient release];
-    [fDarkGreenGradient release];
-    [fYellowGradient release];
-    [fRedGradient release];
-    [fTransparentGradient release];
-    
-    [super dealloc];
-}
-
 - (NSRect) iconRectForBounds: (NSRect) bounds
 {
     float imageSize = [fDefaults boolForKey: @"SmallView"] ? IMAGE_SIZE_MIN : IMAGE_SIZE_REG;
@@ -171,12 +124,18 @@
 
 - (NSRect) minimalStatusRectForBounds: (NSRect) bounds
 {
+    if (![fDefaults boolForKey: @"SmallView"])
+        return NSZeroRect;
+    
     return [self rectForMinimalStatusWithString: [self attributedStatusString: [self minimalStatusString] withColor: nil]
             inBounds: bounds];
 }
 
 - (NSRect) progressRectForBounds: (NSRect) bounds
 {
+    if ([fDefaults boolForKey: @"SmallView"])
+        return NSZeroRect;
+    
     return [self rectForProgressWithString: [self attributedStatusString: [[self representedObject] progressString] withColor: nil]
             inBounds: bounds];
 }
@@ -202,6 +161,9 @@
 
 - (NSRect) statusRectForBounds: (NSRect) bounds
 {
+    if ([fDefaults boolForKey: @"SmallView"])
+        return NSZeroRect;
+    
     return [self rectForStatusWithString: [self attributedStatusString: [self statusString] withColor: nil] inBounds: bounds];
 }
 
@@ -256,9 +218,7 @@
     NSPoint point = [controlView convertPoint: [event locationInWindow] fromView: nil];
     
     if (NSMouseInRect(point, [self controlButtonRectForBounds: cellFrame], [controlView isFlipped])
-        || NSMouseInRect(point, [self revealButtonRectForBounds: cellFrame], [controlView isFlipped])
-        || (NSMouseInRect(point, [self progressRectForBounds: cellFrame], [controlView isFlipped]) && [[self representedObject] folder])
-        || NSMouseInRect(point, [self minimalStatusRectForBounds: cellFrame], [controlView isFlipped]))
+        || NSMouseInRect(point, [self revealButtonRectForBounds: cellFrame], [controlView isFlipped]))
         return NSCellHitContentArea | NSCellHitTrackableArea;
     
     return NSCellHitContentArea;
@@ -283,12 +243,6 @@
     NSRect revealRect = [self revealButtonRectForBounds: cellFrame];
     BOOL checkReveal = NSMouseInRect(point, revealRect, [controlView isFlipped]);
     
-    NSRect progressRect = [self progressRectForBounds: cellFrame];
-    BOOL checkProgress = NSMouseInRect(point, progressRect, [controlView isFlipped]) && [[self representedObject] folder];
-    
-    NSRect minimalStatusRect = [self minimalStatusRectForBounds: cellFrame];
-    BOOL checkMinStatus = NSMouseInRect(point, minimalStatusRect, [controlView isFlipped]);
-    
     [(TorrentTableView *)controlView removeButtonTrackingAreas];
     
     while ([event type] != NSLeftMouseUp)
@@ -313,24 +267,6 @@
                 [controlView setNeedsDisplayInRect: cellFrame];
             }
         }
-        else if (checkProgress)
-        {
-            BOOL inProgressField = NSMouseInRect(point, progressRect, [controlView isFlipped]);
-            if (fMouseDownProgressField != inProgressField)
-            {
-                fMouseDownProgressField = inProgressField;
-                [controlView setNeedsDisplayInRect: cellFrame];
-            }
-        }
-        else if (checkMinStatus)
-        {
-            BOOL inMinStatusField = NSMouseInRect(point, minimalStatusRect, [controlView isFlipped]);
-            if (fMouseDownMinimalStatusField != inMinStatusField)
-            {
-                fMouseDownMinimalStatusField = inMinStatusField;
-                [controlView setNeedsDisplayInRect: cellFrame];
-            }
-        }
         else;
         
         //send events to where necessary
@@ -355,20 +291,6 @@
         
         [[self representedObject] revealData];
     }
-    else if (fMouseDownProgressField)
-    {
-        fMouseDownProgressField = NO;
-        
-        [fDefaults setBool: ![fDefaults boolForKey: @"DisplayStatusProgressSelected"] forKey: @"DisplayStatusProgressSelected"];
-        [(TorrentTableView *)controlView reloadData];
-    }
-    else if (fMouseDownMinimalStatusField)
-    {
-        fMouseDownMinimalStatusField = NO;
-        
-        [fDefaults setBool: ![fDefaults boolForKey: @"DisplaySmallStatusRegular"] forKey: @"DisplaySmallStatusRegular"];
-        [(TorrentTableView *)controlView reloadData];
-    }
     else;
     
     if ([NSApp isOnLeopardOrBetter])
@@ -464,15 +386,28 @@
     int groupValue = [torrent groupValue];
     if (groupValue != -1)
     {
-        NSRect groupRect = NSInsetRect(iconRect, -2.0, -3.0);
+        NSRect groupRect = NSInsetRect(iconRect, -1.0, -2.0);
         if (!minimal)
         {
             groupRect.size.height--;
             groupRect.origin.y--;
         }
+        float radius = minimal ? 3.0 : 6.0;
         
-        [[[GroupsController groups] gradientForIndex: groupValue] fillBezierPath:
-            [NSBezierPath bezierPathWithRoundedRect: groupRect radius: 6.0] angle: 90.0];
+        NSColor * groupColor = [[GroupsController groups] colorForIndex: groupValue],
+                * darkGroupColor = [groupColor blendedColorWithFraction: 0.2 ofColor: [NSColor whiteColor]];
+        
+        //border
+        NSBezierPath * bp = [NSBezierPath bezierPathWithRoundedRect: groupRect radius: radius];
+        [darkGroupColor set];
+        [bp setLineWidth: 2.0];
+        [bp stroke];
+        
+        //inside
+        bp = [NSBezierPath bezierPathWithRoundedRect: groupRect radius: radius];
+        CTGradient * gradient = [CTGradient gradientWithBeginningColor: [groupColor blendedColorWithFraction: 0.7
+                                    ofColor: [NSColor whiteColor]] endingColor: darkGroupColor];
+        [gradient fillBezierPath: bp angle: 90.0];
     }
     
     //error image
@@ -518,12 +453,6 @@
         NSAttributedString * minimalString = [self attributedStatusString: [self minimalStatusString] withColor: statusColor];
         minimalStatusRect = [self rectForMinimalStatusWithString: minimalString inBounds: cellFrame];
         
-        if (fMouseDownMinimalStatusField)
-        {
-            [fFieldBackColor set];
-            [[NSBezierPath bezierPathWithRoundedRect: NSInsetRect(minimalStatusRect, -2.0, 0.0) radius: 5.0] fill];
-        }
-        
         [minimalString drawInRect: minimalStatusRect];
     }
     
@@ -538,12 +467,6 @@
         NSAttributedString * progressString = [self attributedStatusString: [torrent progressString] withColor: statusColor];
         NSRect progressRect = [self rectForProgressWithString: progressString inBounds: cellFrame];
         
-        if (fMouseDownProgressField)
-        {
-            [fFieldBackColor set];
-            [[NSBezierPath bezierPathWithRoundedRect: NSInsetRect(progressRect, -2.0, 0.0) radius: 5.0] fill];
-        }
-        
         [progressString drawInRect: progressRect];
     }
     
@@ -577,31 +500,31 @@
         fraction: 1.0];
     
     //reveal button
-    NSString * revealImageSuffix;
+    NSString * revealImageString;
     if (fMouseDownRevealButton)
-        revealImageSuffix = @"On.png";
+        revealImageString = @"RevealOn.png";
     else if (!fTracking && fHoverReveal)
-        revealImageSuffix = @"Hover.png";
+        revealImageString = @"RevealHover.png";
     else
-        revealImageSuffix = @"Off.png";
+        revealImageString = @"RevealOff.png";
     
-    NSImage * revealImage = [NSImage imageNamed: [@"Reveal" stringByAppendingString: revealImageSuffix]];
+    NSImage * revealImage = [NSImage imageNamed: revealImageString];
     [revealImage setFlipped: YES];
     [revealImage drawInRect: [self revealButtonRectForBounds: cellFrame] fromRect: NSZeroRect operation: NSCompositeSourceOver
         fraction: 1.0];
     
     //action button
-    NSString * actionImageSuffix;
+    NSString * actionImageString;
     if (fMouseDownActionButton)
-        actionImageSuffix = @"On.png";
+        actionImageString = @"ActionOn.png";
     else if (!fTracking && fHoverAction)
-        actionImageSuffix = @"Hover.png";
+        actionImageString = @"ActionHover.png";
     else
-        actionImageSuffix = nil;
+        actionImageString = nil;
     
-    if (actionImageSuffix)
+    if (actionImageString)
     {
-        NSImage * actionImage = [NSImage imageNamed: [@"Action" stringByAppendingString: actionImageSuffix]];
+        NSImage * actionImage = [NSImage imageNamed: actionImageString];
         [actionImage setFlipped: YES];
         [actionImage drawInRect: [self actionButtonRectForBounds: cellFrame] fromRect: NSZeroRect operation: NSCompositeSourceOver
             fraction: 1.0];
@@ -634,14 +557,12 @@
     }
     else
     {
-        [fBitmap release];
-        fBitmap = nil;
-        [[self representedObject] setPreviousAmountFinished: NULL];
+        [[self representedObject] setPreviousFinishedPieces: nil];
         
         [self drawRegularBar: barRect];
     }
     
-    [fBarOverlayColor set];
+    [fBarBorderColor set];
     [NSBezierPath strokeRect: NSInsetRect(barRect, 0.5, 0.5)];
 }
 
@@ -667,9 +588,7 @@
             noIncludeRect.origin.x += barRect.size.width - rightNoIncludeWidth;
             noIncludeRect.size.width = rightNoIncludeWidth;
             
-            if (!fLightGrayGradient)
-                fLightGrayGradient = [[CTGradient progressLightGrayGradient] retain];
-            [fLightGrayGradient fillRect: noIncludeRect angle: -90];
+            [[CTGradient progressLightGrayGradient] fillRect: noIncludeRect angle: 90];
         }
         
         if (rightWidth > 0)
@@ -686,9 +605,7 @@
                     notAvailableRect.origin.x += leftWidth + rightWidth;
                     notAvailableRect.size.width = notAvailableWidth;
                     
-                    if (!fRedGradient)
-                        fRedGradient = [[CTGradient progressRedGradient] retain];
-                    [fRedGradient fillRect: notAvailableRect angle: -90];
+                    [[CTGradient progressRedGradient] fillRect: notAvailableRect angle: 90];
                 }
             }
             
@@ -698,9 +615,7 @@
                 includeRect.origin.x += leftWidth;
                 includeRect.size.width = rightWidth;
                 
-                if (!fWhiteGradient)
-                    fWhiteGradient = [[CTGradient progressWhiteGradient] retain];
-                [fWhiteGradient fillRect: includeRect angle: -90];
+                [[CTGradient progressWhiteGradient] fillRect: includeRect angle: 90];
             }
         }
     }
@@ -713,11 +628,7 @@
         if ([torrent isActive])
         {
             if ([torrent isChecking])
-            {
-                if (!fYellowGradient)
-                    fYellowGradient = [[CTGradient progressYellowGradient] retain];
-                [fYellowGradient fillRect: completeRect angle: -90];
-            }
+                [[CTGradient progressYellowGradient] fillRect: completeRect angle: 90];
             else if ([torrent isSeeding])
             {
                 int ratioLeftWidth = leftWidth * (1.0 - [torrent progressStopRatio]);
@@ -729,111 +640,79 @@
                     ratioLeftRect.origin.x += leftWidth;
                     ratioLeftRect.size.width = ratioLeftWidth;
                     
-                    if (!fLightGreenGradient)
-                        fLightGreenGradient = [[CTGradient progressLightGreenGradient] retain];
-                    [fLightGreenGradient fillRect: ratioLeftRect angle: -90];
+                    [[CTGradient progressLightGreenGradient] fillRect: ratioLeftRect angle: 90];
                 }
                 
                 if (leftWidth > 0)
                 {
                     completeRect.size.width = leftWidth;
                     
-                    if (!fGreenGradient)
-                        fGreenGradient = [[CTGradient progressGreenGradient] retain];
-                    [fGreenGradient fillRect: completeRect angle: -90];
+                    [[CTGradient progressGreenGradient] fillRect: completeRect angle: 90];
                 }
             }
             else
-            {
-                if (!fBlueGradient)
-                    fBlueGradient = [[CTGradient progressBlueGradient] retain];
-                [fBlueGradient fillRect: completeRect angle: -90];
-            }
+                [[CTGradient progressBlueGradient] fillRect: completeRect angle: 90];
         }
         else
         {
             if ([torrent waitingToStart])
             {
                 if ([torrent progressLeft] <= 0.0)
-                {
-                    if (!fDarkGreenGradient)
-                        fDarkGreenGradient = [[CTGradient progressDarkGreenGradient] retain];
-                    [fDarkGreenGradient fillRect: completeRect angle: -90];
-                }
+                    [[CTGradient progressDarkGreenGradient] fillRect: completeRect angle: 90];
                 else
-                {
-                    if (!fDarkBlueGradient)
-                        fDarkBlueGradient = [[CTGradient progressDarkBlueGradient] retain];
-                    [fDarkBlueGradient fillRect: completeRect angle: -90];
-                }
+                    [[CTGradient progressDarkBlueGradient] fillRect: completeRect angle: 90];
             }
             else
-            {
-                if (!fGrayGradient)
-                    fGrayGradient = [[CTGradient progressGrayGradient] retain];
-                [fGrayGradient fillRect: completeRect angle: -90];
-            }
+                [[CTGradient progressGrayGradient] fillRect: completeRect angle: 90];
         }
     }
 }
 
 - (void) drawPiecesBar: (NSRect) barRect
 {
-    if (!fBitmap)
-        fBitmap = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes: nil
-            pixelsWide: MAX_PIECES pixelsHigh: 1 bitsPerSample: 8 samplesPerPixel: 4 hasAlpha: YES
-            isPlanar: NO colorSpaceName: NSCalibratedRGBColorSpace bytesPerRow: 0 bitsPerPixel: 0];
-    
     Torrent * torrent = [self representedObject];
     
     int pieceCount = MIN([torrent pieceCount], MAX_PIECES);
-    float * piecePercent = malloc(pieceCount * sizeof(float)),
-        * previousPiecePercent = [torrent getPreviousAmountFinished];
-    [torrent getAmountFinished: piecePercent size: pieceCount];
+    float * piecesPercent = malloc(pieceCount * sizeof(float));
+    [torrent getAmountFinished: piecesPercent size: pieceCount];
     
-    int i, index;
-    float increment = (float)pieceCount / MAX_PIECES;
-    NSColor * pieceColor;
-    for (i = 0; i < MAX_PIECES; i++)
+    NSBitmapImageRep * bitmap = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes: nil
+                                    pixelsWide: pieceCount pixelsHigh: 1 bitsPerSample: 8 samplesPerPixel: 4 hasAlpha: YES
+                                    isPlanar: NO colorSpaceName: NSCalibratedRGBColorSpace bytesPerRow: 0 bitsPerPixel: 0];
+    
+    NSIndexSet * previousFinishedIndexes = [torrent previousFinishedPieces];
+    NSMutableIndexSet * finishedIndexes = [NSMutableIndexSet indexSet];
+    
+    int i;
+    for (i = 0; i < pieceCount; i++)
     {
-        index = i * increment;
-        if (piecePercent[index] >= 1.0)
+        NSColor * pieceColor;
+        if (piecesPercent[i] == 1.0)
         {
-            if (previousPiecePercent != NULL && previousPiecePercent[index] < 1.0)
-                pieceColor = fOrangeColor;
+            if (previousFinishedIndexes && ![previousFinishedIndexes containsIndex: i])
+                pieceColor = [NSColor orangeColor];
             else
-                pieceColor = fBlueColor;
+                pieceColor = fBluePieceColor;
+            [finishedIndexes addIndex: i];
         }
-        else if (piecePercent[index] <= 0.0)
-            pieceColor = fGrayColor;
-        else if (piecePercent[index] <= 0.25)
-            pieceColor = fBlue1Color;
-        else if (piecePercent[index] <= 0.5)
-            pieceColor = fBlue2Color;
-        else if (piecePercent[index] <= 0.75)
-            pieceColor = fBlue3Color;
         else
-            pieceColor = fBlue4Color;
+            pieceColor = [[NSColor whiteColor] blendedColorWithFraction: piecesPercent[i] ofColor: fBluePieceColor];
         
-        if (![pieceColor isEqual: [fBitmap colorAtX: i y: 0]])
-            [fBitmap setColor: pieceColor atX: i y: 0];
+        //it's faster to just set color instead of checking previous color
+        [bitmap setColor: pieceColor atX: i y: 0];
     }
     
-    [torrent setPreviousAmountFinished: piecePercent];
+    free(piecesPercent);
+    
+    [torrent setPreviousFinishedPieces: [finishedIndexes count] > 0 ? finishedIndexes : nil]; //don't bother saving if none are complete
     
     //actually draw image
-    [fBitmap drawInRect: barRect];
-    
-    if (!fTransparentGradient)
-        fTransparentGradient = [[CTGradient progressTransparentGradient] retain];
-    [fTransparentGradient fillRect: barRect angle: -90];
+    [bitmap drawInRect: barRect];
+    [bitmap release];
 }
 
 - (NSRect) rectForMinimalStatusWithString: (NSAttributedString *) string inBounds: (NSRect) bounds
 {
-    if (![fDefaults boolForKey: @"SmallView"])
-        return NSZeroRect;
-    
     NSRect result = bounds;
     result.size = [string size];
     
@@ -861,9 +740,6 @@
 
 - (NSRect) rectForProgressWithString: (NSAttributedString *) string inBounds: (NSRect) bounds
 {
-    if ([fDefaults boolForKey: @"SmallView"])
-        return NSZeroRect;
-    
     NSRect result = bounds;
     result.origin.y += PADDING_ABOVE_TITLE + HEIGHT_TITLE + PADDING_BETWEEN_TITLE_AND_PROGRESS;
     result.origin.x += PADDING_HORIZONTAL + IMAGE_SIZE_REG + PADDING_BETWEEN_IMAGE_AND_TITLE;
@@ -876,9 +752,6 @@
 
 - (NSRect) rectForStatusWithString: (NSAttributedString *) string inBounds: (NSRect) bounds
 {
-    if ([fDefaults boolForKey: @"SmallView"])
-        return NSZeroRect;
-    
     NSRect result = bounds;
     result.origin.y += PADDING_ABOVE_TITLE + HEIGHT_TITLE + PADDING_BETWEEN_TITLE_AND_PROGRESS + HEIGHT_STATUS
                         + PADDING_BETWEEN_PROGRESS_AND_BAR + BAR_HEIGHT + PADDING_BETWEEN_BAR_AND_STATUS;
