@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: TorrentTableView.h 5734 2008-05-01 02:20:53Z livings124 $
+ * $Id: TorrentTableView.h 6134 2008-06-11 03:29:06Z livings124 $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -49,7 +49,7 @@
     Torrent * fMenuTorrent;
     
     float fPiecesBarPercent;
-    NSTimer * fPiecesBarTimer;
+    NSAnimation * fPiecesBarAnimation;
 }
 
 - (BOOL) isGroupCollapsed: (int) value;
@@ -66,6 +66,8 @@
 - (NSArray *) selectedValues;
 - (NSArray *) selectedTorrents;
 
+- (NSRect) iconRectForRow: (int) row;
+
 - (void) paste: (id) sender;
 
 - (void) toggleControlForTorrent: (Torrent *) torrent;
